// Part of ImGui Bundle - MIT License - Copyright (c) 2022-2024 Pascal Thomet - https://github.com/pthom/imgui_bundle
#include <pybind11/pybind11.h>

#include "imspinner/imspinner.h"  // Change this include to the library you are binding

namespace py = pybind11;

// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  AUTOGENERATED CODE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// <litgen_glue_code>  // Autogenerated code below! Do not edit!

// </litgen_glue_code> // Autogenerated code end
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  AUTOGENERATED CODE END !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



void py_init_module_imspinner(py::module& m)
{
    using namespace ImSpinner;
    m.attr("IM_PI") = IM_PI;
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  AUTOGENERATED CODE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    // <litgen_pydef> // Autogenerated code below! Do not edit!
    ////////////////////    <generated_from:imspinner.h>    ////////////////////
    // #ifndef _IMSPINNER_H_
    //


    m.attr("white") = (const ImColor) ImColor(1.f, 1.f, 1.f, 1.f);
    m.attr("half_white") = (const ImColor) ImColor(1.f, 1.f, 1.f, 0.5f);
    m.attr("red") = (const ImColor) ImColor(1.f, 0.f, 0.f, 1.f);


    py::enum_<ImSpinner::SpinnerTypeT>(m, "SpinnerTypeT", py::arithmetic(), "")
        .value("e_st_rainbow", ImSpinner::e_st_rainbow, "")
        .value("e_st_angle", ImSpinner::e_st_angle, "")
        .value("e_st_dots", ImSpinner::e_st_dots, "")
        .value("e_st_ang", ImSpinner::e_st_ang, "")
        .value("e_st_vdots", ImSpinner::e_st_vdots, "")
        .value("e_st_bounce_ball", ImSpinner::e_st_bounce_ball, "")
        .value("e_st_eclipse", ImSpinner::e_st_eclipse, "")
        .value("e_st_ingyang", ImSpinner::e_st_ingyang, "")
        .value("e_st_barchartsine", ImSpinner::e_st_barchartsine, "")
        .value("e_st_count", ImSpinner::e_st_count, "");
    m.attr("PI_DIV_4") = (float) IM_PI / 4.f;
    m.attr("PI_DIV_2") = (float) IM_PI / 2.f;
    m.attr("PI_2") = (float) IM_PI * 2.f;

    m.def("color_alpha",
        ImSpinner::color_alpha, py::arg("c"), py::arg("alpha"));

    m.def("damped_spring",
        ImSpinner::damped_spring, py::arg("mass"), py::arg("stiffness"), py::arg("damping"), py::arg("time"), py::arg("a") = ImSpinner::PI_DIV_2, py::arg("b") = ImSpinner::PI_DIV_2);

    m.def("damped_gravity",
        ImSpinner::damped_gravity, py::arg("limtime"));

    m.def("damped_trifolium",
        ImSpinner::damped_trifolium, py::arg("limtime"), py::arg("a") = 0.f, py::arg("b") = 1.f);

    m.def("damped_infinity",
        ImSpinner::damped_infinity, py::arg("a"), py::arg("t"));

    m.def("spinner_rainbow",
        ImSpinner::SpinnerRainbow,
        py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("ang_min") = 0.f, py::arg("ang_max") = ImSpinner::PI_2, py::arg("arcs") = 1, py::arg("mode") = 0,
        "\n        const char *label: A string label for the spinner, used to identify it in ImGui.\n        float radius: The radius of the spinner.\n        float thickness: The thickness of the spinner's border.\n        const ImColor &color: The color of the spinner.\n        float speed: The speed of the spinning animation.\n        float ang_min: Minimum angle of spinning.\n        float ang_max: Maximum angle of spinning.\n        int arcs: Number of arcs of the spinner.\n");


    m.def("spinner_rainbow_mix",
        SpinnerRainbowMix, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("ang_min") = 0.f, py::arg("ang_max") = PI_2, py::arg("arcs") = 1, py::arg("mode") = 0);

    m.def("spinner_rotating_heart",
        SpinnerRotatingHeart,
        py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("ang_min") = 0.f,
        "This function draws a rotating heart spinner.");

    m.def("spinner_ang",
        SpinnerAng,
        py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("bg") = white, py::arg("speed") = 2.8f, py::arg("angle") = IM_PI, py::arg("mode") = 0,
        "SpinnerAng is a function that draws a spinner widget with a given angle.");

    m.def("spinner_ang_mix",
        SpinnerAngMix, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("angle") = IM_PI, py::arg("arcs") = 4, py::arg("mode") = 0);

    m.def("spinner_loading_ring",
        SpinnerLoadingRing, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("bg") = half_white, py::arg("speed") = 2.8f, py::arg("segments") = 5);

    m.def("spinner_clock",
        SpinnerClock, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("bg") = half_white, py::arg("speed") = 2.8f);

    m.def("spinner_pulsar",
        SpinnerPulsar, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("bg") = half_white, py::arg("speed") = 2.8f, py::arg("sequence") = true, py::arg("angle") = 0.f, py::arg("mode") = 0);

    m.def("spinner_double_fade_pulsar",
        SpinnerDoubleFadePulsar, py::arg("label"), py::arg("radius"), py::arg("param_2"), py::arg("bg") = half_white, py::arg("speed") = 2.8f);

    m.def("spinner_twin_pulsar",
        SpinnerTwinPulsar, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("rings") = 2);

    m.def("spinner_fade_pulsar",
        SpinnerFadePulsar, py::arg("label"), py::arg("radius"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("rings") = 2);

    m.def("spinner_circular_lines",
        SpinnerCircularLines, py::arg("label"), py::arg("radius"), py::arg("color") = white, py::arg("speed") = 1.8f, py::arg("lines") = 8);

    m.def("spinner_dots",
        SpinnerDots, py::arg("label"), py::arg("nextdot"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("dots") = 12, py::arg("minth") = -1.f, py::arg("mode") = 0);

    m.def("spinner_v_dots",
        SpinnerVDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("bgcolor") = white, py::arg("speed") = 2.8f, py::arg("dots") = 12, py::arg("mdots") = 6);

    m.def("spinner_bounce_dots",
        SpinnerBounceDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("dots") = 3, py::arg("mode") = 0);

    m.def("spinner_zip_dots",
        SpinnerZipDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("dots") = 5);

    m.def("spinner_dots_to_points",
        SpinnerDotsToPoints, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("offset_k"), py::arg("color") = white, py::arg("speed") = 1.8f, py::arg("dots") = 5);

    m.def("spinner_dots_to_bar",
        SpinnerDotsToBar, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("offset_k"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("dots") = 5);

    m.def("spinner_wave_dots",
        SpinnerWaveDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("lt") = 8);

    m.def("spinner_fade_dots",
        SpinnerFadeDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("lt") = 8, py::arg("mode") = 0);

    m.def("spinner_three_dots",
        SpinnerThreeDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("lt") = 8);

    m.def("spinner_five_dots",
        SpinnerFiveDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = 0xffffffff, py::arg("speed") = 2.8f, py::arg("lt") = 8);

    m.def("spinner4_caleidospcope",
        Spinner4Caleidospcope, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = 0xffffffff, py::arg("speed") = 2.8f, py::arg("lt") = 8);

    m.def("spinner_multi_fade_dots",
        SpinnerMultiFadeDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("lt") = 8);

    m.def("spinner_scale_dots",
        SpinnerScaleDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("lt") = 8);

    m.def("spinner_square_spins",
        SpinnerSquareSpins, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f);

    m.def("spinner_moving_dots",
        SpinnerMovingDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("dots") = 3);

    m.def("spinner_rotate_dots",
        SpinnerRotateDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("dots") = 2, py::arg("mode") = 0);

    m.def("spinner_orion_dots",
        SpinnerOrionDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4);

    m.def("spinner_galaxy_dots",
        SpinnerGalaxyDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4);

    m.def("spinner_twin_ang",
        SpinnerTwinAng, py::arg("label"), py::arg("radius1"), py::arg("radius2"), py::arg("thickness"), py::arg("color1") = white, py::arg("color2") = red, py::arg("speed") = 2.8f, py::arg("angle") = IM_PI, py::arg("mode") = 0);

    m.def("spinner_filling",
        SpinnerFilling, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color1") = white, py::arg("color2") = red, py::arg("speed") = 2.8f);

    m.def("spinner_filling_mem",
        SpinnerFillingMem, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("colorbg"), py::arg("speed"));

    m.def("spinner_topup",
        SpinnerTopup, py::arg("label"), py::arg("radius1"), py::arg("radius2"), py::arg("color") = red, py::arg("fg") = white, py::arg("bg") = white, py::arg("speed") = 2.8f);

    m.def("spinner_twin_ang180",
        SpinnerTwinAng180, py::arg("label"), py::arg("radius1"), py::arg("radius2"), py::arg("thickness"), py::arg("color1") = white, py::arg("color2") = red, py::arg("speed") = 2.8f, py::arg("angle") = PI_DIV_4, py::arg("mode") = 0);

    m.def("spinner_twin_ang360",
        SpinnerTwinAng360, py::arg("label"), py::arg("radius1"), py::arg("radius2"), py::arg("thickness"), py::arg("color1") = white, py::arg("color2") = red, py::arg("speed1") = 2.8f, py::arg("speed2") = 2.5f, py::arg("mode") = 0);

    m.def("spinner_inc_dots",
        SpinnerIncDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("dots") = 6);

    m.def("spinner_inc_full_dots",
        SpinnerIncFullDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("dots") = 4);

    m.def("spinner_fade_bars",
        SpinnerFadeBars, py::arg("label"), py::arg("w"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("bars") = 3, py::arg("scale") = false);

    m.def("spinner_fade_tris",
        SpinnerFadeTris, py::arg("label"), py::arg("radius"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("dim") = 2, py::arg("scale") = false);

    m.def("spinner_bars_rotate_fade",
        SpinnerBarsRotateFade, py::arg("label"), py::arg("rmin"), py::arg("rmax"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("bars") = 6);

    m.def("spinner_bars_scale_middle",
        SpinnerBarsScaleMiddle, py::arg("label"), py::arg("w"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("bars") = 3);

    m.def("spinner_ang_twin",
        SpinnerAngTwin, py::arg("label"), py::arg("radius1"), py::arg("radius2"), py::arg("thickness"), py::arg("color") = white, py::arg("bg") = half_white, py::arg("speed") = 2.8f, py::arg("angle") = IM_PI, py::arg("arcs") = 1, py::arg("mode") = 0);

    m.def("spinner_arc_rotation",
        SpinnerArcRotation, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4, py::arg("mode") = 0);

    m.def("spinner_arc_fade",
        SpinnerArcFade, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4);

    m.def("spinner_simple_arc_fade",
        SpinnerSimpleArcFade, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f);

    m.def("spinner_square_stroke_fade",
        SpinnerSquareStrokeFade, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f);

    m.def("spinner_ascii_symbol_points",
        SpinnerAsciiSymbolPoints, py::arg("label"), py::arg("text"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f);

    m.def("spinner_text_fading",
        SpinnerTextFading, py::arg("label"), py::arg("text"), py::arg("radius"), py::arg("fsize"), py::arg("color") = white, py::arg("speed") = 2.8f);

    m.def("spinner_seven_segments",
        SpinnerSevenSegments, py::arg("label"), py::arg("text"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f);

    m.def("spinner_square_stroke_fill",
        SpinnerSquareStrokeFill, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f);

    m.def("spinner_square_stroke_loading",
        SpinnerSquareStrokeLoading, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f);

    m.def("spinner_square_loading",
        SpinnerSquareLoading, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f);

    m.def("spinner_filled_arc_fade",
        SpinnerFilledArcFade, py::arg("label"), py::arg("radius"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4, py::arg("mode") = 0);

    m.def("spinner_points_arc_bounce",
        SpinnerPointsArcBounce, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("points") = 4, py::arg("circles") = 2, py::arg("rspeed") = 0.f);

    m.def("spinner_filled_arc_color",
        SpinnerFilledArcColor, py::arg("label"), py::arg("radius"), py::arg("color") = red, py::arg("bg") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4);

    m.def("spinner_filled_arc_ring",
        SpinnerFilledArcRing, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = red, py::arg("bg") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4);

    m.def("spinner_arc_wedges",
        SpinnerArcWedges, py::arg("label"), py::arg("radius"), py::arg("color") = red, py::arg("speed") = 2.8f, py::arg("arcs") = 4);

    m.def("spinner_twin_ball",
        SpinnerTwinBall, py::arg("label"), py::arg("radius1"), py::arg("radius2"), py::arg("thickness"), py::arg("b_thickness"), py::arg("ball") = white, py::arg("bg") = half_white, py::arg("speed") = 2.8f, py::arg("balls") = 2);

    m.def("spinner_solar_balls",
        SpinnerSolarBalls, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("ball") = white, py::arg("bg") = half_white, py::arg("speed") = 2.8f, py::arg("balls") = 4);

    m.def("spinner_solar_scale_balls",
        SpinnerSolarScaleBalls, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("ball") = white, py::arg("speed") = 2.8f, py::arg("balls") = 4);

    m.def("spinner_solar_arcs",
        SpinnerSolarArcs, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("ball") = white, py::arg("bg") = half_white, py::arg("speed") = 2.8f, py::arg("balls") = 4);

    m.def("spinner_moving_arcs",
        SpinnerMovingArcs, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4);

    m.def("spinner_rainbow_circle",
        SpinnerRainbowCircle, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4, py::arg("mode") = 1);

    m.def("spinner_bounce_ball",
        SpinnerBounceBall, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("dots") = 1, py::arg("shadow") = false);

    m.def("spinner_pulsar_ball",
        SpinnerPulsarBall, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("shadow") = false, py::arg("mode") = 0);

    m.def("spinner_inc_scale_dots",
        SpinnerIncScaleDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("dots") = 6, py::arg("angle") = 0.f, py::arg("mode") = 0);

    m.def("spinner_some_scale_dots",
        SpinnerSomeScaleDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("dots") = 6, py::arg("mode") = 0);

    m.def("spinner_ang_triple",
        SpinnerAngTriple, py::arg("label"), py::arg("radius1"), py::arg("radius2"), py::arg("radius3"), py::arg("thickness"), py::arg("c1") = white, py::arg("c2") = half_white, py::arg("c3") = white, py::arg("speed") = 2.8f, py::arg("angle") = IM_PI);

    m.def("spinner_ang_eclipse",
        SpinnerAngEclipse, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("angle") = IM_PI);

    m.def("spinner_ing_yang",
        SpinnerIngYang, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("reverse"), py::arg("yang_detlta_r"), py::arg("color_i") = white, py::arg("color_y") = white, py::arg("speed") = 2.8f, py::arg("angle") = IM_PI * 0.7f);

    m.def("spinner_gooey_balls",
        SpinnerGooeyBalls, py::arg("label"), py::arg("radius"), py::arg("color"), py::arg("speed"), py::arg("mode") = 0);

    m.def("spinner_dots_loading",
        SpinnerDotsLoading, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("bg"), py::arg("speed"));

    m.def("spinner_rotate_gooey_balls",
        SpinnerRotateGooeyBalls, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("balls"));

    m.def("spinner_herbert_balls",
        SpinnerHerbertBalls, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("balls"));

    m.def("spinner_herbert_balls3_d",
        SpinnerHerbertBalls3D, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"));

    m.def("spinner_rotate_triangles",
        SpinnerRotateTriangles, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("tris"));

    m.def("spinner_rotate_shapes",
        SpinnerRotateShapes, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("shapes"), py::arg("pnt"));

    m.def("spinner_sin_squares",
        SpinnerSinSquares, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"));

    m.def("spinner_moon_line",
        SpinnerMoonLine, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("bg") = red, py::arg("speed") = 2.8f, py::arg("angle") = IM_PI);

    m.def("spinner_circle_drop",
        SpinnerCircleDrop, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("thickness_drop"), py::arg("color") = white, py::arg("bg") = half_white, py::arg("speed") = 2.8f, py::arg("angle") = IM_PI);

    m.def("spinner_surrounded_indicator",
        SpinnerSurroundedIndicator, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("bg") = half_white, py::arg("speed") = 2.8f);

    m.def("spinner_wifi_indicator",
        SpinnerWifiIndicator, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = red, py::arg("bg") = half_white, py::arg("speed") = 2.8f, py::arg("cangle") = 0.f, py::arg("dots") = 3);

    m.def("spinner_triangles_selector",
        SpinnerTrianglesSelector, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("bg") = half_white, py::arg("speed") = 2.8f, py::arg("bars") = 8);

    m.def("spinner_camera",
        SpinnerCamera, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("leaf_color"), py::arg("speed") = 2.8f, py::arg("bars") = 8);

    m.def("spinner_flowing_gradient",
        SpinnerFlowingGradient, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("bg") = red, py::arg("speed") = 2.8f, py::arg("angle") = IM_PI);

    m.def("spinner_rotate_segments",
        SpinnerRotateSegments, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4, py::arg("layers") = 1);

    m.def("spinner_lemniscate",
        SpinnerLemniscate, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("angle") = IM_PI / 2.0f);

    m.def("spinner_rotate_gear",
        SpinnerRotateGear, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("pins") = 12);

    m.def("spinner_rotate_wheel",
        SpinnerRotateWheel, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("bg_color") = white, py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("pins") = 12);

    m.def("spinner_atom",
        SpinnerAtom, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("elipses") = 3);

    m.def("spinner_pattern_rings",
        SpinnerPatternRings, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("elipses") = 3);

    m.def("spinner_pattern_eclipse",
        SpinnerPatternEclipse, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("elipses") = 3, py::arg("delta_a") = 2.f, py::arg("delta_y") = 0.f);

    m.def("spinner_pattern_sphere",
        SpinnerPatternSphere, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("elipses") = 3);

    m.def("spinner_ring_synchronous",
        SpinnerRingSynchronous, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("elipses") = 3);

    m.def("spinner_ring_watermarks",
        SpinnerRingWatermarks, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("elipses") = 3);

    m.def("spinner_rotated_atom",
        SpinnerRotatedAtom, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("elipses") = 3);

    m.def("spinner_rainbow_balls",
        SpinnerRainbowBalls, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("balls") = 5);

    m.def("spinner_rainbow_shot",
        SpinnerRainbowShot, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("balls") = 5);

    m.def("spinner_spiral",
        SpinnerSpiral, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4);

    m.def("spinner_spiral_eye",
        SpinnerSpiralEye, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f);

    m.def("spinner_bar_chart_sine",
        SpinnerBarChartSine, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("bars") = 5, py::arg("mode") = 0);

    m.def("spinner_bar_chart_adv_sine",
        SpinnerBarChartAdvSine, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("mode") = 0);

    m.def("spinner_bar_chart_adv_sine_fade",
        SpinnerBarChartAdvSineFade, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("mode") = 0);

    m.def("spinner_bar_chart_rainbow",
        SpinnerBarChartRainbow, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("bars") = 5);

    m.def("spinner_blocks",
        SpinnerBlocks, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("bg"), py::arg("color"), py::arg("speed"));

    m.def("spinner_twin_blocks",
        SpinnerTwinBlocks, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("bg"), py::arg("color"), py::arg("speed"));

    m.def("spinner_square_random_dots",
        SpinnerSquareRandomDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("bg"), py::arg("color"), py::arg("speed"));

    m.def("spinner_scale_blocks",
        SpinnerScaleBlocks, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("mode") = 0);

    m.def("spinner_scale_squares",
        SpinnerScaleSquares, py::arg("label"), py::arg("radius"), py::arg("thikness"), py::arg("color"), py::arg("speed"));

    m.def("spinner_squish_square",
        SpinnerSquishSquare, py::arg("label"), py::arg("radius"), py::arg("color"), py::arg("speed"));

    m.def("spinner_fluid",
        SpinnerFluid, py::arg("label"), py::arg("radius"), py::arg("color"), py::arg("speed"), py::arg("bars") = 3);

    m.def("spinner_fluid_points",
        SpinnerFluidPoints, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color"), py::arg("speed"), py::arg("dots") = 6, py::arg("delta") = 0.35f);

    m.def("spinner_arc_polar_fade",
        SpinnerArcPolarFade, py::arg("label"), py::arg("radius"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4);

    m.def("spinner_arc_polar_radius",
        SpinnerArcPolarRadius, py::arg("label"), py::arg("radius"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4);

    m.def("spinner_caleidoscope",
        SpinnerCaleidoscope, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 6, py::arg("mode") = 0);

    m.def("spinner_hbo_dots",
        SpinnerHboDots,
        py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("minfade") = 0.0f, py::arg("ryk") = 0.f, py::arg("speed") = 1.1f, py::arg("dots") = 6,
        "spinner idea by nitz 'Chris Dailey'");

    m.def("spinner_moon_dots",
        SpinnerMoonDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("first"), py::arg("second"), py::arg("speed") = 1.1f);

    m.def("spinner_twin_hbo_dots",
        SpinnerTwinHboDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("minfade") = 0.0f, py::arg("ryk") = 0.f, py::arg("speed") = 1.1f, py::arg("dots") = 6, py::arg("delta") = 0.f);

    m.def("spinner_three_dots_star",
        SpinnerThreeDotsStar, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("minfade") = 0.0f, py::arg("ryk") = 0.f, py::arg("speed") = 1.1f, py::arg("delta") = 0.f);

    m.def("spinner_sine_arcs",
        SpinnerSineArcs, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f);

    m.def("spinner_triangles_shift",
        SpinnerTrianglesShift, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("bg") = half_white, py::arg("speed") = 2.8f, py::arg("bars") = 8);

    m.def("spinner_points_shift",
        SpinnerPointsShift, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("bg") = half_white, py::arg("speed") = 2.8f, py::arg("bars") = 8);

    m.def("spinner_swing_dots",
        SpinnerSwingDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f);

    m.def("spinner_circular_points",
        SpinnerCircularPoints, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 1.8f, py::arg("lines") = 8);

    m.def("spinner_curved_circle",
        SpinnerCurvedCircle, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("circles") = 1);

    m.def("spinner_mod_circle",
        SpinnerModCircle, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("ang_min") = 1.f, py::arg("ang_max") = 1.f, py::arg("speed") = 2.8f);

    m.def("spinner_dna_dots",
        SpinnerDnaDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("lt") = 8, py::arg("delta") = 0.5f, py::arg("mode") = 0);

    m.def("spinner3_smuggle_dots",
        Spinner3SmuggleDots, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 4.8f, py::arg("lt") = 8, py::arg("delta") = 0.5f, py::arg("mode") = 0);

    m.def("spinner_rotate_segments_pulsar",
        SpinnerRotateSegmentsPulsar, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("speed") = 2.8f, py::arg("arcs") = 4, py::arg("layers") = 1);

    m.def("spinner_spline_ang",
        SpinnerSplineAng, py::arg("label"), py::arg("radius"), py::arg("thickness"), py::arg("color") = white, py::arg("bg") = white, py::arg("speed") = 2.8f, py::arg("angle") = IM_PI, py::arg("mode") = 0);
    // #endif
    // #endif
    ////////////////////    </generated_from:imspinner.h>    ////////////////////

    // </litgen_pydef> // Autogenerated code end
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  AUTOGENERATED CODE END !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
}
