// Part of ImGui Bundle - MIT License - Copyright (c) 2022-2024 Pascal Thomet - https://github.com/pthom/imgui_bundle
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include <pybind11/functional.h>
#include <pybind11/numpy.h>

#include "imgui.h"
#include "imgui_internal.h"
#include "misc/cpp/imgui_stdlib.h"
#include "imgui_internal_pywrappers.h"


namespace py = pybind11;



// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  AUTOGENERATED CODE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// <litgen_glue_code>  // Autogenerated code below! Do not edit!

// </litgen_glue_code> // Autogenerated code end
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  AUTOGENERATED CODE END !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


void py_init_module_imgui_internal(py::module& m)
{
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  AUTOGENERATED CODE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    // <litgen_pydef> // Autogenerated code below! Do not edit!
    ////////////////////    <generated_from:imgui_internal.h>    ////////////////////
    // #ifndef IMGUI_DISABLE
    //
    // #ifdef IMGUI_BUNDLE_PYTHON_API
    //
    // #endif
    //
    // #ifdef IMGUI_BUNDLE_PYTHON_API
    //
    // #endif
    //

    m.def("im_hash_data",
        ImHashData, py::arg("data"), py::arg("data_size"), py::arg("seed") = 0);

    m.def("im_hash_str",
        ImHashStr, py::arg("data"), py::arg("data_size") = 0, py::arg("seed") = 0);

    m.def("im_alpha_blend_colors",
        ImAlphaBlendColors,
        py::arg("col_a"), py::arg("col_b"),
        "Helpers: Color Blending");

    m.def("im_is_power_of_two",
        py::overload_cast<int>(ImIsPowerOfTwo),
        py::arg("v"),
        "(private API)");

    m.def("im_is_power_of_two",
        py::overload_cast<ImU64>(ImIsPowerOfTwo),
        py::arg("v"),
        "(private API)");

    m.def("im_upper_power_of_two",
        ImUpperPowerOfTwo,
        py::arg("v"),
        "(private API)");

    m.def("im_char_is_blank_w",
        ImCharIsBlankW,
        py::arg("c"),
        "(private API)");

    m.def("im_text_find_previous_utf8_codepoint",
        ImTextFindPreviousUtf8Codepoint,
        py::arg("in_text_start"), py::arg("in_text_curr"),
        pybind11::return_value_policy::reference);

    m.def("im_min",
        [](const ImVec2 & lhs, const ImVec2 & rhs) -> ImVec2
        {
            auto ImMin_adapt_force_lambda = [](const ImVec2 & lhs, const ImVec2 & rhs) -> ImVec2
            {
                auto lambda_result = ImMin(lhs, rhs);
                return lambda_result;
            };

            return ImMin_adapt_force_lambda(lhs, rhs);
        },
        py::arg("lhs"), py::arg("rhs"),
        "(private API)");

    m.def("im_max",
        [](const ImVec2 & lhs, const ImVec2 & rhs) -> ImVec2
        {
            auto ImMax_adapt_force_lambda = [](const ImVec2 & lhs, const ImVec2 & rhs) -> ImVec2
            {
                auto lambda_result = ImMax(lhs, rhs);
                return lambda_result;
            };

            return ImMax_adapt_force_lambda(lhs, rhs);
        },
        py::arg("lhs"), py::arg("rhs"),
        "(private API)");

    m.def("im_clamp",
        [](const ImVec2 & v, const ImVec2 & mn, ImVec2 mx) -> ImVec2
        {
            auto ImClamp_adapt_force_lambda = [](const ImVec2 & v, const ImVec2 & mn, ImVec2 mx) -> ImVec2
            {
                auto lambda_result = ImClamp(v, mn, mx);
                return lambda_result;
            };

            return ImClamp_adapt_force_lambda(v, mn, mx);
        },
        py::arg("v"), py::arg("mn"), py::arg("mx"),
        "(private API)");

    m.def("im_lerp",
        [](const ImVec2 & a, const ImVec2 & b, float t) -> ImVec2
        {
            auto ImLerp_adapt_force_lambda = [](const ImVec2 & a, const ImVec2 & b, float t) -> ImVec2
            {
                auto lambda_result = ImLerp(a, b, t);
                return lambda_result;
            };

            return ImLerp_adapt_force_lambda(a, b, t);
        },
        py::arg("a"), py::arg("b"), py::arg("t"),
        "(private API)");

    m.def("im_lerp",
        [](const ImVec2 & a, const ImVec2 & b, const ImVec2 & t) -> ImVec2
        {
            auto ImLerp_adapt_force_lambda = [](const ImVec2 & a, const ImVec2 & b, const ImVec2 & t) -> ImVec2
            {
                auto lambda_result = ImLerp(a, b, t);
                return lambda_result;
            };

            return ImLerp_adapt_force_lambda(a, b, t);
        },
        py::arg("a"), py::arg("b"), py::arg("t"),
        "(private API)");

    m.def("im_lerp",
        [](const ImVec4 & a, const ImVec4 & b, float t) -> ImVec4
        {
            auto ImLerp_adapt_force_lambda = [](const ImVec4 & a, const ImVec4 & b, float t) -> ImVec4
            {
                auto lambda_result = ImLerp(a, b, t);
                return lambda_result;
            };

            return ImLerp_adapt_force_lambda(a, b, t);
        },
        py::arg("a"), py::arg("b"), py::arg("t"),
        "(private API)");

    m.def("im_saturate",
        ImSaturate,
        py::arg("f"),
        "(private API)");

    m.def("im_length_sqr",
        py::overload_cast<const ImVec2 &>(ImLengthSqr),
        py::arg("lhs"),
        "(private API)");

    m.def("im_length_sqr",
        py::overload_cast<const ImVec4 &>(ImLengthSqr),
        py::arg("lhs"),
        "(private API)");

    m.def("im_inv_length",
        ImInvLength,
        py::arg("lhs"), py::arg("fail_value"),
        "(private API)");

    m.def("im_trunc",
        py::overload_cast<float>(ImTrunc),
        py::arg("f"),
        "(private API)");

    m.def("im_trunc",
        py::overload_cast<const ImVec2 &>(ImTrunc),
        py::arg("v"),
        "(private API)");

    m.def("im_floor",
        py::overload_cast<float>(ImFloor),
        py::arg("f"),
        "(private API)\n\n Decent replacement for floorf()");

    m.def("im_floor",
        py::overload_cast<const ImVec2 &>(ImFloor),
        py::arg("v"),
        "(private API)");

    m.def("im_mod_positive",
        ImModPositive,
        py::arg("a"), py::arg("b"),
        "(private API)");

    m.def("im_dot",
        ImDot,
        py::arg("a"), py::arg("b"),
        "(private API)");

    m.def("im_rotate",
        ImRotate,
        py::arg("v"), py::arg("cos_a"), py::arg("sin_a"),
        "(private API)");

    m.def("im_linear_sweep",
        ImLinearSweep,
        py::arg("current"), py::arg("target"), py::arg("speed"),
        "(private API)");

    m.def("im_mul",
        ImMul,
        py::arg("lhs"), py::arg("rhs"),
        "(private API)");

    m.def("im_is_float_above_guaranteed_integer_precision",
        ImIsFloatAboveGuaranteedIntegerPrecision,
        py::arg("f"),
        "(private API)");

    m.def("im_exponential_moving_average",
        ImExponentialMovingAverage,
        py::arg("avg"), py::arg("sample"), py::arg("n"),
        "(private API)");

    m.def("im_bezier_cubic_calc",
        ImBezierCubicCalc, py::arg("p1"), py::arg("p2"), py::arg("p3"), py::arg("p4"), py::arg("t"));

    m.def("im_bezier_cubic_closest_point",
        ImBezierCubicClosestPoint,
        py::arg("p1"), py::arg("p2"), py::arg("p3"), py::arg("p4"), py::arg("p"), py::arg("num_segments"),
        "For curves with explicit number of segments");

    m.def("im_bezier_cubic_closest_point_casteljau",
        ImBezierCubicClosestPointCasteljau,
        py::arg("p1"), py::arg("p2"), py::arg("p3"), py::arg("p4"), py::arg("p"), py::arg("tess_tol"),
        "For auto-tessellated curves you can use tess_tol = style.CurveTessellationTol");

    m.def("im_bezier_quadratic_calc",
        ImBezierQuadraticCalc, py::arg("p1"), py::arg("p2"), py::arg("p3"), py::arg("t"));

    m.def("im_line_closest_point",
        ImLineClosestPoint, py::arg("a"), py::arg("b"), py::arg("p"));

    m.def("im_triangle_contains_point",
        ImTriangleContainsPoint, py::arg("a"), py::arg("b"), py::arg("c"), py::arg("p"));

    m.def("im_triangle_closest_point",
        ImTriangleClosestPoint, py::arg("a"), py::arg("b"), py::arg("c"), py::arg("p"));

    m.def("im_triangle_barycentric_coords",
        [](const ImVec2 & a, const ImVec2 & b, const ImVec2 & c, const ImVec2 & p, float out_u, float out_v, float out_w) -> std::tuple<float, float, float>
        {
            auto ImTriangleBarycentricCoords_adapt_modifiable_immutable_to_return = [](const ImVec2 & a, const ImVec2 & b, const ImVec2 & c, const ImVec2 & p, float out_u, float out_v, float out_w) -> std::tuple<float, float, float>
            {
                float & out_u_adapt_modifiable = out_u;
                float & out_v_adapt_modifiable = out_v;
                float & out_w_adapt_modifiable = out_w;

                ImTriangleBarycentricCoords(a, b, c, p, out_u_adapt_modifiable, out_v_adapt_modifiable, out_w_adapt_modifiable);
                return std::make_tuple(out_u, out_v, out_w);
            };

            return ImTriangleBarycentricCoords_adapt_modifiable_immutable_to_return(a, b, c, p, out_u, out_v, out_w);
        },     py::arg("a"), py::arg("b"), py::arg("c"), py::arg("p"), py::arg("out_u"), py::arg("out_v"), py::arg("out_w"));

    m.def("im_triangle_area",
        ImTriangleArea,
        py::arg("a"), py::arg("b"), py::arg("c"),
        "(private API)");

    m.def("im_triangle_is_clockwise",
        ImTriangleIsClockwise,
        py::arg("a"), py::arg("b"), py::arg("c"),
        "(private API)");


    auto pyClassImVec1 =
        py::class_<ImVec1>
            (m, "ImVec1", "")
        .def_readwrite("x", &ImVec1::x, "")
        .def(py::init<>())
        .def(py::init<float>(),
            py::arg("_x"))
        ;


    auto pyClassImVec2ih =
        py::class_<ImVec2ih>
            (m, "ImVec2ih", "Helper: ImVec2ih (2D vector, half-size integer, for long-term packed storage)")
        .def_readwrite("x", &ImVec2ih::x, "")
        .def_readwrite("y", &ImVec2ih::y, "")
        .def(py::init<>())
        .def(py::init<short, short>(),
            py::arg("_x"), py::arg("_y"))
        .def(py::init<const ImVec2 &>(),
            py::arg("rhs"))
        ;


    auto pyClassImRect =
        py::class_<ImRect>
            (m, "ImRect", " Helper: ImRect (2D axis aligned bounding-box)\n NB: we can't rely on ImVec2 math operators being available here!")
        .def_readwrite("min", &ImRect::Min, "Upper-left")
        .def_readwrite("max", &ImRect::Max, "Lower-right")
        .def(py::init<>())
        .def(py::init<const ImVec2 &, const ImVec2 &>(),
            py::arg("min"), py::arg("max"))
        .def(py::init<const ImVec4 &>(),
            py::arg("v"))
        .def(py::init<float, float, float, float>(),
            py::arg("x1"), py::arg("y1"), py::arg("x2"), py::arg("y2"))
        .def("get_center",
            &ImRect::GetCenter, "(private API)")
        .def("get_size",
            &ImRect::GetSize, "(private API)")
        .def("get_width",
            &ImRect::GetWidth, "(private API)")
        .def("get_height",
            &ImRect::GetHeight, "(private API)")
        .def("get_area",
            &ImRect::GetArea, "(private API)")
        .def("get_tl",
            &ImRect::GetTL, "(private API)\n\n Top-left")
        .def("get_tr",
            &ImRect::GetTR, "(private API)\n\n Top-right")
        .def("get_bl",
            &ImRect::GetBL, "(private API)\n\n Bottom-left")
        .def("get_br",
            &ImRect::GetBR, "(private API)\n\n Bottom-right")
        .def("contains",
            [](const ImRect & self, const ImVec2 & p) -> bool
            {
                auto Contains_adapt_force_lambda = [&self](const ImVec2 & p) -> bool
                {
                    auto lambda_result = self.Contains(p);
                    return lambda_result;
                };

                return Contains_adapt_force_lambda(p);
            },
            py::arg("p"),
            "(private API)")
        .def("contains",
            [](const ImRect & self, const ImRect & r) -> bool
            {
                auto Contains_adapt_force_lambda = [&self](const ImRect & r) -> bool
                {
                    auto lambda_result = self.Contains(r);
                    return lambda_result;
                };

                return Contains_adapt_force_lambda(r);
            },
            py::arg("r"),
            "(private API)")
        .def("contains_with_pad",
            &ImRect::ContainsWithPad,
            py::arg("p"), py::arg("pad"),
            "(private API)")
        .def("overlaps",
            &ImRect::Overlaps,
            py::arg("r"),
            "(private API)")
        .def("add",
            py::overload_cast<const ImVec2 &>(&ImRect::Add),
            py::arg("p"),
            "(private API)")
        .def("add",
            py::overload_cast<const ImRect &>(&ImRect::Add),
            py::arg("r"),
            "(private API)")
        .def("expand",
            py::overload_cast<const float>(&ImRect::Expand),
            py::arg("amount"),
            "(private API)")
        .def("expand",
            py::overload_cast<const ImVec2 &>(&ImRect::Expand),
            py::arg("amount"),
            "(private API)")
        .def("translate",
            &ImRect::Translate,
            py::arg("d"),
            "(private API)")
        .def("translate_x",
            &ImRect::TranslateX,
            py::arg("dx"),
            "(private API)")
        .def("translate_y",
            &ImRect::TranslateY,
            py::arg("dy"),
            "(private API)")
        .def("clip_with",
            &ImRect::ClipWith,
            py::arg("r"),
            "(private API)\n\n Simple version, may lead to an inverted rectangle, which is fine for Contains/Overlaps test but not for display.")
        .def("clip_with_full",
            &ImRect::ClipWithFull,
            py::arg("r"),
            "(private API)\n\n Full version, ensure both points are fully clipped.")
        .def("floor",
            &ImRect::Floor, "(private API)")
        .def("is_inverted",
            &ImRect::IsInverted, "(private API)")
        .def("to_vec4",
            &ImRect::ToVec4, "(private API)")
        .def("__copy__",  [](const ImRect &self) {
            return ImRect(self);
        })    ;


    m.def("im_bit_array_get_storage_size_in_bytes",
        ImBitArrayGetStorageSizeInBytes,
        py::arg("bitcount"),
        "(private API)");

    m.def("im_bit_array_clear_all_bits",
        ImBitArrayClearAllBits,
        py::arg("arr"), py::arg("bitcount"),
        "(private API)");

    m.def("im_bit_array_test_bit",
        ImBitArrayTestBit,
        py::arg("arr"), py::arg("n"),
        "(private API)");

    m.def("im_bit_array_clear_bit",
        ImBitArrayClearBit,
        py::arg("arr"), py::arg("n"),
        "(private API)");

    m.def("im_bit_array_set_bit",
        ImBitArraySetBit,
        py::arg("arr"), py::arg("n"),
        "(private API)");

    m.def("im_bit_array_set_bit_range",
        ImBitArraySetBitRange,
        py::arg("arr"), py::arg("n"), py::arg("n2"),
        "(private API)\n\n// Works on range [n..n2)");


    auto pyClassImBitVector =
        py::class_<ImBitVector>
            (m, "ImBitVector", " Helper: ImBitVector\n Store 1-bit per value.")
        .def(py::init<>([](
        ImVector<ImU32> Storage = ImVector<ImU32>())
        {
            auto r = std::make_unique<ImBitVector>();
            r->Storage = Storage;
            return r;
        })
        , py::arg("storage") = ImVector<ImU32>()
        )
        .def_readwrite("storage", &ImBitVector::Storage, "")
        .def("create",
            &ImBitVector::Create,
            py::arg("sz"),
            "(private API)")
        .def("clear",
            &ImBitVector::Clear, "(private API)")
        .def("test_bit",
            &ImBitVector::TestBit,
            py::arg("n"),
            "(private API)")
        .def("set_bit",
            &ImBitVector::SetBit,
            py::arg("n"),
            "(private API)")
        .def("clear_bit",
            &ImBitVector::ClearBit,
            py::arg("n"),
            "(private API)")
        ;


    auto pyClassImGuiTextIndex =
        py::class_<ImGuiTextIndex>
            (m, "TextIndex", " Helper: ImGuiTextIndex<>\n Maintain a line index for a text buffer. This is a strong candidate to be moved into the public API.")
        .def(py::init<>([](
        ImVector<int> LineOffsets = ImVector<int>(), int EndOffset = 0)
        {
            auto r = std::make_unique<ImGuiTextIndex>();
            r->LineOffsets = LineOffsets;
            r->EndOffset = EndOffset;
            return r;
        })
        , py::arg("line_offsets") = ImVector<int>(), py::arg("end_offset") = 0
        )
        .def_readwrite("line_offsets", &ImGuiTextIndex::LineOffsets, "")
        .def_readwrite("end_offset", &ImGuiTextIndex::EndOffset, "Because we don't own text buffer we need to maintain EndOffset (may bake in LineOffsets?)")
        .def("clear",
            &ImGuiTextIndex::clear, "(private API)")
        .def("size",
            &ImGuiTextIndex::size, "(private API)")
        .def("get_line_begin",
            &ImGuiTextIndex::get_line_begin,
            py::arg("base"), py::arg("n"),
            "(private API)",
            pybind11::return_value_policy::reference)
        .def("get_line_end",
            &ImGuiTextIndex::get_line_end,
            py::arg("base"), py::arg("n"),
            "(private API)",
            pybind11::return_value_policy::reference)
        .def("append",
            &ImGuiTextIndex::append,
            py::arg("base"), py::arg("old_size"), py::arg("new_size"),
            "(private API)")
        ;


    auto pyClassImDrawListSharedData =
        py::class_<ImDrawListSharedData>
            (m, "ImDrawListSharedData", " Data shared between all ImDrawList instances\n You may want to create your own instance of this if you want to use ImDrawList completely without ImGui. In that case, watch out for future changes to this structure.")
        .def_readwrite("tex_uv_white_pixel", &ImDrawListSharedData::TexUvWhitePixel, "UV of white pixel in the atlas")
        .def_readwrite("font", &ImDrawListSharedData::Font, "Current/default font (optional, for simplified AddText overload)")
        .def_readwrite("font_size", &ImDrawListSharedData::FontSize, "Current/default font size (optional, for simplified AddText overload)")
        .def_readwrite("curve_tessellation_tol", &ImDrawListSharedData::CurveTessellationTol, "Tessellation tolerance when using PathBezierCurveTo()")
        .def_readwrite("circle_segment_max_error", &ImDrawListSharedData::CircleSegmentMaxError, "Number of circle segments to use per pixel of radius for AddCircle() etc")
        .def_readwrite("clip_rect_fullscreen", &ImDrawListSharedData::ClipRectFullscreen, "Value for PushClipRectFullscreen()")
        .def_readwrite("initial_flags", &ImDrawListSharedData::InitialFlags, "Initial flags at the beginning of the frame (it is possible to alter flags on a per-drawlist basis afterwards)")
        .def_readwrite("temp_buffer", &ImDrawListSharedData::TempBuffer, "[Internal] Temp write buffer")
        .def_readwrite("arc_fast_radius_cutoff", &ImDrawListSharedData::ArcFastRadiusCutoff, "Cutoff radius after which arc drawing will fallback to slower PathArcTo()")
        .def_property("circle_segment_counts",
            [](ImDrawListSharedData &self) -> pybind11::array
            {
                auto dtype = pybind11::dtype(pybind11::format_descriptor<ImU8>::format());
                auto base = pybind11::array(dtype, {64}, {sizeof(ImU8)});
                return pybind11::array(dtype, {64}, {sizeof(ImU8)}, self.CircleSegmentCounts, base);
            }, [](ImDrawListSharedData& self) {},
            "Precomputed segment count for given radius before we calculate it dynamically (to avoid calculation overhead)")
        .def_readonly("tex_uv_lines", &ImDrawListSharedData::TexUvLines, "UV of anti-aliased lines in the atlas")
        .def(py::init<>())
        .def("set_circle_tessellation_max_error",
            &ImDrawListSharedData::SetCircleTessellationMaxError,
            py::arg("max_error"),
            "(private API)")
        ;


    auto pyClassImDrawDataBuilder =
        py::class_<ImDrawDataBuilder>
            (m, "ImDrawDataBuilder", "")
        .def_readwrite("layer_data1", &ImDrawDataBuilder::LayerData1, "")
        .def(py::init<>())
        ;


    py::enum_<ImGuiItemFlags_>(m, "ItemFlags_", py::arithmetic(), " Flags used by upcoming items\n - input: PushItemFlag() manipulates g.CurrentItemFlags, ItemAdd() calls may add extra flags.\n - output: stored in g.LastItemData.InFlags\n Current window shared by all windows.\n This is going to be exposed in imgui.h when stabilized enough.")
        .value("none", ImGuiItemFlags_None, "")
        .value("no_tab_stop", ImGuiItemFlags_NoTabStop, "False     // Disable keyboard tabbing. This is a \"lighter\" version of ImGuiItemFlags_NoNav.")
        .value("button_repeat", ImGuiItemFlags_ButtonRepeat, "False     // Button() will return True multiple times based on io.KeyRepeatDelay and io.KeyRepeatRate settings.")
        .value("disabled", ImGuiItemFlags_Disabled, "False     // Disable interactions but doesn't affect visuals. See BeginDisabled()/EndDisabled(). See github.com/ocornut/imgui/issues/211")
        .value("no_nav", ImGuiItemFlags_NoNav, "False     // Disable any form of focusing (keyboard/gamepad directional navigation and SetKeyboardFocusHere() calls)")
        .value("no_nav_default_focus", ImGuiItemFlags_NoNavDefaultFocus, "False     // Disable item being a candidate for default focus (e.g. used by title bar items)")
        .value("selectable_dont_close_popup", ImGuiItemFlags_SelectableDontClosePopup, "False     // Disable MenuItem/Selectable() automatically closing their popup window")
        .value("mixed_value", ImGuiItemFlags_MixedValue, "False     // [BETA] Represent a mixed/indeterminate value, generally multi-selection where values differ. Currently only supported by Checkbox() (later should support all sorts of widgets)")
        .value("read_only", ImGuiItemFlags_ReadOnly, "False     // [ALPHA] Allow hovering interactions but underlying value is not changed.")
        .value("no_window_hoverable_check", ImGuiItemFlags_NoWindowHoverableCheck, "False     // Disable hoverable check in ItemHoverable()")
        .value("allow_overlap", ImGuiItemFlags_AllowOverlap, "False     // Allow being overlapped by another widget. Not-hovered to Hovered transition deferred by a frame.")
        .value("inputable", ImGuiItemFlags_Inputable, "False     // [WIP] Auto-activate input mode when tab focused. Currently only used and supported by a few items before it becomes a generic feature.")
        .value("has_selection_user_data", ImGuiItemFlags_HasSelectionUserData, "False     // Set by SetNextItemSelectionUserData()");


    py::enum_<ImGuiItemStatusFlags_>(m, "ItemStatusFlags_", py::arithmetic(), " Status flags for an already submitted item\n - output: stored in g.LastItemData.StatusFlags")
        .value("none", ImGuiItemStatusFlags_None, "")
        .value("hovered_rect", ImGuiItemStatusFlags_HoveredRect, "Mouse position is within item rectangle (does NOT mean that the window is in correct z-order and can be hovered!, this is only one part of the most-common IsItemHovered test)")
        .value("has_display_rect", ImGuiItemStatusFlags_HasDisplayRect, "g.LastItemData.DisplayRect is valid")
        .value("edited", ImGuiItemStatusFlags_Edited, "Value exposed by item was edited in the current frame (should match the bool return value of most widgets)")
        .value("toggled_selection", ImGuiItemStatusFlags_ToggledSelection, "Set when Selectable(), TreeNode() reports toggling a selection. We can't report \"Selected\", only state changes, in order to easily handle clipping with less issues.")
        .value("toggled_open", ImGuiItemStatusFlags_ToggledOpen, "Set when TreeNode() reports toggling their open state.")
        .value("has_deactivated", ImGuiItemStatusFlags_HasDeactivated, "Set if the widget/group is able to provide data for the ImGuiItemStatusFlags_Deactivated flag.")
        .value("deactivated", ImGuiItemStatusFlags_Deactivated, "Only valid if ImGuiItemStatusFlags_HasDeactivated is set.")
        .value("hovered_window", ImGuiItemStatusFlags_HoveredWindow, "Override the HoveredWindow test to allow cross-window hover testing.")
        .value("visible", ImGuiItemStatusFlags_Visible, "[WIP] Set when item is overlapping the current clipping rectangle (Used internally. Please don't use yet: API/system will change as we refactor Itemadd()).")
        .value("has_clip_rect", ImGuiItemStatusFlags_HasClipRect, "g.LastItemData.ClipRect is valid");


    py::enum_<ImGuiHoveredFlagsPrivate_>(m, "HoveredFlagsPrivate_", py::arithmetic(), "Extend ImGuiHoveredFlags_")
        .value("im_gui_hovered_flags_delay_mask_", ImGuiHoveredFlags_DelayMask_, "")
        .value("im_gui_hovered_flags_allowed_mask_for_is_window_hovered", ImGuiHoveredFlags_AllowedMaskForIsWindowHovered, "")
        .value("im_gui_hovered_flags_allowed_mask_for_is_item_hovered", ImGuiHoveredFlags_AllowedMaskForIsItemHovered, "");


    py::enum_<ImGuiInputTextFlagsPrivate_>(m, "InputTextFlagsPrivate_", py::arithmetic(), "Extend ImGuiInputTextFlags_")
        .value("im_gui_input_text_flags_multiline", ImGuiInputTextFlags_Multiline, "For internal use by InputTextMultiline()")
        .value("im_gui_input_text_flags_no_mark_edited", ImGuiInputTextFlags_NoMarkEdited, "For internal use by functions using InputText() before reformatting data")
        .value("im_gui_input_text_flags_merged_item", ImGuiInputTextFlags_MergedItem, "For internal use by TempInputText(), will skip calling ItemAdd(). Require bounding-box to strictly match.")
        .value("im_gui_input_text_flags_localize_decimal_point", ImGuiInputTextFlags_LocalizeDecimalPoint, "For internal use by InputScalar() and TempInputScalar()");


    py::enum_<ImGuiButtonFlagsPrivate_>(m, "ButtonFlagsPrivate_", py::arithmetic(), "Extend ImGuiButtonFlags_")
        .value("im_gui_button_flags_pressed_on_click", ImGuiButtonFlags_PressedOnClick, "return True on click (mouse down event)")
        .value("im_gui_button_flags_pressed_on_click_release", ImGuiButtonFlags_PressedOnClickRelease, "[Default] return True on click + release on same item <-- this is what the majority of Button are using")
        .value("im_gui_button_flags_pressed_on_click_release_anywhere", ImGuiButtonFlags_PressedOnClickReleaseAnywhere, "return True on click + release even if the release event is not done while hovering the item")
        .value("im_gui_button_flags_pressed_on_release", ImGuiButtonFlags_PressedOnRelease, "return True on release (default requires click+release)")
        .value("im_gui_button_flags_pressed_on_double_click", ImGuiButtonFlags_PressedOnDoubleClick, "return True on double-click (default requires click+release)")
        .value("im_gui_button_flags_pressed_on_drag_drop_hold", ImGuiButtonFlags_PressedOnDragDropHold, "return True when held into while we are drag and dropping another item (used by e.g. tree nodes, collapsing headers)")
        .value("im_gui_button_flags_repeat", ImGuiButtonFlags_Repeat, "hold to repeat")
        .value("im_gui_button_flags_flatten_children", ImGuiButtonFlags_FlattenChildren, "allow interactions even if a child window is overlapping")
        .value("im_gui_button_flags_allow_overlap", ImGuiButtonFlags_AllowOverlap, "require previous frame HoveredId to either match id or be null before being usable.")
        .value("im_gui_button_flags_dont_close_popups", ImGuiButtonFlags_DontClosePopups, "disable automatically closing parent popup on press // [UNUSED]")
        .value("im_gui_button_flags_align_text_base_line", ImGuiButtonFlags_AlignTextBaseLine, "vertically align button to match text baseline - ButtonEx() only // FIXME: Should be removed and handled by SmallButton(), not possible currently because of DC.CursorPosPrevLine")
        .value("im_gui_button_flags_no_key_modifiers", ImGuiButtonFlags_NoKeyModifiers, "disable mouse interaction if a key modifier is held")
        .value("im_gui_button_flags_no_holding_active_id", ImGuiButtonFlags_NoHoldingActiveId, "don't set ActiveId while holding the mouse (ImGuiButtonFlags_PressedOnClick only)")
        .value("im_gui_button_flags_no_nav_focus", ImGuiButtonFlags_NoNavFocus, "don't override navigation focus when activated (FIXME: this is essentially used everytime an item uses ImGuiItemFlags_NoNav, but because legacy specs don't requires LastItemData to be set ButtonBehavior(), we can't poll g.LastItemData.InFlags)")
        .value("im_gui_button_flags_no_hovered_on_focus", ImGuiButtonFlags_NoHoveredOnFocus, "don't report as hovered when nav focus is on this item")
        .value("im_gui_button_flags_no_set_key_owner", ImGuiButtonFlags_NoSetKeyOwner, "don't set key/input owner on the initial click (note: mouse buttons are keys! often, the key in question will be ImGuiKey_MouseLeft!)")
        .value("im_gui_button_flags_no_test_key_owner", ImGuiButtonFlags_NoTestKeyOwner, "don't test key/input owner when polling the key (note: mouse buttons are keys! often, the key in question will be ImGuiKey_MouseLeft!)")
        .value("im_gui_button_flags_pressed_on_mask_", ImGuiButtonFlags_PressedOnMask_, "")
        .value("im_gui_button_flags_pressed_on_default_", ImGuiButtonFlags_PressedOnDefault_, "");


    py::enum_<ImGuiComboFlagsPrivate_>(m, "ComboFlagsPrivate_", py::arithmetic(), "Extend ImGuiComboFlags_")
        .value("im_gui_combo_flags_custom_preview", ImGuiComboFlags_CustomPreview, "enable BeginComboPreview()");


    py::enum_<ImGuiSliderFlagsPrivate_>(m, "SliderFlagsPrivate_", py::arithmetic(), "Extend ImGuiSliderFlags_")
        .value("im_gui_slider_flags_vertical", ImGuiSliderFlags_Vertical, "Should this slider be orientated vertically?")
        .value("im_gui_slider_flags_read_only", ImGuiSliderFlags_ReadOnly, "Consider using g.NextItemData.ItemFlags |= ImGuiItemFlags_ReadOnly instead.");


    py::enum_<ImGuiSelectableFlagsPrivate_>(m, "SelectableFlagsPrivate_", py::arithmetic(), "Extend ImGuiSelectableFlags_")
        .value("no_holding_active_id", ImGuiSelectableFlags_NoHoldingActiveID, "")
        .value("select_on_nav", ImGuiSelectableFlags_SelectOnNav, "(WIP) Auto-select when moved into. This is not exposed in public API as to handle multi-select and modifiers we will need user to explicitly control focus scope. May be replaced with a BeginSelection() API.")
        .value("select_on_click", ImGuiSelectableFlags_SelectOnClick, "Override button behavior to react on Click (default is Click+Release)")
        .value("select_on_release", ImGuiSelectableFlags_SelectOnRelease, "Override button behavior to react on Release (default is Click+Release)")
        .value("span_avail_width", ImGuiSelectableFlags_SpanAvailWidth, "Span all avail width even if we declared less for layout purpose. FIXME: We may be able to remove this (added in 6251379, 2bcafc86 for menus)")
        .value("set_nav_id_on_hover", ImGuiSelectableFlags_SetNavIdOnHover, "Set Nav/Focus ID on mouse hover (used by MenuItem)")
        .value("no_pad_with_half_spacing", ImGuiSelectableFlags_NoPadWithHalfSpacing, "Disable padding each side with ItemSpacing * 0.5")
        .value("no_set_key_owner", ImGuiSelectableFlags_NoSetKeyOwner, "Don't set key/input owner on the initial click (note: mouse buttons are keys! often, the key in question will be ImGuiKey_MouseLeft!)");


    py::enum_<ImGuiTreeNodeFlagsPrivate_>(m, "TreeNodeFlagsPrivate_", py::arithmetic(), "Extend ImGuiTreeNodeFlags_")
        .value("im_gui_tree_node_flags_clip_label_for_trailing_button", ImGuiTreeNodeFlags_ClipLabelForTrailingButton, "")
        .value("im_gui_tree_node_flags_upside_down_arrow", ImGuiTreeNodeFlags_UpsideDownArrow, "(FIXME-WIP) Turn Down arrow into an Up arrow, but reversed trees (#6517)");


    py::enum_<ImGuiSeparatorFlags_>(m, "SeparatorFlags_", py::arithmetic(), "")
        .value("none", ImGuiSeparatorFlags_None, "")
        .value("horizontal", ImGuiSeparatorFlags_Horizontal, "Axis default to current layout type, so generally Horizontal unless e.g. in a menu bar")
        .value("vertical", ImGuiSeparatorFlags_Vertical, "")
        .value("span_all_columns", ImGuiSeparatorFlags_SpanAllColumns, "Make separator cover all columns of a legacy Columns() set.");


    py::enum_<ImGuiFocusRequestFlags_>(m, "FocusRequestFlags_", py::arithmetic(), " Flags for FocusWindow(). This is not called ImGuiFocusFlags to avoid confusion with public-facing ImGuiFocusedFlags.\n FIXME: Once we finishing replacing more uses of GetTopMostPopupModal()+IsWindowWithinBeginStackOf()\n and FindBlockingModal() with this, we may want to change the flag to be opt-out instead of opt-in.")
        .value("none", ImGuiFocusRequestFlags_None, "")
        .value("restore_focused_child", ImGuiFocusRequestFlags_RestoreFocusedChild, "Find last focused child (if any) and focus it instead.")
        .value("unless_below_modal", ImGuiFocusRequestFlags_UnlessBelowModal, "Do not set focus if the window is below a modal.");


    py::enum_<ImGuiTextFlags_>(m, "TextFlags_", py::arithmetic(), "")
        .value("none", ImGuiTextFlags_None, "")
        .value("no_width_for_large_clipped_text", ImGuiTextFlags_NoWidthForLargeClippedText, "");


    py::enum_<ImGuiTooltipFlags_>(m, "TooltipFlags_", py::arithmetic(), "")
        .value("none", ImGuiTooltipFlags_None, "")
        .value("override_previous", ImGuiTooltipFlags_OverridePrevious, "Clear/ignore previously submitted tooltip (defaults to append)");


    py::enum_<ImGuiLayoutType_>(m, "LayoutType_", py::arithmetic(), " FIXME: this is in development, not exposed/functional as a generic feature yet.\n Horizontal/Vertical enums are fixed to 0/1 so they may be used to index ImVec2")
        .value("horizontal", ImGuiLayoutType_Horizontal, "")
        .value("vertical", ImGuiLayoutType_Vertical, "");


    py::enum_<ImGuiLogType>(m, "LogType", py::arithmetic(), "")
        .value("none", ImGuiLogType_None, "")
        .value("tty", ImGuiLogType_TTY, "")
        .value("file", ImGuiLogType_File, "")
        .value("buffer", ImGuiLogType_Buffer, "")
        .value("clipboard", ImGuiLogType_Clipboard, "");


    py::enum_<ImGuiAxis>(m, "Axis", py::arithmetic(), "X/Y enums are fixed to 0/1 so they may be used to index ImVec2")
        .value("none", ImGuiAxis_None, "")
        .value("x", ImGuiAxis_X, "")
        .value("y", ImGuiAxis_Y, "");


    py::enum_<ImGuiPlotType>(m, "PlotType", py::arithmetic(), "")
        .value("lines", ImGuiPlotType_Lines, "")
        .value("histogram", ImGuiPlotType_Histogram, "");


    auto pyClassImGuiColorMod =
        py::class_<ImGuiColorMod>
            (m, "ColorMod", "Stacked color modifier, backup of modified data so we can restore it")
        .def(py::init<>([](
        ImGuiCol Col = ImGuiCol(), ImVec4 BackupValue = ImVec4())
        {
            auto r = std::make_unique<ImGuiColorMod>();
            r->Col = Col;
            r->BackupValue = BackupValue;
            return r;
        })
        , py::arg("col") = ImGuiCol(), py::arg("backup_value") = ImVec4()
        )
        .def_readwrite("col", &ImGuiColorMod::Col, "")
        .def_readwrite("backup_value", &ImGuiColorMod::BackupValue, "")
        ;


    auto pyClassImGuiStyleMod =
        py::class_<ImGuiStyleMod>
            (m, "StyleMod", "Stacked style modifier, backup of modified data so we can restore it. Data type inferred from the variable.")
        .def_readwrite("var_idx", &ImGuiStyleMod::VarIdx, "")
        .def(py::init<ImGuiStyleVar, int>(),
            py::arg("idx"), py::arg("v"))
        .def(py::init<ImGuiStyleVar, float>(),
            py::arg("idx"), py::arg("v"))
        .def(py::init<ImGuiStyleVar, ImVec2>(),
            py::arg("idx"), py::arg("v"))
        ;


    auto pyClassImGuiComboPreviewData =
        py::class_<ImGuiComboPreviewData>
            (m, "ComboPreviewData", "Storage data for BeginComboPreview()/EndComboPreview()")
        .def_readwrite("preview_rect", &ImGuiComboPreviewData::PreviewRect, "")
        .def_readwrite("backup_cursor_pos", &ImGuiComboPreviewData::BackupCursorPos, "")
        .def_readwrite("backup_cursor_max_pos", &ImGuiComboPreviewData::BackupCursorMaxPos, "")
        .def_readwrite("backup_cursor_pos_prev_line", &ImGuiComboPreviewData::BackupCursorPosPrevLine, "")
        .def_readwrite("backup_prev_line_text_base_offset", &ImGuiComboPreviewData::BackupPrevLineTextBaseOffset, "")
        .def_readwrite("backup_layout", &ImGuiComboPreviewData::BackupLayout, "")
        .def(py::init<>())
        ;


    auto pyClassImGuiGroupData =
        py::class_<ImGuiGroupData>
            (m, "GroupData", "Stacked storage data for BeginGroup()/EndGroup()")
        .def(py::init<>([](
        ImGuiID WindowID = ImGuiID(), ImVec2 BackupCursorPos = ImVec2(), ImVec2 BackupCursorMaxPos = ImVec2(), ImVec2 BackupCursorPosPrevLine = ImVec2(), ImVec1 BackupIndent = ImVec1(), ImVec1 BackupGroupOffset = ImVec1(), ImVec2 BackupCurrLineSize = ImVec2(), float BackupCurrLineTextBaseOffset = float(), ImGuiID BackupActiveIdIsAlive = ImGuiID(), bool BackupActiveIdPreviousFrameIsAlive = bool(), bool BackupHoveredIdIsAlive = bool(), bool BackupIsSameLine = bool(), bool EmitItem = bool())
        {
            auto r = std::make_unique<ImGuiGroupData>();
            r->WindowID = WindowID;
            r->BackupCursorPos = BackupCursorPos;
            r->BackupCursorMaxPos = BackupCursorMaxPos;
            r->BackupCursorPosPrevLine = BackupCursorPosPrevLine;
            r->BackupIndent = BackupIndent;
            r->BackupGroupOffset = BackupGroupOffset;
            r->BackupCurrLineSize = BackupCurrLineSize;
            r->BackupCurrLineTextBaseOffset = BackupCurrLineTextBaseOffset;
            r->BackupActiveIdIsAlive = BackupActiveIdIsAlive;
            r->BackupActiveIdPreviousFrameIsAlive = BackupActiveIdPreviousFrameIsAlive;
            r->BackupHoveredIdIsAlive = BackupHoveredIdIsAlive;
            r->BackupIsSameLine = BackupIsSameLine;
            r->EmitItem = EmitItem;
            return r;
        })
        , py::arg("window_id") = ImGuiID(), py::arg("backup_cursor_pos") = ImVec2(), py::arg("backup_cursor_max_pos") = ImVec2(), py::arg("backup_cursor_pos_prev_line") = ImVec2(), py::arg("backup_indent") = ImVec1(), py::arg("backup_group_offset") = ImVec1(), py::arg("backup_curr_line_size") = ImVec2(), py::arg("backup_curr_line_text_base_offset") = float(), py::arg("backup_active_id_is_alive") = ImGuiID(), py::arg("backup_active_id_previous_frame_is_alive") = bool(), py::arg("backup_hovered_id_is_alive") = bool(), py::arg("backup_is_same_line") = bool(), py::arg("emit_item") = bool()
        )
        .def_readwrite("window_id", &ImGuiGroupData::WindowID, "")
        .def_readwrite("backup_cursor_pos", &ImGuiGroupData::BackupCursorPos, "")
        .def_readwrite("backup_cursor_max_pos", &ImGuiGroupData::BackupCursorMaxPos, "")
        .def_readwrite("backup_cursor_pos_prev_line", &ImGuiGroupData::BackupCursorPosPrevLine, "")
        .def_readwrite("backup_indent", &ImGuiGroupData::BackupIndent, "")
        .def_readwrite("backup_group_offset", &ImGuiGroupData::BackupGroupOffset, "")
        .def_readwrite("backup_curr_line_size", &ImGuiGroupData::BackupCurrLineSize, "")
        .def_readwrite("backup_curr_line_text_base_offset", &ImGuiGroupData::BackupCurrLineTextBaseOffset, "")
        .def_readwrite("backup_active_id_is_alive", &ImGuiGroupData::BackupActiveIdIsAlive, "")
        .def_readwrite("backup_active_id_previous_frame_is_alive", &ImGuiGroupData::BackupActiveIdPreviousFrameIsAlive, "")
        .def_readwrite("backup_hovered_id_is_alive", &ImGuiGroupData::BackupHoveredIdIsAlive, "")
        .def_readwrite("backup_is_same_line", &ImGuiGroupData::BackupIsSameLine, "")
        .def_readwrite("emit_item", &ImGuiGroupData::EmitItem, "")
        ;


    auto pyClassImGuiMenuColumns =
        py::class_<ImGuiMenuColumns>
            (m, "MenuColumns", "Simple column measurement, currently used for MenuItem() only.. This is very short-sighted/throw-away code and NOT a generic helper.")
        .def_readwrite("total_width", &ImGuiMenuColumns::TotalWidth, "")
        .def_readwrite("next_total_width", &ImGuiMenuColumns::NextTotalWidth, "")
        .def_readwrite("spacing", &ImGuiMenuColumns::Spacing, "")
        .def_readwrite("offset_icon", &ImGuiMenuColumns::OffsetIcon, "Always zero for now")
        .def_readwrite("offset_label", &ImGuiMenuColumns::OffsetLabel, "Offsets are locked in Update()")
        .def_readwrite("offset_shortcut", &ImGuiMenuColumns::OffsetShortcut, "")
        .def_readwrite("offset_mark", &ImGuiMenuColumns::OffsetMark, "")
        .def_property("widths",
            [](ImGuiMenuColumns &self) -> pybind11::array
            {
                auto dtype = pybind11::dtype(pybind11::format_descriptor<ImU16>::format());
                auto base = pybind11::array(dtype, {4}, {sizeof(ImU16)});
                return pybind11::array(dtype, {4}, {sizeof(ImU16)}, self.Widths, base);
            }, [](ImGuiMenuColumns& self) {},
            "Width of:   Icon, Label, Shortcut, Mark  (accumulators for current frame)")
        .def(py::init<>())
        .def("update",
            &ImGuiMenuColumns::Update,
            py::arg("spacing"), py::arg("window_reappearing"),
            "(private API)")
        .def("decl_columns",
            &ImGuiMenuColumns::DeclColumns,
            py::arg("w_icon"), py::arg("w_label"), py::arg("w_shortcut"), py::arg("w_mark"),
            "(private API)")
        .def("calc_next_total_width",
            &ImGuiMenuColumns::CalcNextTotalWidth,
            py::arg("update_offsets"),
            "(private API)")
        ;


    auto pyClassImGuiInputTextDeactivatedState =
        py::class_<ImGuiInputTextDeactivatedState>
            (m, "InputTextDeactivatedState", "Internal temporary state for deactivating InputText() instances.")
        .def_readwrite("id_", &ImGuiInputTextDeactivatedState::ID, "widget id owning the text state (which just got deactivated)")
        .def_readwrite("text_a", &ImGuiInputTextDeactivatedState::TextA, "text buffer")
        .def(py::init<>())
        .def("clear_free_memory",
            &ImGuiInputTextDeactivatedState::ClearFreeMemory, "(private API)")
        ;


    auto pyClassImGuiInputTextState =
        py::class_<ImGuiInputTextState>
            (m, "InputTextState", " Internal state of the currently focused/edited text input box\n For a given item ID, access with ImGui::GetInputTextState()")
        .def_readwrite("ctx", &ImGuiInputTextState::Ctx, "parent UI context (needs to be set explicitly by parent).")
        .def_readwrite("id_", &ImGuiInputTextState::ID, "widget id owning the text state")
        .def_readwrite("cur_len_w", &ImGuiInputTextState::CurLenW, "we need to maintain our buffer length in both UTF-8 and wchar format. UTF-8 length is valid even if TextA is not.")
        .def_readwrite("cur_len_a", &ImGuiInputTextState::CurLenA, "we need to maintain our buffer length in both UTF-8 and wchar format. UTF-8 length is valid even if TextA is not.")
        .def_readwrite("text_w", &ImGuiInputTextState::TextW, "edit buffer, we need to persist but can't guarantee the persistence of the user-provided buffer. so we copy into own buffer.")
        .def_readwrite("text_a", &ImGuiInputTextState::TextA, "temporary UTF8 buffer for callbacks and other operations. this is not updated in every code-path! size=capacity.")
        .def_readwrite("initial_text_a", &ImGuiInputTextState::InitialTextA, "value to revert to when pressing Escape = backup of end-user buffer at the time of focus (in UTF-8, unaltered)")
        .def_readwrite("text_a_is_valid", &ImGuiInputTextState::TextAIsValid, "temporary UTF8 buffer is not initially valid before we make the widget active (until then we pull the data from user argument)")
        .def_readwrite("buf_capacity_a", &ImGuiInputTextState::BufCapacityA, "end-user buffer capacity")
        .def_readwrite("scroll_x", &ImGuiInputTextState::ScrollX, "horizontal scrolling/offset")
        .def_readwrite("cursor_anim", &ImGuiInputTextState::CursorAnim, "timer for cursor blink, reset on every user action so the cursor reappears immediately")
        .def_readwrite("cursor_follow", &ImGuiInputTextState::CursorFollow, "set when we want scrolling to follow the current cursor position (not always!)")
        .def_readwrite("selected_all_mouse_lock", &ImGuiInputTextState::SelectedAllMouseLock, "after a double-click to select all, we ignore further mouse drags to update selection")
        .def_readwrite("edited", &ImGuiInputTextState::Edited, "edited this frame")
        .def_readwrite("flags", &ImGuiInputTextState::Flags, "copy of InputText() flags. may be used to check if e.g. ImGuiInputTextFlags_Password is set.")
        .def_readwrite("reload_user_buf", &ImGuiInputTextState::ReloadUserBuf, "force a reload of user buf so it may be modified externally. may be automatic in future version.")
        .def_readwrite("reload_selection_start", &ImGuiInputTextState::ReloadSelectionStart, "POSITIONS ARE IN IMWCHAR units *NOT* UTF-8 this is why this is not exposed yet.")
        .def_readwrite("reload_selection_end", &ImGuiInputTextState::ReloadSelectionEnd, "")
        .def(py::init<>())
        .def("clear_text",
            &ImGuiInputTextState::ClearText, "(private API)")
        .def("clear_free_memory",
            &ImGuiInputTextState::ClearFreeMemory, "(private API)")
        .def("get_undo_avail_count",
            &ImGuiInputTextState::GetUndoAvailCount, "(private API)")
        .def("get_redo_avail_count",
            &ImGuiInputTextState::GetRedoAvailCount, "(private API)")
        .def("on_key_pressed",
            &ImGuiInputTextState::OnKeyPressed,
            py::arg("key"),
            "(private API)\n\n Cannot be inline because we call in code in stb_textedit.h implementation")
        .def("cursor_anim_reset",
            &ImGuiInputTextState::CursorAnimReset, "(private API)\n\n After a user-input the cursor stays on for a while without blinking")
        .def("cursor_clamp",
            &ImGuiInputTextState::CursorClamp, "(private API)")
        .def("has_selection",
            &ImGuiInputTextState::HasSelection, "(private API)")
        .def("clear_selection",
            &ImGuiInputTextState::ClearSelection, "(private API)")
        .def("get_cursor_pos",
            &ImGuiInputTextState::GetCursorPos, "(private API)")
        .def("get_selection_start",
            &ImGuiInputTextState::GetSelectionStart, "(private API)")
        .def("get_selection_end",
            &ImGuiInputTextState::GetSelectionEnd, "(private API)")
        .def("select_all",
            &ImGuiInputTextState::SelectAll, "(private API)")
        .def("reload_user_buf_and_select_all",
            &ImGuiInputTextState::ReloadUserBufAndSelectAll, "(private API)")
        .def("reload_user_buf_and_keep_selection",
            &ImGuiInputTextState::ReloadUserBufAndKeepSelection, "(private API)")
        .def("reload_user_buf_and_move_to_end",
            &ImGuiInputTextState::ReloadUserBufAndMoveToEnd, "(private API)")
        ;


    py::enum_<ImGuiNextWindowDataFlags_>(m, "NextWindowDataFlags_", py::arithmetic(), "")
        .value("none", ImGuiNextWindowDataFlags_None, "")
        .value("has_pos", ImGuiNextWindowDataFlags_HasPos, "")
        .value("has_size", ImGuiNextWindowDataFlags_HasSize, "")
        .value("has_content_size", ImGuiNextWindowDataFlags_HasContentSize, "")
        .value("has_collapsed", ImGuiNextWindowDataFlags_HasCollapsed, "")
        .value("has_size_constraint", ImGuiNextWindowDataFlags_HasSizeConstraint, "")
        .value("has_focus", ImGuiNextWindowDataFlags_HasFocus, "")
        .value("has_bg_alpha", ImGuiNextWindowDataFlags_HasBgAlpha, "")
        .value("has_scroll", ImGuiNextWindowDataFlags_HasScroll, "")
        .value("has_child_flags", ImGuiNextWindowDataFlags_HasChildFlags, "")
        .value("has_viewport", ImGuiNextWindowDataFlags_HasViewport, "")
        .value("has_dock", ImGuiNextWindowDataFlags_HasDock, "")
        .value("has_window_class", ImGuiNextWindowDataFlags_HasWindowClass, "");


    auto pyClassImGuiNextWindowData =
        py::class_<ImGuiNextWindowData>
            (m, "NextWindowData", "Storage for SetNexWindow** functions")
        .def_readwrite("flags", &ImGuiNextWindowData::Flags, "")
        .def_readwrite("pos_cond", &ImGuiNextWindowData::PosCond, "")
        .def_readwrite("size_cond", &ImGuiNextWindowData::SizeCond, "")
        .def_readwrite("collapsed_cond", &ImGuiNextWindowData::CollapsedCond, "")
        .def_readwrite("dock_cond", &ImGuiNextWindowData::DockCond, "")
        .def_readwrite("pos_val", &ImGuiNextWindowData::PosVal, "")
        .def_readwrite("pos_pivot_val", &ImGuiNextWindowData::PosPivotVal, "")
        .def_readwrite("size_val", &ImGuiNextWindowData::SizeVal, "")
        .def_readwrite("content_size_val", &ImGuiNextWindowData::ContentSizeVal, "")
        .def_readwrite("scroll_val", &ImGuiNextWindowData::ScrollVal, "")
        .def_readwrite("child_flags", &ImGuiNextWindowData::ChildFlags, "")
        .def_readwrite("pos_undock", &ImGuiNextWindowData::PosUndock, "")
        .def_readwrite("collapsed_val", &ImGuiNextWindowData::CollapsedVal, "")
        .def_readwrite("size_constraint_rect", &ImGuiNextWindowData::SizeConstraintRect, "")
        .def_readwrite("size_callback", &ImGuiNextWindowData::SizeCallback, "")
        .def_readwrite("size_callback_user_data", &ImGuiNextWindowData::SizeCallbackUserData, "")
        .def_readwrite("bg_alpha_val", &ImGuiNextWindowData::BgAlphaVal, "Override background alpha")
        .def_readwrite("viewport_id", &ImGuiNextWindowData::ViewportId, "")
        .def_readwrite("dock_id", &ImGuiNextWindowData::DockId, "")
        .def_readwrite("window_class", &ImGuiNextWindowData::WindowClass, "")
        .def_readwrite("menu_bar_offset_min_val", &ImGuiNextWindowData::MenuBarOffsetMinVal, "(Always on) This is not exposed publicly, so we don't clear it and it doesn't have a corresponding flag (could we? for consistency?)")
        .def(py::init<>())
        .def("clear_flags",
            &ImGuiNextWindowData::ClearFlags, "(private API)")
        ;


    py::enum_<ImGuiNextItemDataFlags_>(m, "NextItemDataFlags_", py::arithmetic(), "")
        .value("none", ImGuiNextItemDataFlags_None, "")
        .value("has_width", ImGuiNextItemDataFlags_HasWidth, "")
        .value("has_open", ImGuiNextItemDataFlags_HasOpen, "")
        .value("has_shortcut", ImGuiNextItemDataFlags_HasShortcut, "");


    auto pyClassImGuiNextItemData =
        py::class_<ImGuiNextItemData>
            (m, "NextItemData", "")
        .def_readwrite("flags", &ImGuiNextItemData::Flags, "")
        .def_readwrite("item_flags", &ImGuiNextItemData::ItemFlags, "Currently only tested/used for ImGuiItemFlags_AllowOverlap.")
        .def_readwrite("selection_user_data", &ImGuiNextItemData::SelectionUserData, "Set by SetNextItemSelectionUserData() (note that None/0 is a valid value, we use -1 == ImGuiSelectionUserData_Invalid to mark invalid values)")
        .def_readwrite("width", &ImGuiNextItemData::Width, "Set by SetNextItemWidth()")
        .def_readwrite("shortcut", &ImGuiNextItemData::Shortcut, "Set by SetNextItemShortcut()")
        .def_readwrite("open_val", &ImGuiNextItemData::OpenVal, "Set by SetNextItemOpen()")
        .def(py::init<>())
        .def("clear_flags",
            &ImGuiNextItemData::ClearFlags, "(private API)\n\n Also cleared manually by ItemAdd()!")
        ;


    auto pyClassImGuiLastItemData =
        py::class_<ImGuiLastItemData>
            (m, "LastItemData", "Status storage for the last submitted item")
        .def_readwrite("id_", &ImGuiLastItemData::ID, "")
        .def_readwrite("in_flags", &ImGuiLastItemData::InFlags, "See ImGuiItemFlags_")
        .def_readwrite("status_flags", &ImGuiLastItemData::StatusFlags, "See ImGuiItemStatusFlags_")
        .def_readwrite("rect", &ImGuiLastItemData::Rect, "Full rectangle")
        .def_readwrite("nav_rect", &ImGuiLastItemData::NavRect, "Navigation scoring rectangle (not displayed)")
        .def_readwrite("display_rect", &ImGuiLastItemData::DisplayRect, "Display rectangle (ONLY VALID IF ImGuiItemStatusFlags_HasDisplayRect is set)")
        .def_readwrite("clip_rect", &ImGuiLastItemData::ClipRect, "Clip rectangle at the time of submitting item (ONLY VALID IF ImGuiItemStatusFlags_HasClipRect is set)")
        .def(py::init<>())
        ;


    auto pyClassImGuiNavTreeNodeData =
        py::class_<ImGuiNavTreeNodeData>
            (m, "NavTreeNodeData", " Store data emitted by TreeNode() for usage by TreePop() to implement ImGuiTreeNodeFlags_NavLeftJumpsBackHere.\n This is the minimum amount of data that we need to perform the equivalent of NavApplyItemToResult() and which we can't infer in TreePop()\n Only stored when the node is a potential candidate for landing on a Left arrow jump.")
        .def(py::init<>([](
        ImGuiID ID = ImGuiID(), ImGuiItemFlags InFlags = ImGuiItemFlags(), ImRect NavRect = ImRect())
        {
            auto r = std::make_unique<ImGuiNavTreeNodeData>();
            r->ID = ID;
            r->InFlags = InFlags;
            r->NavRect = NavRect;
            return r;
        })
        , py::arg("id_") = ImGuiID(), py::arg("in_flags") = ImGuiItemFlags(), py::arg("nav_rect") = ImRect()
        )
        .def_readwrite("id_", &ImGuiNavTreeNodeData::ID, "")
        .def_readwrite("in_flags", &ImGuiNavTreeNodeData::InFlags, "")
        .def_readwrite("nav_rect", &ImGuiNavTreeNodeData::NavRect, "")
        ;


    auto pyClassImGuiStackSizes =
        py::class_<ImGuiStackSizes>
            (m, "StackSizes", "")
        .def_readwrite("size_of_id_stack", &ImGuiStackSizes::SizeOfIDStack, "")
        .def_readwrite("size_of_color_stack", &ImGuiStackSizes::SizeOfColorStack, "")
        .def_readwrite("size_of_style_var_stack", &ImGuiStackSizes::SizeOfStyleVarStack, "")
        .def_readwrite("size_of_font_stack", &ImGuiStackSizes::SizeOfFontStack, "")
        .def_readwrite("size_of_focus_scope_stack", &ImGuiStackSizes::SizeOfFocusScopeStack, "")
        .def_readwrite("size_of_group_stack", &ImGuiStackSizes::SizeOfGroupStack, "")
        .def_readwrite("size_of_item_flags_stack", &ImGuiStackSizes::SizeOfItemFlagsStack, "")
        .def_readwrite("size_of_begin_popup_stack", &ImGuiStackSizes::SizeOfBeginPopupStack, "")
        .def_readwrite("size_of_disabled_stack", &ImGuiStackSizes::SizeOfDisabledStack, "")
        .def(py::init<>())
        .def("set_to_context_state",
            &ImGuiStackSizes::SetToContextState,
            py::arg("ctx"),
            "(private API)")
        .def("compare_with_context_state",
            &ImGuiStackSizes::CompareWithContextState,
            py::arg("ctx"),
            "(private API)")
        ;


    auto pyClassImGuiWindowStackData =
        py::class_<ImGuiWindowStackData>
            (m, "WindowStackData", "Data saved for each window pushed into the stack")
        .def(py::init<>([](
        ImGuiLastItemData ParentLastItemDataBackup = ImGuiLastItemData(), ImGuiStackSizes StackSizesOnBegin = ImGuiStackSizes())
        {
            auto r = std::make_unique<ImGuiWindowStackData>();
            r->ParentLastItemDataBackup = ParentLastItemDataBackup;
            r->StackSizesOnBegin = StackSizesOnBegin;
            return r;
        })
        , py::arg("parent_last_item_data_backup") = ImGuiLastItemData(), py::arg("stack_sizes_on_begin") = ImGuiStackSizes()
        )
        .def_readwrite("window", &ImGuiWindowStackData::Window, "")
        .def_readwrite("parent_last_item_data_backup", &ImGuiWindowStackData::ParentLastItemDataBackup, "")
        .def_readwrite("stack_sizes_on_begin", &ImGuiWindowStackData::StackSizesOnBegin, "Store size of various stacks for asserting")
        ;


    auto pyClassImGuiShrinkWidthItem =
        py::class_<ImGuiShrinkWidthItem>
            (m, "ShrinkWidthItem", "")
        .def(py::init<>([](
        int Index = int(), float Width = float(), float InitialWidth = float())
        {
            auto r = std::make_unique<ImGuiShrinkWidthItem>();
            r->Index = Index;
            r->Width = Width;
            r->InitialWidth = InitialWidth;
            return r;
        })
        , py::arg("index") = int(), py::arg("width") = float(), py::arg("initial_width") = float()
        )
        .def_readwrite("index", &ImGuiShrinkWidthItem::Index, "")
        .def_readwrite("width", &ImGuiShrinkWidthItem::Width, "")
        .def_readwrite("initial_width", &ImGuiShrinkWidthItem::InitialWidth, "")
        ;


    auto pyClassImGuiPtrOrIndex =
        py::class_<ImGuiPtrOrIndex>
            (m, "PtrOrIndex", "")
        .def_readwrite("ptr", &ImGuiPtrOrIndex::Ptr, "Either field can be set, not both. e.g. Dock node tab bars are loose while BeginTabBar() ones are in a pool.")
        .def_readwrite("index", &ImGuiPtrOrIndex::Index, "Usually index in a main pool.")
        .def(py::init<void *>(),
            py::arg("ptr"))
        .def(py::init<int>(),
            py::arg("index"))
        ;


    auto pyClassImGuiDataVarInfo =
        py::class_<ImGuiDataVarInfo>
            (m, "DataVarInfo", "")
        .def(py::init<>([](
        ImGuiDataType Type = ImGuiDataType(), ImU32 Count = ImU32(), ImU32 Offset = ImU32())
        {
            auto r = std::make_unique<ImGuiDataVarInfo>();
            r->Type = Type;
            r->Count = Count;
            r->Offset = Offset;
            return r;
        })
        , py::arg("type") = ImGuiDataType(), py::arg("count") = ImU32(), py::arg("offset") = ImU32()
        )
        .def_readwrite("type", &ImGuiDataVarInfo::Type, "")
        .def_readwrite("count", &ImGuiDataVarInfo::Count, "1+")
        .def_readwrite("offset", &ImGuiDataVarInfo::Offset, "Offset in parent structure")
        .def("get_var_ptr",
            &ImGuiDataVarInfo::GetVarPtr,
            py::arg("parent"),
            "(private API)",
            pybind11::return_value_policy::reference)
        ;


    auto pyClassImGuiDataTypeTempStorage =
        py::class_<ImGuiDataTypeTempStorage>
            (m, "DataTypeTempStorage", "")
        .def(py::init<>()) // implicit default constructor
        .def_property("data",
            [](ImGuiDataTypeTempStorage &self) -> pybind11::array
            {
                auto dtype = pybind11::dtype(pybind11::format_descriptor<ImU8>::format());
                auto base = pybind11::array(dtype, {8}, {sizeof(ImU8)});
                return pybind11::array(dtype, {8}, {sizeof(ImU8)}, self.Data, base);
            }, [](ImGuiDataTypeTempStorage& self) {},
            "Can fit any data up to ImGuiDataType_COUNT")
        ;


    auto pyClassImGuiDataTypeInfo =
        py::class_<ImGuiDataTypeInfo>
            (m, "DataTypeInfo", "Type information associated to one ImGuiDataType. Retrieve with DataTypeGetInfo().")
        .def(py::init<>([](
        size_t Size = size_t())
        {
            auto r = std::make_unique<ImGuiDataTypeInfo>();
            r->Size = Size;
            return r;
        })
        , py::arg("size") = size_t()
        )
        .def_readwrite("size", &ImGuiDataTypeInfo::Size, "Size in bytes")
        .def_readonly("name", &ImGuiDataTypeInfo::Name, "Short descriptive name for the type, for debugging")
        .def_readonly("print_fmt", &ImGuiDataTypeInfo::PrintFmt, "Default printf format for the type")
        .def_readonly("scan_fmt", &ImGuiDataTypeInfo::ScanFmt, "Default scanf format for the type")
        ;


    py::enum_<ImGuiDataTypePrivate_>(m, "DataTypePrivate_", py::arithmetic(), "Extend ImGuiDataType_")
        .value("im_gui_data_type_string", ImGuiDataType_String, "")
        .value("im_gui_data_type_pointer", ImGuiDataType_Pointer, "")
        .value("im_gui_data_type_id", ImGuiDataType_ID, "");


    py::enum_<ImGuiPopupPositionPolicy>(m, "PopupPositionPolicy", py::arithmetic(), "")
        .value("default", ImGuiPopupPositionPolicy_Default, "")
        .value("combo_box", ImGuiPopupPositionPolicy_ComboBox, "")
        .value("tooltip", ImGuiPopupPositionPolicy_Tooltip, "");


    auto pyClassImGuiPopupData =
        py::class_<ImGuiPopupData>
            (m, "PopupData", "Storage for popup stacks (g.OpenPopupStack and g.BeginPopupStack)")
        .def_readwrite("popup_id", &ImGuiPopupData::PopupId, "Set on OpenPopup()")
        .def_readwrite("window", &ImGuiPopupData::Window, "Resolved on BeginPopup() - may stay unresolved if user never calls OpenPopup()")
        .def_readwrite("restore_nav_window", &ImGuiPopupData::RestoreNavWindow, "Set on OpenPopup(), a NavWindow that will be restored on popup close")
        .def_readwrite("parent_nav_layer", &ImGuiPopupData::ParentNavLayer, "Resolved on BeginPopup(). Actually a ImGuiNavLayer type (declared down below), initialized to -1 which is not part of an enum, but serves well-enough as \"not any of layers\" value")
        .def_readwrite("open_frame_count", &ImGuiPopupData::OpenFrameCount, "Set on OpenPopup()")
        .def_readwrite("open_parent_id", &ImGuiPopupData::OpenParentId, "Set on OpenPopup(), we need this to differentiate multiple menu sets from each others (e.g. inside menu bar vs loose menu items)")
        .def_readwrite("open_popup_pos", &ImGuiPopupData::OpenPopupPos, "Set on OpenPopup(), preferred popup position (typically == OpenMousePos when using mouse)")
        .def_readwrite("open_mouse_pos", &ImGuiPopupData::OpenMousePos, "Set on OpenPopup(), copy of mouse position at the time of opening popup")
        .def(py::init<>())
        ;


    py::enum_<ImGuiInputEventType>(m, "InputEventType", py::arithmetic(), "")
        .value("none", ImGuiInputEventType_None, "")
        .value("mouse_pos", ImGuiInputEventType_MousePos, "")
        .value("mouse_wheel", ImGuiInputEventType_MouseWheel, "")
        .value("mouse_button", ImGuiInputEventType_MouseButton, "")
        .value("mouse_viewport", ImGuiInputEventType_MouseViewport, "")
        .value("key", ImGuiInputEventType_Key, "")
        .value("text", ImGuiInputEventType_Text, "")
        .value("focus", ImGuiInputEventType_Focus, "")
        .value("count", ImGuiInputEventType_COUNT, "");


    py::enum_<ImGuiInputSource>(m, "InputSource", py::arithmetic(), "")
        .value("none", ImGuiInputSource_None, "")
        .value("mouse", ImGuiInputSource_Mouse, "Note: may be Mouse or TouchScreen or Pen. See io.MouseSource to distinguish them.")
        .value("keyboard", ImGuiInputSource_Keyboard, "")
        .value("gamepad", ImGuiInputSource_Gamepad, "")
        .value("count", ImGuiInputSource_COUNT, "");


    auto pyClassImGuiInputEventMousePos =
        py::class_<ImGuiInputEventMousePos>
            (m, "InputEventMousePos", "")
        .def(py::init<>([](
        float PosX = float(), float PosY = float(), ImGuiMouseSource MouseSource = ImGuiMouseSource())
        {
            auto r = std::make_unique<ImGuiInputEventMousePos>();
            r->PosX = PosX;
            r->PosY = PosY;
            r->MouseSource = MouseSource;
            return r;
        })
        , py::arg("pos_x") = float(), py::arg("pos_y") = float(), py::arg("mouse_source") = ImGuiMouseSource()
        )
        .def_readwrite("pos_x", &ImGuiInputEventMousePos::PosX, "")
        .def_readwrite("pos_y", &ImGuiInputEventMousePos::PosY, "")
        .def_readwrite("mouse_source", &ImGuiInputEventMousePos::MouseSource, "")
        ;


    auto pyClassImGuiInputEventMouseWheel =
        py::class_<ImGuiInputEventMouseWheel>
            (m, "InputEventMouseWheel", "")
        .def(py::init<>([](
        float WheelX = float(), float WheelY = float(), ImGuiMouseSource MouseSource = ImGuiMouseSource())
        {
            auto r = std::make_unique<ImGuiInputEventMouseWheel>();
            r->WheelX = WheelX;
            r->WheelY = WheelY;
            r->MouseSource = MouseSource;
            return r;
        })
        , py::arg("wheel_x") = float(), py::arg("wheel_y") = float(), py::arg("mouse_source") = ImGuiMouseSource()
        )
        .def_readwrite("wheel_x", &ImGuiInputEventMouseWheel::WheelX, "")
        .def_readwrite("wheel_y", &ImGuiInputEventMouseWheel::WheelY, "")
        .def_readwrite("mouse_source", &ImGuiInputEventMouseWheel::MouseSource, "")
        ;


    auto pyClassImGuiInputEventMouseButton =
        py::class_<ImGuiInputEventMouseButton>
            (m, "InputEventMouseButton", "")
        .def(py::init<>([](
        int Button = int(), bool Down = bool(), ImGuiMouseSource MouseSource = ImGuiMouseSource())
        {
            auto r = std::make_unique<ImGuiInputEventMouseButton>();
            r->Button = Button;
            r->Down = Down;
            r->MouseSource = MouseSource;
            return r;
        })
        , py::arg("button") = int(), py::arg("down") = bool(), py::arg("mouse_source") = ImGuiMouseSource()
        )
        .def_readwrite("button", &ImGuiInputEventMouseButton::Button, "")
        .def_readwrite("down", &ImGuiInputEventMouseButton::Down, "")
        .def_readwrite("mouse_source", &ImGuiInputEventMouseButton::MouseSource, "")
        ;


    auto pyClassImGuiInputEventMouseViewport =
        py::class_<ImGuiInputEventMouseViewport>
            (m, "InputEventMouseViewport", "")
        .def(py::init<>([](
        ImGuiID HoveredViewportID = ImGuiID())
        {
            auto r = std::make_unique<ImGuiInputEventMouseViewport>();
            r->HoveredViewportID = HoveredViewportID;
            return r;
        })
        , py::arg("hovered_viewport_id") = ImGuiID()
        )
        .def_readwrite("hovered_viewport_id", &ImGuiInputEventMouseViewport::HoveredViewportID, "")
        ;


    auto pyClassImGuiInputEventKey =
        py::class_<ImGuiInputEventKey>
            (m, "InputEventKey", "")
        .def(py::init<>([](
        ImGuiKey Key = ImGuiKey(), bool Down = bool(), float AnalogValue = float())
        {
            auto r = std::make_unique<ImGuiInputEventKey>();
            r->Key = Key;
            r->Down = Down;
            r->AnalogValue = AnalogValue;
            return r;
        })
        , py::arg("key") = ImGuiKey(), py::arg("down") = bool(), py::arg("analog_value") = float()
        )
        .def_readwrite("key", &ImGuiInputEventKey::Key, "")
        .def_readwrite("down", &ImGuiInputEventKey::Down, "")
        .def_readwrite("analog_value", &ImGuiInputEventKey::AnalogValue, "")
        ;


    auto pyClassImGuiInputEventText =
        py::class_<ImGuiInputEventText>
            (m, "InputEventText", "")
        .def(py::init<>()) // implicit default constructor
        .def_readwrite("char", &ImGuiInputEventText::Char, "")
        ;


    auto pyClassImGuiInputEventAppFocused =
        py::class_<ImGuiInputEventAppFocused>
            (m, "InputEventAppFocused", "")
        .def(py::init<>([](
        bool Focused = bool())
        {
            auto r = std::make_unique<ImGuiInputEventAppFocused>();
            r->Focused = Focused;
            return r;
        })
        , py::arg("focused") = bool()
        )
        .def_readwrite("focused", &ImGuiInputEventAppFocused::Focused, "")
        ;


    auto pyClassImGuiInputEvent =
        py::class_<ImGuiInputEvent>
            (m, "InputEvent", "")
        .def_readwrite("type", &ImGuiInputEvent::Type, "")
        .def_readwrite("source", &ImGuiInputEvent::Source, "")
        .def_readwrite("event_id", &ImGuiInputEvent::EventId, "Unique, sequential increasing integer to identify an event (if you need to correlate them to other data).")
        .def_readwrite("added_by_test_engine", &ImGuiInputEvent::AddedByTestEngine, "")
        .def(py::init<>())
        ;


    auto pyClassImGuiKeyRoutingData =
        py::class_<ImGuiKeyRoutingData>
            (m, "KeyRoutingData", "Routing table entry (sizeof() == 16 bytes)")
        .def_readwrite("next_entry_index", &ImGuiKeyRoutingData::NextEntryIndex, "")
        .def_readwrite("mods", &ImGuiKeyRoutingData::Mods, "Technically we'd only need 4-bits but for simplify we store ImGuiMod_ values which need 16-bits. ImGuiMod_Shortcut is already translated to Ctrl/Super.")
        .def_readwrite("routing_curr_score", &ImGuiKeyRoutingData::RoutingCurrScore, "[DEBUG] For debug display")
        .def_readwrite("routing_next_score", &ImGuiKeyRoutingData::RoutingNextScore, "Lower is better (0: perfect score)")
        .def_readwrite("routing_curr", &ImGuiKeyRoutingData::RoutingCurr, "")
        .def_readwrite("routing_next", &ImGuiKeyRoutingData::RoutingNext, "")
        .def(py::init<>())
        ;


    auto pyClassImGuiKeyRoutingTable =
        py::class_<ImGuiKeyRoutingTable>
            (m, "KeyRoutingTable", " Routing table: maintain a desired owner for each possible key-chord (key + mods), and setup owner in NewFrame() when mods are matching.\n Stored in main context (1 instance)")
        .def_readwrite("entries", &ImGuiKeyRoutingTable::Entries, "")
        .def_readwrite("entries_next", &ImGuiKeyRoutingTable::EntriesNext, "Double-buffer to avoid reallocation (could use a shared buffer)")
        .def(py::init<>())
        .def("clear",
            &ImGuiKeyRoutingTable::Clear, "(private API)")
        ;


    auto pyClassImGuiKeyOwnerData =
        py::class_<ImGuiKeyOwnerData>
            (m, "KeyOwnerData", " This extends ImGuiKeyData but only for named keys (legacy keys don't support the new features)\n Stored in main context (1 per named key). In the future it might be merged into ImGuiKeyData.")
        .def_readwrite("owner_curr", &ImGuiKeyOwnerData::OwnerCurr, "")
        .def_readwrite("owner_next", &ImGuiKeyOwnerData::OwnerNext, "")
        .def_readwrite("lock_this_frame", &ImGuiKeyOwnerData::LockThisFrame, "Reading this key requires explicit owner id (until end of frame). Set by ImGuiInputFlags_LockThisFrame.")
        .def_readwrite("lock_until_release", &ImGuiKeyOwnerData::LockUntilRelease, "Reading this key requires explicit owner id (until key is released). Set by ImGuiInputFlags_LockUntilRelease. When this is True LockThisFrame is always True as well.")
        .def(py::init<>())
        ;


    py::enum_<ImGuiInputFlags_>(m, "InputFlags_", py::arithmetic(), " Flags for extended versions of IsKeyPressed(), IsMouseClicked(), Shortcut(), SetKeyOwner(), SetItemKeyOwner()\n Don't mistake with ImGuiInputTextFlags! (which is for ImGui::InputText() function)")
        .value("none", ImGuiInputFlags_None, "Flags for IsKeyPressed(), IsKeyChordPressed(), IsMouseClicked(), Shortcut()")
        .value("repeat", ImGuiInputFlags_Repeat, "Enable repeat. Return True on successive repeats. Default for legacy IsKeyPressed(). NOT Default for legacy IsMouseClicked(). MUST BE == 1.")
        .value("repeat_rate_default", ImGuiInputFlags_RepeatRateDefault, "Repeat rate: Regular (default)")
        .value("repeat_rate_nav_move", ImGuiInputFlags_RepeatRateNavMove, "Repeat rate: Fast")
        .value("repeat_rate_nav_tweak", ImGuiInputFlags_RepeatRateNavTweak, "Repeat rate: Faster")
        .value("repeat_until_release", ImGuiInputFlags_RepeatUntilRelease, "Stop repeating when released (default for all functions except Shortcut). This only exists to allow overriding Shortcut() default behavior.")
        .value("repeat_until_key_mods_change", ImGuiInputFlags_RepeatUntilKeyModsChange, "Stop repeating when released OR if keyboard mods are changed (default for Shortcut)")
        .value("repeat_until_key_mods_change_from_none", ImGuiInputFlags_RepeatUntilKeyModsChangeFromNone, "Stop repeating when released OR if keyboard mods are leaving the None state. Allows going from Mod+Key to Key by releasing Mod.")
        .value("repeat_until_other_key_press", ImGuiInputFlags_RepeatUntilOtherKeyPress, "Stop repeating when released OR if any other keyboard key is pressed during the repeat")
        .value("cond_hovered", ImGuiInputFlags_CondHovered, "Only set if item is hovered (default to both)")
        .value("cond_active", ImGuiInputFlags_CondActive, "Only set if item is active (default to both)")
        .value("cond_default_", ImGuiInputFlags_CondDefault_, "")
        .value("lock_this_frame", ImGuiInputFlags_LockThisFrame, "Further accesses to key data will require EXPLICIT owner ID (ImGuiKeyOwner_Any/0 will NOT accepted for polling). Cleared at end of frame.")
        .value("lock_until_release", ImGuiInputFlags_LockUntilRelease, "Further accesses to key data will require EXPLICIT owner ID (ImGuiKeyOwner_Any/0 will NOT accepted for polling). Cleared when the key is released or at end of each frame if key is released.")
        .value("route_focused", ImGuiInputFlags_RouteFocused, "(Default) Honor focus route: Accept inputs if window is in focus stack. Deep-most focused window takes inputs. ActiveId takes inputs over deep-most focused window.")
        .value("route_global_low", ImGuiInputFlags_RouteGlobalLow, "Register route globally (lowest priority: unless a focused window or active item registered the route) -> recommended Global priority IF you need a Global priority.")
        .value("route_global", ImGuiInputFlags_RouteGlobal, "Register route globally (medium priority: unless an active item registered the route, e.g. CTRL+A registered by InputText will take priority over this).")
        .value("route_global_high", ImGuiInputFlags_RouteGlobalHigh, "Register route globally (higher priority: unlikely you need to use that: will interfere with every active items, e.g. CTRL+A registered by InputText will be overriden by this)")
        .value("route_always", ImGuiInputFlags_RouteAlways, "Do not register route, poll keys directly.")
        .value("route_unless_bg_focused", ImGuiInputFlags_RouteUnlessBgFocused, "Global routes will not be applied if underlying background/None is focused (== no Dear ImGui windows are focused). Useful for overlay applications.")
        .value("repeat_rate_mask_", ImGuiInputFlags_RepeatRateMask_, "")
        .value("repeat_until_mask_", ImGuiInputFlags_RepeatUntilMask_, "")
        .value("repeat_mask_", ImGuiInputFlags_RepeatMask_, "")
        .value("cond_mask_", ImGuiInputFlags_CondMask_, "")
        .value("route_mask_", ImGuiInputFlags_RouteMask_, "_Always not part of this!")
        .value("supported_by_is_key_pressed", ImGuiInputFlags_SupportedByIsKeyPressed, "")
        .value("supported_by_is_mouse_clicked", ImGuiInputFlags_SupportedByIsMouseClicked, "")
        .value("supported_by_shortcut", ImGuiInputFlags_SupportedByShortcut, "")
        .value("supported_by_set_key_owner", ImGuiInputFlags_SupportedBySetKeyOwner, "")
        .value("supported_by_set_item_key_owner", ImGuiInputFlags_SupportedBySetItemKeyOwner, "");


    auto pyClassImGuiListClipperRange =
        py::class_<ImGuiListClipperRange>
            (m, "ListClipperRange", "Note that Max is exclusive, so perhaps should be using a Begin/End convention.")
        .def(py::init<>([](
        int Min = int(), int Max = int(), bool PosToIndexConvert = bool(), ImS8 PosToIndexOffsetMin = ImS8(), ImS8 PosToIndexOffsetMax = ImS8())
        {
            auto r = std::make_unique<ImGuiListClipperRange>();
            r->Min = Min;
            r->Max = Max;
            r->PosToIndexConvert = PosToIndexConvert;
            r->PosToIndexOffsetMin = PosToIndexOffsetMin;
            r->PosToIndexOffsetMax = PosToIndexOffsetMax;
            return r;
        })
        , py::arg("min") = int(), py::arg("max") = int(), py::arg("pos_to_index_convert") = bool(), py::arg("pos_to_index_offset_min") = ImS8(), py::arg("pos_to_index_offset_max") = ImS8()
        )
        .def_readwrite("min", &ImGuiListClipperRange::Min, "")
        .def_readwrite("max", &ImGuiListClipperRange::Max, "")
        .def_readwrite("pos_to_index_convert", &ImGuiListClipperRange::PosToIndexConvert, "Begin/End are absolute position (will be converted to indices later)")
        .def_readwrite("pos_to_index_offset_min", &ImGuiListClipperRange::PosToIndexOffsetMin, "Add to Min after converting to indices")
        .def_readwrite("pos_to_index_offset_max", &ImGuiListClipperRange::PosToIndexOffsetMax, "Add to Min after converting to indices")
        .def_static("from_indices",
            &ImGuiListClipperRange::FromIndices,
            py::arg("min"), py::arg("max"),
            "(private API)")
        .def_static("from_positions",
            &ImGuiListClipperRange::FromPositions,
            py::arg("y1"), py::arg("y2"), py::arg("off_min"), py::arg("off_max"),
            "(private API)")
        ;


    auto pyClassImGuiListClipperData =
        py::class_<ImGuiListClipperData>
            (m, "ListClipperData", "Temporary clipper data, buffers shared/reused between instances")
        .def_readwrite("list_clipper", &ImGuiListClipperData::ListClipper, "")
        .def_readwrite("lossyness_offset", &ImGuiListClipperData::LossynessOffset, "")
        .def_readwrite("step_no", &ImGuiListClipperData::StepNo, "")
        .def_readwrite("items_frozen", &ImGuiListClipperData::ItemsFrozen, "")
        .def_readwrite("ranges", &ImGuiListClipperData::Ranges, "")
        .def(py::init<>())
        .def("reset",
            &ImGuiListClipperData::Reset,
            py::arg("clipper"),
            "(private API)")
        ;


    py::enum_<ImGuiActivateFlags_>(m, "ActivateFlags_", py::arithmetic(), "")
        .value("none", ImGuiActivateFlags_None, "")
        .value("prefer_input", ImGuiActivateFlags_PreferInput, "Favor activation that requires keyboard text input (e.g. for Slider/Drag). Default for Enter key.")
        .value("prefer_tweak", ImGuiActivateFlags_PreferTweak, "Favor activation for tweaking with arrows or gamepad (e.g. for Slider/Drag). Default for Space key and if keyboard is not used.")
        .value("try_to_preserve_state", ImGuiActivateFlags_TryToPreserveState, "Request widget to preserve state if it can (e.g. InputText will try to preserve cursor/selection)")
        .value("from_tabbing", ImGuiActivateFlags_FromTabbing, "Activation requested by a tabbing request")
        .value("from_shortcut", ImGuiActivateFlags_FromShortcut, "Activation requested by an item shortcut via SetNextItemShortcut() function.");


    py::enum_<ImGuiScrollFlags_>(m, "ScrollFlags_", py::arithmetic(), "Early work-in-progress API for ScrollToItem()")
        .value("none", ImGuiScrollFlags_None, "")
        .value("keep_visible_edge_x", ImGuiScrollFlags_KeepVisibleEdgeX, "If item is not visible: scroll as little as possible on X axis to bring item back into view [default for X axis]")
        .value("keep_visible_edge_y", ImGuiScrollFlags_KeepVisibleEdgeY, "If item is not visible: scroll as little as possible on Y axis to bring item back into view [default for Y axis for windows that are already visible]")
        .value("keep_visible_center_x", ImGuiScrollFlags_KeepVisibleCenterX, "If item is not visible: scroll to make the item centered on X axis [rarely used]")
        .value("keep_visible_center_y", ImGuiScrollFlags_KeepVisibleCenterY, "If item is not visible: scroll to make the item centered on Y axis")
        .value("always_center_x", ImGuiScrollFlags_AlwaysCenterX, "Always center the result item on X axis [rarely used]")
        .value("always_center_y", ImGuiScrollFlags_AlwaysCenterY, "Always center the result item on Y axis [default for Y axis for appearing window)")
        .value("no_scroll_parent", ImGuiScrollFlags_NoScrollParent, "Disable forwarding scrolling to parent window if required to keep item/rect visible (only scroll window the function was applied to).")
        .value("mask_x_", ImGuiScrollFlags_MaskX_, "")
        .value("mask_y_", ImGuiScrollFlags_MaskY_, "");


    py::enum_<ImGuiNavHighlightFlags_>(m, "NavHighlightFlags_", py::arithmetic(), "")
        .value("none", ImGuiNavHighlightFlags_None, "")
        .value("compact", ImGuiNavHighlightFlags_Compact, "Compact highlight, no padding")
        .value("always_draw", ImGuiNavHighlightFlags_AlwaysDraw, "Draw rectangular highlight if (g.NavId == id) _even_ when using the mouse.")
        .value("no_rounding", ImGuiNavHighlightFlags_NoRounding, "");


    py::enum_<ImGuiNavMoveFlags_>(m, "NavMoveFlags_", py::arithmetic(), "")
        .value("none", ImGuiNavMoveFlags_None, "")
        .value("loop_x", ImGuiNavMoveFlags_LoopX, "On failed request, restart from opposite side")
        .value("loop_y", ImGuiNavMoveFlags_LoopY, "")
        .value("wrap_x", ImGuiNavMoveFlags_WrapX, "On failed request, request from opposite side one line down (when NavDir==right) or one line up (when NavDir==left)")
        .value("wrap_y", ImGuiNavMoveFlags_WrapY, "This is not super useful but provided for completeness")
        .value("wrap_mask_", ImGuiNavMoveFlags_WrapMask_, "")
        .value("allow_current_nav_id", ImGuiNavMoveFlags_AllowCurrentNavId, "Allow scoring and considering the current NavId as a move target candidate. This is used when the move source is offset (e.g. pressing PageDown actually needs to send a Up move request, if we are pressing PageDown from the bottom-most item we need to stay in place)")
        .value("also_score_visible_set", ImGuiNavMoveFlags_AlsoScoreVisibleSet, "Store alternate result in NavMoveResultLocalVisible that only comprise elements that are already fully visible (used by PageUp/PageDown)")
        .value("scroll_to_edge_y", ImGuiNavMoveFlags_ScrollToEdgeY, "Force scrolling to min/max (used by Home/End) // FIXME-NAV: Aim to remove or reword, probably unnecessary")
        .value("forwarded", ImGuiNavMoveFlags_Forwarded, "")
        .value("debug_no_result", ImGuiNavMoveFlags_DebugNoResult, "Dummy scoring for debug purpose, don't apply result")
        .value("focus_api", ImGuiNavMoveFlags_FocusApi, "Requests from focus API can land/focus/activate items even if they are marked with _NoTabStop (see NavProcessItemForTabbingRequest() for details)")
        .value("is_tabbing", ImGuiNavMoveFlags_IsTabbing, "== Focus + Activate if item is Inputable + DontChangeNavHighlight")
        .value("is_page_move", ImGuiNavMoveFlags_IsPageMove, "Identify a PageDown/PageUp request.")
        .value("activate", ImGuiNavMoveFlags_Activate, "Activate/select target item.")
        .value("no_select", ImGuiNavMoveFlags_NoSelect, "Don't trigger selection by not setting g.NavJustMovedTo")
        .value("no_set_nav_highlight", ImGuiNavMoveFlags_NoSetNavHighlight, "Do not alter the visible state of keyboard vs mouse nav highlight")
        .value("no_clear_active_id", ImGuiNavMoveFlags_NoClearActiveId, "(Experimental) Do not clear active id when applying move result");


    py::enum_<ImGuiNavLayer>(m, "NavLayer", py::arithmetic(), "")
        .value("main", ImGuiNavLayer_Main, "Main scrolling layer")
        .value("menu", ImGuiNavLayer_Menu, "Menu layer (access with Alt)")
        .value("count", ImGuiNavLayer_COUNT, "");


    auto pyClassImGuiNavItemData =
        py::class_<ImGuiNavItemData>
            (m, "NavItemData", "")
        .def_readwrite("window", &ImGuiNavItemData::Window, "Init,Move    // Best candidate window (result->ItemWindow->RootWindowForNav == request->Window)")
        .def_readwrite("id_", &ImGuiNavItemData::ID, "Init,Move    // Best candidate item ID")
        .def_readwrite("focus_scope_id", &ImGuiNavItemData::FocusScopeId, "Init,Move    // Best candidate focus scope ID")
        .def_readwrite("rect_rel", &ImGuiNavItemData::RectRel, "Init,Move    // Best candidate bounding box in window relative space")
        .def_readwrite("in_flags", &ImGuiNavItemData::InFlags, "????,Move    // Best candidate item flags")
        .def_readwrite("selection_user_data", &ImGuiNavItemData::SelectionUserData, "I+Mov    // Best candidate SetNextItemSelectionData() value.")
        .def_readwrite("dist_box", &ImGuiNavItemData::DistBox, "Move    // Best candidate box distance to current NavId")
        .def_readwrite("dist_center", &ImGuiNavItemData::DistCenter, "Move    // Best candidate center distance to current NavId")
        .def_readwrite("dist_axial", &ImGuiNavItemData::DistAxial, "Move    // Best candidate axial distance to current NavId")
        .def(py::init<>())
        .def("clear",
            &ImGuiNavItemData::Clear, "(private API)")
        ;


    auto pyClassImGuiFocusScopeData =
        py::class_<ImGuiFocusScopeData>
            (m, "FocusScopeData", "")
        .def(py::init<>([](
        ImGuiID ID = ImGuiID(), ImGuiID WindowID = ImGuiID())
        {
            auto r = std::make_unique<ImGuiFocusScopeData>();
            r->ID = ID;
            r->WindowID = WindowID;
            return r;
        })
        , py::arg("id_") = ImGuiID(), py::arg("window_id") = ImGuiID()
        )
        .def_readwrite("id_", &ImGuiFocusScopeData::ID, "")
        .def_readwrite("window_id", &ImGuiFocusScopeData::WindowID, "")
        ;


    py::enum_<ImGuiTypingSelectFlags_>(m, "TypingSelectFlags_", py::arithmetic(), "Flags for GetTypingSelectRequest()")
        .value("none", ImGuiTypingSelectFlags_None, "")
        .value("allow_backspace", ImGuiTypingSelectFlags_AllowBackspace, "Backspace to delete character inputs. If using: ensure GetTypingSelectRequest() is not called more than once per frame (filter by e.g. focus state)")
        .value("allow_single_char_mode", ImGuiTypingSelectFlags_AllowSingleCharMode, "Allow \"single char\" search mode which is activated when pressing the same character multiple times.");


    auto pyClassImGuiTypingSelectRequest =
        py::class_<ImGuiTypingSelectRequest>
            (m, "TypingSelectRequest", "Returned by GetTypingSelectRequest(), designed to eventually be public.")
        .def(py::init<>([](
        ImGuiTypingSelectFlags Flags = ImGuiTypingSelectFlags(), int SearchBufferLen = int(), bool SelectRequest = bool(), bool SingleCharMode = bool(), ImS8 SingleCharSize = ImS8())
        {
            auto r = std::make_unique<ImGuiTypingSelectRequest>();
            r->Flags = Flags;
            r->SearchBufferLen = SearchBufferLen;
            r->SelectRequest = SelectRequest;
            r->SingleCharMode = SingleCharMode;
            r->SingleCharSize = SingleCharSize;
            return r;
        })
        , py::arg("flags") = ImGuiTypingSelectFlags(), py::arg("search_buffer_len") = int(), py::arg("select_request") = bool(), py::arg("single_char_mode") = bool(), py::arg("single_char_size") = ImS8()
        )
        .def_readwrite("flags", &ImGuiTypingSelectRequest::Flags, "Flags passed to GetTypingSelectRequest()")
        .def_readwrite("search_buffer_len", &ImGuiTypingSelectRequest::SearchBufferLen, "")
        .def_readonly("search_buffer", &ImGuiTypingSelectRequest::SearchBuffer, "Search buffer contents (use full string. unless SingleCharMode is set, in which case use SingleCharSize).")
        .def_readwrite("select_request", &ImGuiTypingSelectRequest::SelectRequest, "Set when buffer was modified this frame, requesting a selection.")
        .def_readwrite("single_char_mode", &ImGuiTypingSelectRequest::SingleCharMode, "Notify when buffer contains same character repeated, to implement special mode. In this situation it preferred to not display any on-screen search indication.")
        .def_readwrite("single_char_size", &ImGuiTypingSelectRequest::SingleCharSize, "Length in bytes of first letter codepoint (1 for ascii, 2-4 for UTF-8). If (SearchBufferLen==RepeatCharSize) only 1 letter has been input.")
        ;


    auto pyClassImGuiTypingSelectState =
        py::class_<ImGuiTypingSelectState>
            (m, "TypingSelectState", "Storage for GetTypingSelectRequest()")
        .def_readwrite("request", &ImGuiTypingSelectState::Request, "User-facing data")
        .def_readwrite("focus_scope", &ImGuiTypingSelectState::FocusScope, "")
        .def_readwrite("last_request_frame", &ImGuiTypingSelectState::LastRequestFrame, "")
        .def_readwrite("last_request_time", &ImGuiTypingSelectState::LastRequestTime, "")
        .def_readwrite("single_char_mode_lock", &ImGuiTypingSelectState::SingleCharModeLock, "After a certain single char repeat count we lock into SingleCharMode. Two benefits: 1) buffer never fill, 2) we can provide an immediate SingleChar mode without timer elapsing.")
        .def(py::init<>())
        .def("clear",
            &ImGuiTypingSelectState::Clear, "(private API)\n\n We preserve remaining data for easier debugging")
        ;


    py::enum_<ImGuiOldColumnFlags_>(m, "OldColumnFlags_", py::arithmetic(), "Flags for internal's BeginColumns(). This is an obsolete API. Prefer using BeginTable() nowadays!")
        .value("none", ImGuiOldColumnFlags_None, "")
        .value("no_border", ImGuiOldColumnFlags_NoBorder, "Disable column dividers")
        .value("no_resize", ImGuiOldColumnFlags_NoResize, "Disable resizing columns when clicking on the dividers")
        .value("no_preserve_widths", ImGuiOldColumnFlags_NoPreserveWidths, "Disable column width preservation when adjusting columns")
        .value("no_force_within_window", ImGuiOldColumnFlags_NoForceWithinWindow, "Disable forcing columns to fit within window")
        .value("grow_parent_contents_size", ImGuiOldColumnFlags_GrowParentContentsSize, "Restore pre-1.51 behavior of extending the parent window contents size but _without affecting the columns width at all_. Will eventually remove.");


    auto pyClassImGuiOldColumnData =
        py::class_<ImGuiOldColumnData>
            (m, "OldColumnData", "")
        .def_readwrite("offset_norm", &ImGuiOldColumnData::OffsetNorm, "Column start offset, normalized 0.0 (far left) -> 1.0 (far right)")
        .def_readwrite("offset_norm_before_resize", &ImGuiOldColumnData::OffsetNormBeforeResize, "")
        .def_readwrite("flags", &ImGuiOldColumnData::Flags, "Not exposed")
        .def_readwrite("clip_rect", &ImGuiOldColumnData::ClipRect, "")
        .def(py::init<>())
        ;


    auto pyClassImGuiOldColumns =
        py::class_<ImGuiOldColumns>
            (m, "OldColumns", "")
        .def_readwrite("id_", &ImGuiOldColumns::ID, "")
        .def_readwrite("flags", &ImGuiOldColumns::Flags, "")
        .def_readwrite("is_first_frame", &ImGuiOldColumns::IsFirstFrame, "")
        .def_readwrite("is_being_resized", &ImGuiOldColumns::IsBeingResized, "")
        .def_readwrite("current", &ImGuiOldColumns::Current, "")
        .def_readwrite("count", &ImGuiOldColumns::Count, "")
        .def_readwrite("off_min_x", &ImGuiOldColumns::OffMinX, "Offsets from HostWorkRect.Min.x")
        .def_readwrite("off_max_x", &ImGuiOldColumns::OffMaxX, "Offsets from HostWorkRect.Min.x")
        .def_readwrite("line_min_y", &ImGuiOldColumns::LineMinY, "")
        .def_readwrite("line_max_y", &ImGuiOldColumns::LineMaxY, "")
        .def_readwrite("host_cursor_pos_y", &ImGuiOldColumns::HostCursorPosY, "Backup of CursorPos at the time of BeginColumns()")
        .def_readwrite("host_cursor_max_pos_x", &ImGuiOldColumns::HostCursorMaxPosX, "Backup of CursorMaxPos at the time of BeginColumns()")
        .def_readwrite("host_initial_clip_rect", &ImGuiOldColumns::HostInitialClipRect, "Backup of ClipRect at the time of BeginColumns()")
        .def_readwrite("host_backup_clip_rect", &ImGuiOldColumns::HostBackupClipRect, "Backup of ClipRect during PushColumnsBackground()/PopColumnsBackground()")
        .def_readwrite("host_backup_parent_work_rect", &ImGuiOldColumns::HostBackupParentWorkRect, "Backup of WorkRect at the time of BeginColumns()")
        .def_readwrite("columns", &ImGuiOldColumns::Columns, "")
        .def_readwrite("splitter", &ImGuiOldColumns::Splitter, "")
        .def(py::init<>())
        ;
    // #ifdef IMGUI_HAS_DOCK
    //


    py::enum_<ImGuiDockNodeFlagsPrivate_>(m, "DockNodeFlagsPrivate_", py::arithmetic(), "Extend ImGuiDockNodeFlags_")
        .value("dock_space", ImGuiDockNodeFlags_DockSpace, "Saved // A dockspace is a node that occupy space within an existing user window. Otherwise the node is floating and create its own window.")
        .value("central_node", ImGuiDockNodeFlags_CentralNode, "Saved // The central node has 2 main properties: stay visible when empty, only use \"remaining\" spaces from its neighbor.")
        .value("no_tab_bar", ImGuiDockNodeFlags_NoTabBar, "Saved // Tab bar is completely unavailable. No triangle in the corner to enable it back.")
        .value("hidden_tab_bar", ImGuiDockNodeFlags_HiddenTabBar, "Saved // Tab bar is hidden, with a triangle in the corner to show it again (NB: actual tab-bar instance may be destroyed as this is only used for single-window tab bar)")
        .value("no_window_menu_button", ImGuiDockNodeFlags_NoWindowMenuButton, "Saved // Disable window/docking menu (that one that appears instead of the collapse button)")
        .value("no_close_button", ImGuiDockNodeFlags_NoCloseButton, "Saved // Disable close button")
        .value("no_resize_x", ImGuiDockNodeFlags_NoResizeX, "//")
        .value("no_resize_y", ImGuiDockNodeFlags_NoResizeY, "//")
        .value("docked_windows_in_focus_route", ImGuiDockNodeFlags_DockedWindowsInFocusRoute, "// Any docked window will be automatically be focus-route chained (window->ParentWindowForFocusRoute set to this) so Shortcut() in this window can run when any docked window is focused.")
        .value("no_docking_split_other", ImGuiDockNodeFlags_NoDockingSplitOther, "// Disable this node from splitting other windows/nodes.")
        .value("no_docking_over_me", ImGuiDockNodeFlags_NoDockingOverMe, "// Disable other windows/nodes from being docked over this node.")
        .value("no_docking_over_other", ImGuiDockNodeFlags_NoDockingOverOther, "// Disable this node from being docked over another window or non-empty node.")
        .value("no_docking_over_empty", ImGuiDockNodeFlags_NoDockingOverEmpty, "// Disable this node from being docked over an empty node (e.g. DockSpace with no other windows)")
        .value("no_docking", ImGuiDockNodeFlags_NoDocking, "")
        .value("shared_flags_inherit_mask_", ImGuiDockNodeFlags_SharedFlagsInheritMask_, "")
        .value("no_resize_flags_mask_", ImGuiDockNodeFlags_NoResizeFlagsMask_, "")
        .value("local_flags_transfer_mask_", ImGuiDockNodeFlags_LocalFlagsTransferMask_, "")
        .value("saved_flags_mask_", ImGuiDockNodeFlags_SavedFlagsMask_, "");


    py::enum_<ImGuiDataAuthority_>(m, "DataAuthority_", py::arithmetic(), "Store the source authority (dock node vs window) of a field")
        .value("auto", ImGuiDataAuthority_Auto, "")
        .value("dock_node", ImGuiDataAuthority_DockNode, "")
        .value("window", ImGuiDataAuthority_Window, "");


    py::enum_<ImGuiDockNodeState>(m, "DockNodeState", py::arithmetic(), "")
        .value("unknown", ImGuiDockNodeState_Unknown, "")
        .value("host_window_hidden_because_single_window", ImGuiDockNodeState_HostWindowHiddenBecauseSingleWindow, "")
        .value("host_window_hidden_because_windows_are_resizing", ImGuiDockNodeState_HostWindowHiddenBecauseWindowsAreResizing, "")
        .value("host_window_visible", ImGuiDockNodeState_HostWindowVisible, "");


    auto pyClassImGuiDockNode =
        py::class_<ImGuiDockNode>
            (m, "DockNode", "sizeof() 156~192")
        .def_readwrite("id_", &ImGuiDockNode::ID, "")
        .def_readwrite("shared_flags", &ImGuiDockNode::SharedFlags, "(Write) Flags shared by all nodes of a same dockspace hierarchy (inherited from the root node)")
        .def_readwrite("local_flags", &ImGuiDockNode::LocalFlags, "(Write) Flags specific to this node")
        .def_readwrite("local_flags_in_windows", &ImGuiDockNode::LocalFlagsInWindows, "(Write) Flags specific to this node, applied from windows")
        .def_readwrite("merged_flags", &ImGuiDockNode::MergedFlags, "(Read)  Effective flags (== SharedFlags | LocalFlagsInNode | LocalFlagsInWindows)")
        .def_readwrite("state", &ImGuiDockNode::State, "")
        .def_readwrite("parent_node", &ImGuiDockNode::ParentNode, "")
        .def_readwrite("windows", &ImGuiDockNode::Windows, "Note: unordered list! Iterate TabBar->Tabs for user-order.")
        .def_readwrite("tab_bar", &ImGuiDockNode::TabBar, "")
        .def_readwrite("pos", &ImGuiDockNode::Pos, "Current position")
        .def_readwrite("size", &ImGuiDockNode::Size, "Current size")
        .def_readwrite("size_ref", &ImGuiDockNode::SizeRef, "[Split node only] Last explicitly written-to size (overridden when using a splitter affecting the node), used to calculate Size.")
        .def_readwrite("split_axis", &ImGuiDockNode::SplitAxis, "[Split node only] Split axis (X or Y)")
        .def_readwrite("window_class", &ImGuiDockNode::WindowClass, "[Root node only]")
        .def_readwrite("last_bg_color", &ImGuiDockNode::LastBgColor, "")
        .def_readwrite("host_window", &ImGuiDockNode::HostWindow, "")
        .def_readwrite("visible_window", &ImGuiDockNode::VisibleWindow, "Generally point to window which is ID is == SelectedTabID, but when CTRL+Tabbing this can be a different window.")
        .def_readwrite("central_node", &ImGuiDockNode::CentralNode, "[Root node only] Pointer to central node.")
        .def_readwrite("only_node_with_windows", &ImGuiDockNode::OnlyNodeWithWindows, "[Root node only] Set when there is a single visible node within the hierarchy.")
        .def_readwrite("count_node_with_windows", &ImGuiDockNode::CountNodeWithWindows, "[Root node only]")
        .def_readwrite("last_frame_alive", &ImGuiDockNode::LastFrameAlive, "Last frame number the node was updated or kept alive explicitly with DockSpace() + ImGuiDockNodeFlags_KeepAliveOnly")
        .def_readwrite("last_frame_active", &ImGuiDockNode::LastFrameActive, "Last frame number the node was updated.")
        .def_readwrite("last_frame_focused", &ImGuiDockNode::LastFrameFocused, "Last frame number the node was focused.")
        .def_readwrite("last_focused_node_id", &ImGuiDockNode::LastFocusedNodeId, "[Root node only] Which of our child docking node (any ancestor in the hierarchy) was last focused.")
        .def_readwrite("selected_tab_id", &ImGuiDockNode::SelectedTabId, "[Leaf node only] Which of our tab/window is selected.")
        .def_readwrite("want_close_tab_id", &ImGuiDockNode::WantCloseTabId, "[Leaf node only] Set when closing a specific tab/window.")
        .def_readwrite("ref_viewport_id", &ImGuiDockNode::RefViewportId, "Reference viewport ID from visible window when HostWindow == None.")
        .def(py::init<ImGuiID>(),
            py::arg("id_"))
        .def("is_root_node",
            &ImGuiDockNode::IsRootNode, "(private API)")
        .def("is_dock_space",
            &ImGuiDockNode::IsDockSpace, "(private API)")
        .def("is_floating_node",
            &ImGuiDockNode::IsFloatingNode, "(private API)")
        .def("is_central_node",
            &ImGuiDockNode::IsCentralNode, "(private API)")
        .def("is_hidden_tab_bar",
            &ImGuiDockNode::IsHiddenTabBar, "(private API)\n\n Hidden tab bar can be shown back by clicking the small triangle")
        .def("is_no_tab_bar",
            &ImGuiDockNode::IsNoTabBar, "(private API)\n\n Never show a tab bar")
        .def("is_split_node",
            &ImGuiDockNode::IsSplitNode, "(private API)")
        .def("is_leaf_node",
            &ImGuiDockNode::IsLeafNode, "(private API)")
        .def("is_empty",
            &ImGuiDockNode::IsEmpty, "(private API)")
        .def("rect",
            &ImGuiDockNode::Rect, "(private API)")
        .def("set_local_flags",
            &ImGuiDockNode::SetLocalFlags,
            py::arg("flags"),
            "(private API)")
        .def("update_merged_flags",
            &ImGuiDockNode::UpdateMergedFlags, "(private API)")
        ;


    py::enum_<ImGuiWindowDockStyleCol>(m, "WindowDockStyleCol", py::arithmetic(), " List of colors that are stored at the time of Begin() into Docked Windows.\n We currently store the packed colors in a simple array window->DockStyle.Colors[].\n A better solution may involve appending into a log of colors in ImGuiContext + store offsets into those arrays in ImGuiWindow,\n but it would be more complex as we'd need to double-buffer both as e.g. drop target may refer to window from last frame.")
        .value("text", ImGuiWindowDockStyleCol_Text, "")
        .value("tab", ImGuiWindowDockStyleCol_Tab, "")
        .value("tab_hovered", ImGuiWindowDockStyleCol_TabHovered, "")
        .value("tab_active", ImGuiWindowDockStyleCol_TabActive, "")
        .value("tab_unfocused", ImGuiWindowDockStyleCol_TabUnfocused, "")
        .value("tab_unfocused_active", ImGuiWindowDockStyleCol_TabUnfocusedActive, "")
        .value("count", ImGuiWindowDockStyleCol_COUNT, "");


    auto pyClassImGuiWindowDockStyle =
        py::class_<ImGuiWindowDockStyle>
            (m, "WindowDockStyle", "")
        .def(py::init<>()) // implicit default constructor
        ;


    auto pyClassImGuiDockContext =
        py::class_<ImGuiDockContext>
            (m, "DockContext", "")
        .def_readwrite("nodes", &ImGuiDockContext::Nodes, "Map ID -> ImGuiDockNode*: Active nodes")
        .def_readwrite("want_full_rebuild", &ImGuiDockContext::WantFullRebuild, "")
        .def(py::init<>())
        ;
    // #endif


    auto pyClassImGuiViewportP =
        py::class_<ImGuiViewportP>
            (m, "ViewportP", " ImGuiViewport Private/Internals fields (cardinal sin: we are using inheritance!)\n Every instance of ImGuiViewport is in fact a ImGuiViewportP.")
        .def_readwrite("window", &ImGuiViewportP::Window, "Set when the viewport is owned by a window (and ImGuiViewportFlags_CanHostOtherWindows is NOT set)")
        .def_readwrite("idx", &ImGuiViewportP::Idx, "")
        .def_readwrite("last_frame_active", &ImGuiViewportP::LastFrameActive, "Last frame number this viewport was activated by a window")
        .def_readwrite("last_focused_stamp_count", &ImGuiViewportP::LastFocusedStampCount, "Last stamp number from when a window hosted by this viewport was focused (by comparing this value between two viewport we have an implicit viewport z-order we use as fallback)")
        .def_readwrite("last_name_hash", &ImGuiViewportP::LastNameHash, "")
        .def_readwrite("last_pos", &ImGuiViewportP::LastPos, "")
        .def_readwrite("alpha", &ImGuiViewportP::Alpha, "Window opacity (when dragging dockable windows/viewports we make them transparent)")
        .def_readwrite("last_alpha", &ImGuiViewportP::LastAlpha, "")
        .def_readwrite("last_focused_had_nav_window", &ImGuiViewportP::LastFocusedHadNavWindow, "Instead of maintaining a LastFocusedWindow (which may harder to correctly maintain), we merely store weither NavWindow != None last time the viewport was focused.")
        .def_readwrite("platform_monitor", &ImGuiViewportP::PlatformMonitor, "")
        .def_property("bg_fg_draw_lists_last_frame",
            [](ImGuiViewportP &self) -> pybind11::array
            {
                auto dtype = pybind11::dtype(pybind11::format_descriptor<int>::format());
                auto base = pybind11::array(dtype, {2}, {sizeof(int)});
                return pybind11::array(dtype, {2}, {sizeof(int)}, self.BgFgDrawListsLastFrame, base);
            }, [](ImGuiViewportP& self) {},
            "Last frame number the background (0) and foreground (1) draw lists were used")
        .def_readwrite("draw_data_p", &ImGuiViewportP::DrawDataP, "")
        .def_readwrite("draw_data_builder", &ImGuiViewportP::DrawDataBuilder, "Temporary data while building final ImDrawData")
        .def_readwrite("last_platform_pos", &ImGuiViewportP::LastPlatformPos, "")
        .def_readwrite("last_platform_size", &ImGuiViewportP::LastPlatformSize, "")
        .def_readwrite("last_renderer_size", &ImGuiViewportP::LastRendererSize, "")
        .def_readwrite("work_offset_min", &ImGuiViewportP::WorkOffsetMin, "Work Area: Offset from Pos to top-left corner of Work Area. Generally (0,0) or (0,+main_menu_bar_height). Work Area is Full Area but without menu-bars/status-bars (so WorkArea always fit inside Pos/Size!)")
        .def_readwrite("work_offset_max", &ImGuiViewportP::WorkOffsetMax, "Work Area: Offset from Pos+Size to bottom-right corner of Work Area. Generally (0,0) or (0,-status_bar_height).")
        .def_readwrite("build_work_offset_min", &ImGuiViewportP::BuildWorkOffsetMin, "Work Area: Offset being built during current frame. Generally >= 0.0.")
        .def_readwrite("build_work_offset_max", &ImGuiViewportP::BuildWorkOffsetMax, "Work Area: Offset being built during current frame. Generally <= 0.0.")
        .def(py::init<>())
        .def("clear_request_flags",
            &ImGuiViewportP::ClearRequestFlags, "(private API)")
        .def("calc_work_rect_pos",
            &ImGuiViewportP::CalcWorkRectPos,
            py::arg("off_min"),
            "(private API)")
        .def("calc_work_rect_size",
            &ImGuiViewportP::CalcWorkRectSize,
            py::arg("off_min"), py::arg("off_max"),
            "(private API)")
        .def("update_work_rect",
            &ImGuiViewportP::UpdateWorkRect, "(private API)\n\n Update public fields")
        .def("get_main_rect",
            &ImGuiViewportP::GetMainRect, "(private API)")
        .def("get_work_rect",
            &ImGuiViewportP::GetWorkRect, "(private API)")
        .def("get_build_work_rect",
            &ImGuiViewportP::GetBuildWorkRect, "(private API)")
        ;


    auto pyClassImGuiWindowSettings =
        py::class_<ImGuiWindowSettings>
            (m, "WindowSettings", " Windows data saved in imgui.ini file\n Because we never destroy or rename ImGuiWindowSettings, we can store the names in a separate buffer easily.\n (this is designed to be stored in a ImChunkStream buffer, with the variable-length Name following our structure)")
        .def_readwrite("id_", &ImGuiWindowSettings::ID, "")
        .def_readwrite("pos", &ImGuiWindowSettings::Pos, "NB: Settings position are stored RELATIVE to the viewport! Whereas runtime ones are absolute positions.")
        .def_readwrite("size", &ImGuiWindowSettings::Size, "")
        .def_readwrite("viewport_pos", &ImGuiWindowSettings::ViewportPos, "")
        .def_readwrite("viewport_id", &ImGuiWindowSettings::ViewportId, "")
        .def_readwrite("dock_id", &ImGuiWindowSettings::DockId, "ID of last known DockNode (even if the DockNode is invisible because it has only 1 active window), or 0 if none.")
        .def_readwrite("class_id", &ImGuiWindowSettings::ClassId, "ID of window class if specified")
        .def_readwrite("dock_order", &ImGuiWindowSettings::DockOrder, "Order of the last time the window was visible within its DockNode. This is used to reorder windows that are reappearing on the same frame. Same value between windows that were active and windows that were none are possible.")
        .def_readwrite("collapsed", &ImGuiWindowSettings::Collapsed, "")
        .def_readwrite("is_child", &ImGuiWindowSettings::IsChild, "")
        .def_readwrite("want_apply", &ImGuiWindowSettings::WantApply, "Set when loaded from .ini data (to enable merging/loading .ini data into an already running context)")
        .def_readwrite("want_delete", &ImGuiWindowSettings::WantDelete, "Set to invalidate/delete the settings entry")
        .def(py::init<>())
        // #ifdef IMGUI_BUNDLE_PYTHON_API
        //
        .def("get_name_str",
            &ImGuiWindowSettings::GetNameStr, "(private API)")
        // #endif
        //
        ;


    auto pyClassImGuiSettingsHandler =
        py::class_<ImGuiSettingsHandler>
            (m, "SettingsHandler", "")
        .def_readonly("type_name", &ImGuiSettingsHandler::TypeName, "Short description stored in .ini file. Disallowed characters: '[' ']'")
        .def_readwrite("type_hash", &ImGuiSettingsHandler::TypeHash, "== ImHashStr(TypeName)")
        .def_readwrite("user_data", &ImGuiSettingsHandler::UserData, "")
        .def(py::init<>())
        ;


    py::enum_<ImGuiLocKey>(m, "LocKey", py::arithmetic(), "This is experimental and not officially supported, it'll probably fall short of features, if/when it does we may backtrack.")
        .value("version_str", ImGuiLocKey_VersionStr, "")
        .value("table_size_one", ImGuiLocKey_TableSizeOne, "")
        .value("table_size_all_fit", ImGuiLocKey_TableSizeAllFit, "")
        .value("table_size_all_default", ImGuiLocKey_TableSizeAllDefault, "")
        .value("table_reset_order", ImGuiLocKey_TableResetOrder, "")
        .value("windowing_main_menu_bar", ImGuiLocKey_WindowingMainMenuBar, "")
        .value("windowing_popup", ImGuiLocKey_WindowingPopup, "")
        .value("windowing_untitled", ImGuiLocKey_WindowingUntitled, "")
        .value("docking_hide_tab_bar", ImGuiLocKey_DockingHideTabBar, "")
        .value("docking_hold_shift_to_dock", ImGuiLocKey_DockingHoldShiftToDock, "")
        .value("docking_drag_to_undock_or_move_node", ImGuiLocKey_DockingDragToUndockOrMoveNode, "")
        .value("count", ImGuiLocKey_COUNT, "");


    auto pyClassImGuiLocEntry =
        py::class_<ImGuiLocEntry>
            (m, "LocEntry", "")
        .def(py::init<>([](
        ImGuiLocKey Key = ImGuiLocKey())
        {
            auto r = std::make_unique<ImGuiLocEntry>();
            r->Key = Key;
            return r;
        })
        , py::arg("key") = ImGuiLocKey()
        )
        .def_readwrite("key", &ImGuiLocEntry::Key, "")
        .def_readonly("text", &ImGuiLocEntry::Text, "")
        ;


    py::enum_<ImGuiDebugLogFlags_>(m, "DebugLogFlags_", py::arithmetic(), "")
        .value("none", ImGuiDebugLogFlags_None, "")
        .value("event_active_id", ImGuiDebugLogFlags_EventActiveId, "")
        .value("event_focus", ImGuiDebugLogFlags_EventFocus, "")
        .value("event_popup", ImGuiDebugLogFlags_EventPopup, "")
        .value("event_nav", ImGuiDebugLogFlags_EventNav, "")
        .value("event_clipper", ImGuiDebugLogFlags_EventClipper, "")
        .value("event_selection", ImGuiDebugLogFlags_EventSelection, "")
        .value("event_io", ImGuiDebugLogFlags_EventIO, "")
        .value("event_input_routing", ImGuiDebugLogFlags_EventInputRouting, "")
        .value("event_docking", ImGuiDebugLogFlags_EventDocking, "")
        .value("event_viewport", ImGuiDebugLogFlags_EventViewport, "")
        .value("event_mask_", ImGuiDebugLogFlags_EventMask_, "")
        .value("output_to_tty", ImGuiDebugLogFlags_OutputToTTY, "Also send output to TTY")
        .value("output_to_test_engine", ImGuiDebugLogFlags_OutputToTestEngine, "Also send output to Test Engine");


    auto pyClassImGuiDebugAllocEntry =
        py::class_<ImGuiDebugAllocEntry>
            (m, "DebugAllocEntry", "")
        .def(py::init<>([](
        int FrameCount = int(), ImS16 AllocCount = ImS16(), ImS16 FreeCount = ImS16())
        {
            auto r = std::make_unique<ImGuiDebugAllocEntry>();
            r->FrameCount = FrameCount;
            r->AllocCount = AllocCount;
            r->FreeCount = FreeCount;
            return r;
        })
        , py::arg("frame_count") = int(), py::arg("alloc_count") = ImS16(), py::arg("free_count") = ImS16()
        )
        .def_readwrite("frame_count", &ImGuiDebugAllocEntry::FrameCount, "")
        .def_readwrite("alloc_count", &ImGuiDebugAllocEntry::AllocCount, "")
        .def_readwrite("free_count", &ImGuiDebugAllocEntry::FreeCount, "")
        ;


    auto pyClassImGuiDebugAllocInfo =
        py::class_<ImGuiDebugAllocInfo>
            (m, "DebugAllocInfo", "")
        .def_readwrite("total_alloc_count", &ImGuiDebugAllocInfo::TotalAllocCount, "Number of call to MemAlloc().")
        .def_readwrite("total_free_count", &ImGuiDebugAllocInfo::TotalFreeCount, "")
        .def_readwrite("last_entries_idx", &ImGuiDebugAllocInfo::LastEntriesIdx, "Current index in buffer")
        .def(py::init<>())
        ;


    auto pyClassImGuiMetricsConfig =
        py::class_<ImGuiMetricsConfig>
            (m, "MetricsConfig", "")
        .def(py::init<>([](
        bool ShowDebugLog = false, bool ShowIDStackTool = false, bool ShowWindowsRects = false, bool ShowWindowsBeginOrder = false, bool ShowTablesRects = false, bool ShowDrawCmdMesh = true, bool ShowDrawCmdBoundingBoxes = true, bool ShowTextEncodingViewer = false, bool ShowAtlasTintedWithTextColor = false, bool ShowDockingNodes = false, int ShowWindowsRectsType = -1, int ShowTablesRectsType = -1, int HighlightMonitorIdx = -1, ImGuiID HighlightViewportID = 0)
        {
            auto r = std::make_unique<ImGuiMetricsConfig>();
            r->ShowDebugLog = ShowDebugLog;
            r->ShowIDStackTool = ShowIDStackTool;
            r->ShowWindowsRects = ShowWindowsRects;
            r->ShowWindowsBeginOrder = ShowWindowsBeginOrder;
            r->ShowTablesRects = ShowTablesRects;
            r->ShowDrawCmdMesh = ShowDrawCmdMesh;
            r->ShowDrawCmdBoundingBoxes = ShowDrawCmdBoundingBoxes;
            r->ShowTextEncodingViewer = ShowTextEncodingViewer;
            r->ShowAtlasTintedWithTextColor = ShowAtlasTintedWithTextColor;
            r->ShowDockingNodes = ShowDockingNodes;
            r->ShowWindowsRectsType = ShowWindowsRectsType;
            r->ShowTablesRectsType = ShowTablesRectsType;
            r->HighlightMonitorIdx = HighlightMonitorIdx;
            r->HighlightViewportID = HighlightViewportID;
            return r;
        })
        , py::arg("show_debug_log") = false, py::arg("show_id_stack_tool") = false, py::arg("show_windows_rects") = false, py::arg("show_windows_begin_order") = false, py::arg("show_tables_rects") = false, py::arg("show_draw_cmd_mesh") = true, py::arg("show_draw_cmd_bounding_boxes") = true, py::arg("show_text_encoding_viewer") = false, py::arg("show_atlas_tinted_with_text_color") = false, py::arg("show_docking_nodes") = false, py::arg("show_windows_rects_type") = -1, py::arg("show_tables_rects_type") = -1, py::arg("highlight_monitor_idx") = -1, py::arg("highlight_viewport_id") = 0
        )
        .def_readwrite("show_debug_log", &ImGuiMetricsConfig::ShowDebugLog, "")
        .def_readwrite("show_id_stack_tool", &ImGuiMetricsConfig::ShowIDStackTool, "")
        .def_readwrite("show_windows_rects", &ImGuiMetricsConfig::ShowWindowsRects, "")
        .def_readwrite("show_windows_begin_order", &ImGuiMetricsConfig::ShowWindowsBeginOrder, "")
        .def_readwrite("show_tables_rects", &ImGuiMetricsConfig::ShowTablesRects, "")
        .def_readwrite("show_draw_cmd_mesh", &ImGuiMetricsConfig::ShowDrawCmdMesh, "")
        .def_readwrite("show_draw_cmd_bounding_boxes", &ImGuiMetricsConfig::ShowDrawCmdBoundingBoxes, "")
        .def_readwrite("show_text_encoding_viewer", &ImGuiMetricsConfig::ShowTextEncodingViewer, "")
        .def_readwrite("show_atlas_tinted_with_text_color", &ImGuiMetricsConfig::ShowAtlasTintedWithTextColor, "")
        .def_readwrite("show_docking_nodes", &ImGuiMetricsConfig::ShowDockingNodes, "")
        .def_readwrite("show_windows_rects_type", &ImGuiMetricsConfig::ShowWindowsRectsType, "")
        .def_readwrite("show_tables_rects_type", &ImGuiMetricsConfig::ShowTablesRectsType, "")
        .def_readwrite("highlight_monitor_idx", &ImGuiMetricsConfig::HighlightMonitorIdx, "")
        .def_readwrite("highlight_viewport_id", &ImGuiMetricsConfig::HighlightViewportID, "")
        ;


    auto pyClassImGuiStackLevelInfo =
        py::class_<ImGuiStackLevelInfo>
            (m, "StackLevelInfo", "")
        .def_readwrite("id_", &ImGuiStackLevelInfo::ID, "")
        .def_readwrite("query_frame_count", &ImGuiStackLevelInfo::QueryFrameCount, ">= 1: Query in progress")
        .def_readwrite("query_success", &ImGuiStackLevelInfo::QuerySuccess, "Obtained result from DebugHookIdInfo()")
        .def(py::init<>())
        ;


    auto pyClassImGuiIDStackTool =
        py::class_<ImGuiIDStackTool>
            (m, "IDStackTool", "State for ID Stack tool queries")
        .def_readwrite("last_active_frame", &ImGuiIDStackTool::LastActiveFrame, "")
        .def_readwrite("stack_level", &ImGuiIDStackTool::StackLevel, "-1: query stack and resize Results, >= 0: individual stack level")
        .def_readwrite("query_id", &ImGuiIDStackTool::QueryId, "ID to query details for")
        .def_readwrite("results", &ImGuiIDStackTool::Results, "")
        .def_readwrite("copy_to_clipboard_on_ctrl_c", &ImGuiIDStackTool::CopyToClipboardOnCtrlC, "")
        .def_readwrite("copy_to_clipboard_last_time", &ImGuiIDStackTool::CopyToClipboardLastTime, "")
        .def(py::init<>())
        ;


    py::enum_<ImGuiContextHookType>(m, "ContextHookType", py::arithmetic(), "")
        .value("new_frame_pre", ImGuiContextHookType_NewFramePre, "")
        .value("new_frame_post", ImGuiContextHookType_NewFramePost, "")
        .value("end_frame_pre", ImGuiContextHookType_EndFramePre, "")
        .value("end_frame_post", ImGuiContextHookType_EndFramePost, "")
        .value("render_pre", ImGuiContextHookType_RenderPre, "")
        .value("render_post", ImGuiContextHookType_RenderPost, "")
        .value("shutdown", ImGuiContextHookType_Shutdown, "")
        .value("pending_removal_", ImGuiContextHookType_PendingRemoval_, "");


    auto pyClassImGuiContextHook =
        py::class_<ImGuiContextHook>
            (m, "ContextHook", "")
        .def_readwrite("hook_id", &ImGuiContextHook::HookId, "A unique ID assigned by AddContextHook()")
        .def_readwrite("type", &ImGuiContextHook::Type, "")
        .def_readwrite("owner", &ImGuiContextHook::Owner, "")
        .def_readwrite("user_data", &ImGuiContextHook::UserData, "")
        .def(py::init<>())
        ;


    auto pyClassImGuiContext =
        py::class_<ImGuiContext>
            (m, "Context", "")
        .def_readwrite("initialized", &ImGuiContext::Initialized, "")
        .def_readwrite("font_atlas_owned_by_context", &ImGuiContext::FontAtlasOwnedByContext, "IO.Fonts-> is owned by the ImGuiContext and will be destructed along with it.")
        .def_readwrite("io", &ImGuiContext::IO, "")
        .def_readwrite("platform_io", &ImGuiContext::PlatformIO, "")
        .def_readwrite("style", &ImGuiContext::Style, "")
        .def_readwrite("config_flags_curr_frame", &ImGuiContext::ConfigFlagsCurrFrame, "= g.IO.ConfigFlags at the time of NewFrame()")
        .def_readwrite("config_flags_last_frame", &ImGuiContext::ConfigFlagsLastFrame, "")
        .def_readwrite("font", &ImGuiContext::Font, "(Shortcut) == FontStack.empty() ? IO.Font : FontStack.back()")
        .def_readwrite("font_size", &ImGuiContext::FontSize, "(Shortcut) == FontBaseSize * g.CurrentWindow->FontWindowScale == window->FontSize(). Text height for current window.")
        .def_readwrite("font_base_size", &ImGuiContext::FontBaseSize, "(Shortcut) == IO.FontGlobalScale * Font->Scale * Font->FontSize. Base text height.")
        .def_readwrite("draw_list_shared_data", &ImGuiContext::DrawListSharedData, "")
        .def_readwrite("time", &ImGuiContext::Time, "")
        .def_readwrite("frame_count", &ImGuiContext::FrameCount, "")
        .def_readwrite("frame_count_ended", &ImGuiContext::FrameCountEnded, "")
        .def_readwrite("frame_count_platform_ended", &ImGuiContext::FrameCountPlatformEnded, "")
        .def_readwrite("frame_count_rendered", &ImGuiContext::FrameCountRendered, "")
        .def_readwrite("within_frame_scope", &ImGuiContext::WithinFrameScope, "Set by NewFrame(), cleared by EndFrame()")
        .def_readwrite("within_frame_scope_with_implicit_window", &ImGuiContext::WithinFrameScopeWithImplicitWindow, "Set by NewFrame(), cleared by EndFrame() when the implicit debug window has been pushed")
        .def_readwrite("within_end_child", &ImGuiContext::WithinEndChild, "Set within EndChild()")
        .def_readwrite("gc_compact_all", &ImGuiContext::GcCompactAll, "Request full GC")
        .def_readwrite("test_engine_hook_items", &ImGuiContext::TestEngineHookItems, "Will call test engine hooks: ImGuiTestEngineHook_ItemAdd(), ImGuiTestEngineHook_ItemInfo(), ImGuiTestEngineHook_Log()")
        .def_readwrite("test_engine", &ImGuiContext::TestEngine, "Test engine user data")
        .def_readwrite("input_events_queue", &ImGuiContext::InputEventsQueue, "Input events which will be trickled/written into IO structure.")
        .def_readwrite("input_events_trail", &ImGuiContext::InputEventsTrail, "Past input events processed in NewFrame(). This is to allow domain-specific application to access e.g mouse/pen trail.")
        .def_readwrite("input_events_next_mouse_source", &ImGuiContext::InputEventsNextMouseSource, "")
        .def_readwrite("input_events_next_event_id", &ImGuiContext::InputEventsNextEventId, "")
        .def_readwrite("windows", &ImGuiContext::Windows, "Windows, sorted in display order, back to front")
        .def_readwrite("windows_focus_order", &ImGuiContext::WindowsFocusOrder, "Root windows, sorted in focus order, back to front.")
        .def_readwrite("windows_temp_sort_buffer", &ImGuiContext::WindowsTempSortBuffer, "Temporary buffer used in EndFrame() to reorder windows so parents are kept before their child")
        .def_readwrite("current_window_stack", &ImGuiContext::CurrentWindowStack, "")
        .def_readwrite("windows_by_id", &ImGuiContext::WindowsById, "Map window's ImGuiID to ImGuiWindow*")
        .def_readwrite("windows_active_count", &ImGuiContext::WindowsActiveCount, "Number of unique windows submitted by frame")
        .def_readwrite("windows_hover_padding", &ImGuiContext::WindowsHoverPadding, "Padding around resizable windows for which hovering on counts as hovering the window == ImMax(style.TouchExtraPadding, WINDOWS_HOVER_PADDING)")
        .def_readwrite("debug_break_in_window", &ImGuiContext::DebugBreakInWindow, "Set to break in Begin() call.")
        .def_readwrite("current_window", &ImGuiContext::CurrentWindow, "Window being drawn into")
        .def_readwrite("hovered_window", &ImGuiContext::HoveredWindow, "Window the mouse is hovering. Will typically catch mouse inputs.")
        .def_readwrite("hovered_window_under_moving_window", &ImGuiContext::HoveredWindowUnderMovingWindow, "Hovered window ignoring MovingWindow. Only set if MovingWindow is set.")
        .def_readwrite("moving_window", &ImGuiContext::MovingWindow, "Track the window we clicked on (in order to preserve focus). The actual window that is moved is generally MovingWindow->RootWindowDockTree.")
        .def_readwrite("wheeling_window", &ImGuiContext::WheelingWindow, "Track the window we started mouse-wheeling on. Until a timer elapse or mouse has moved, generally keep scrolling the same window even if during the course of scrolling the mouse ends up hovering a child window.")
        .def_readwrite("wheeling_window_ref_mouse_pos", &ImGuiContext::WheelingWindowRefMousePos, "")
        .def_readwrite("wheeling_window_start_frame", &ImGuiContext::WheelingWindowStartFrame, "This may be set one frame before WheelingWindow is != None")
        .def_readwrite("wheeling_window_scrolled_frame", &ImGuiContext::WheelingWindowScrolledFrame, "")
        .def_readwrite("wheeling_window_release_timer", &ImGuiContext::WheelingWindowReleaseTimer, "")
        .def_readwrite("wheeling_window_wheel_remainder", &ImGuiContext::WheelingWindowWheelRemainder, "")
        .def_readwrite("wheeling_axis_avg", &ImGuiContext::WheelingAxisAvg, "")
        .def_readwrite("debug_hook_id_info", &ImGuiContext::DebugHookIdInfo, "Will call core hooks: DebugHookIdInfo() from GetID functions, used by ID Stack Tool [next HoveredId/ActiveId to not pull in an extra cache-line]")
        .def_readwrite("hovered_id", &ImGuiContext::HoveredId, "Hovered widget, filled during the frame")
        .def_readwrite("hovered_id_previous_frame", &ImGuiContext::HoveredIdPreviousFrame, "")
        .def_readwrite("hovered_id_allow_overlap", &ImGuiContext::HoveredIdAllowOverlap, "")
        .def_readwrite("hovered_id_disabled", &ImGuiContext::HoveredIdDisabled, "At least one widget passed the rect test, but has been discarded by disabled flag or popup inhibit. May be True even if HoveredId == 0.")
        .def_readwrite("hovered_id_timer", &ImGuiContext::HoveredIdTimer, "Measure contiguous hovering time")
        .def_readwrite("hovered_id_not_active_timer", &ImGuiContext::HoveredIdNotActiveTimer, "Measure contiguous hovering time where the item has not been active")
        .def_readwrite("active_id", &ImGuiContext::ActiveId, "Active widget")
        .def_readwrite("active_id_is_alive", &ImGuiContext::ActiveIdIsAlive, "Active widget has been seen this frame (we can't use a bool as the ActiveId may change within the frame)")
        .def_readwrite("active_id_timer", &ImGuiContext::ActiveIdTimer, "")
        .def_readwrite("active_id_is_just_activated", &ImGuiContext::ActiveIdIsJustActivated, "Set at the time of activation for one frame")
        .def_readwrite("active_id_allow_overlap", &ImGuiContext::ActiveIdAllowOverlap, "Active widget allows another widget to steal active id (generally for overlapping widgets, but not always)")
        .def_readwrite("active_id_no_clear_on_focus_loss", &ImGuiContext::ActiveIdNoClearOnFocusLoss, "Disable losing active id if the active id window gets unfocused.")
        .def_readwrite("active_id_has_been_pressed_before", &ImGuiContext::ActiveIdHasBeenPressedBefore, "Track whether the active id led to a press (this is to allow changing between PressOnClick and PressOnRelease without pressing twice). Used by range_select branch.")
        .def_readwrite("active_id_has_been_edited_before", &ImGuiContext::ActiveIdHasBeenEditedBefore, "Was the value associated to the widget Edited over the course of the Active state.")
        .def_readwrite("active_id_has_been_edited_this_frame", &ImGuiContext::ActiveIdHasBeenEditedThisFrame, "")
        .def_readwrite("active_id_from_shortcut", &ImGuiContext::ActiveIdFromShortcut, "")
        .def_readwrite("active_id_click_offset", &ImGuiContext::ActiveIdClickOffset, "Clicked offset from upper-left corner, if applicable (currently only set by ButtonBehavior)")
        .def_readwrite("active_id_window", &ImGuiContext::ActiveIdWindow, "")
        .def_readwrite("active_id_source", &ImGuiContext::ActiveIdSource, "Activating source: ImGuiInputSource_Mouse OR ImGuiInputSource_Keyboard OR ImGuiInputSource_Gamepad")
        .def_readwrite("active_id_previous_frame", &ImGuiContext::ActiveIdPreviousFrame, "")
        .def_readwrite("active_id_previous_frame_is_alive", &ImGuiContext::ActiveIdPreviousFrameIsAlive, "")
        .def_readwrite("active_id_previous_frame_has_been_edited_before", &ImGuiContext::ActiveIdPreviousFrameHasBeenEditedBefore, "")
        .def_readwrite("active_id_previous_frame_window", &ImGuiContext::ActiveIdPreviousFrameWindow, "")
        .def_readwrite("last_active_id", &ImGuiContext::LastActiveId, "Store the last non-zero ActiveId, useful for animation.")
        .def_readwrite("last_active_id_timer", &ImGuiContext::LastActiveIdTimer, "Store the last non-zero ActiveId timer since the beginning of activation, useful for animation.")
        .def_readwrite("last_key_mods_change_time", &ImGuiContext::LastKeyModsChangeTime, "Record the last time key mods changed (affect repeat delay when using shortcut logic)")
        .def_readwrite("last_key_mods_change_from_none_time", &ImGuiContext::LastKeyModsChangeFromNoneTime, "Record the last time key mods changed away from being 0 (affect repeat delay when using shortcut logic)")
        .def_readwrite("last_keyboard_key_press_time", &ImGuiContext::LastKeyboardKeyPressTime, "Record the last time a keyboard key (ignore mouse/gamepad ones) was pressed.")
        .def_readwrite("keys_routing_table", &ImGuiContext::KeysRoutingTable, "")
        .def_readwrite("active_id_using_nav_dir_mask", &ImGuiContext::ActiveIdUsingNavDirMask, "Active widget will want to read those nav move requests (e.g. can activate a button and move away from it)")
        .def_readwrite("active_id_using_all_keyboard_keys", &ImGuiContext::ActiveIdUsingAllKeyboardKeys, "Active widget will want to read all keyboard keys inputs. (FIXME: This is a shortcut for not taking ownership of 100+ keys but perhaps best to not have the inconsistency)")
        .def_readwrite("debug_break_in_shortcut_routing", &ImGuiContext::DebugBreakInShortcutRouting, "")
        .def_readwrite("current_focus_scope_id", &ImGuiContext::CurrentFocusScopeId, "Value for currently appending items == g.FocusScopeStack.back(). Not to be mistaken with g.NavFocusScopeId.")
        .def_readwrite("current_item_flags", &ImGuiContext::CurrentItemFlags, "Value for currently appending items == g.ItemFlagsStack.back()")
        .def_readwrite("debug_locate_id", &ImGuiContext::DebugLocateId, "Storage for DebugLocateItemOnHover() feature: this is read by ItemAdd() so we keep it in a hot/cached location")
        .def_readwrite("next_item_data", &ImGuiContext::NextItemData, "Storage for SetNextItem** functions")
        .def_readwrite("last_item_data", &ImGuiContext::LastItemData, "Storage for last submitted item (setup by ItemAdd)")
        .def_readwrite("next_window_data", &ImGuiContext::NextWindowData, "Storage for SetNextWindow** functions")
        .def_readwrite("debug_show_group_rects", &ImGuiContext::DebugShowGroupRects, "")
        .def_readwrite("debug_flash_style_color_idx", &ImGuiContext::DebugFlashStyleColorIdx, "(Keep close to ColorStack to share cache line)")
        .def_readwrite("color_stack", &ImGuiContext::ColorStack, "Stack for PushStyleColor()/PopStyleColor() - inherited by Begin()")
        .def_readwrite("style_var_stack", &ImGuiContext::StyleVarStack, "Stack for PushStyleVar()/PopStyleVar() - inherited by Begin()")
        .def_readwrite("font_stack", &ImGuiContext::FontStack, "Stack for PushFont()/PopFont() - inherited by Begin()")
        .def_readwrite("focus_scope_stack", &ImGuiContext::FocusScopeStack, "Stack for PushFocusScope()/PopFocusScope() - inherited by BeginChild(), pushed into by Begin()")
        .def_readwrite("item_flags_stack", &ImGuiContext::ItemFlagsStack, "Stack for PushItemFlag()/PopItemFlag() - inherited by Begin()")
        .def_readwrite("group_stack", &ImGuiContext::GroupStack, "Stack for BeginGroup()/EndGroup() - not inherited by Begin()")
        .def_readwrite("open_popup_stack", &ImGuiContext::OpenPopupStack, "Which popups are open (persistent)")
        .def_readwrite("begin_popup_stack", &ImGuiContext::BeginPopupStack, "Which level of BeginPopup() we are in (reset every frame)")
        .def_readwrite("nav_tree_node_stack", &ImGuiContext::NavTreeNodeStack, "Stack for TreeNode() when a NavLeft requested is emitted.")
        .def_readwrite("viewports", &ImGuiContext::Viewports, "Active viewports (always 1+, and generally 1 unless multi-viewports are enabled). Each viewports hold their copy of ImDrawData.")
        .def_readwrite("current_dpi_scale", &ImGuiContext::CurrentDpiScale, "== CurrentViewport->DpiScale")
        .def_readwrite("current_viewport", &ImGuiContext::CurrentViewport, "We track changes of viewport (happening in Begin) so we can call Platform_OnChangedViewport()")
        .def_readwrite("mouse_viewport", &ImGuiContext::MouseViewport, "")
        .def_readwrite("mouse_last_hovered_viewport", &ImGuiContext::MouseLastHoveredViewport, "Last known viewport that was hovered by mouse (even if we are not hovering any viewport any more) + honoring the _NoInputs flag.")
        .def_readwrite("platform_last_focused_viewport_id", &ImGuiContext::PlatformLastFocusedViewportId, "")
        .def_readwrite("fallback_monitor", &ImGuiContext::FallbackMonitor, "Virtual monitor used as fallback if backend doesn't provide monitor information.")
        .def_readwrite("platform_monitors_full_work_rect", &ImGuiContext::PlatformMonitorsFullWorkRect, "Bounding box of all platform monitors")
        .def_readwrite("viewport_created_count", &ImGuiContext::ViewportCreatedCount, "Unique sequential creation counter (mostly for testing/debugging)")
        .def_readwrite("platform_windows_created_count", &ImGuiContext::PlatformWindowsCreatedCount, "Unique sequential creation counter (mostly for testing/debugging)")
        .def_readwrite("viewport_focused_stamp_count", &ImGuiContext::ViewportFocusedStampCount, "Every time the front-most window changes, we stamp its viewport with an incrementing counter")
        .def_readwrite("nav_window", &ImGuiContext::NavWindow, "Focused window for navigation. Could be called 'FocusedWindow'")
        .def_readwrite("nav_id", &ImGuiContext::NavId, "Focused item for navigation")
        .def_readwrite("nav_focus_scope_id", &ImGuiContext::NavFocusScopeId, "Focused focus scope (e.g. selection code often wants to \"clear other items\" when landing on an item of the same scope)")
        .def_readwrite("nav_focus_route", &ImGuiContext::NavFocusRoute, "Reversed copy focus scope stack for NavId (should contains NavFocusScopeId). This essentially follow the window->ParentWindowForFocusRoute chain.")
        .def_readwrite("nav_activate_id", &ImGuiContext::NavActivateId, "~~ (g.ActiveId == 0) && (IsKeyPressed(ImGuiKey_Space) || IsKeyDown(ImGuiKey_Enter) || IsKeyPressed(ImGuiKey_NavGamepadActivate)) ? NavId : 0, also set when calling ActivateItem()")
        .def_readwrite("nav_activate_down_id", &ImGuiContext::NavActivateDownId, "~~ IsKeyDown(ImGuiKey_Space) || IsKeyDown(ImGuiKey_Enter) || IsKeyDown(ImGuiKey_NavGamepadActivate) ? NavId : 0")
        .def_readwrite("nav_activate_pressed_id", &ImGuiContext::NavActivatePressedId, "~~ IsKeyPressed(ImGuiKey_Space) || IsKeyPressed(ImGuiKey_Enter) || IsKeyPressed(ImGuiKey_NavGamepadActivate) ? NavId : 0 (no repeat)")
        .def_readwrite("nav_activate_flags", &ImGuiContext::NavActivateFlags, "")
        .def_readwrite("nav_highlight_activated_id", &ImGuiContext::NavHighlightActivatedId, "")
        .def_readwrite("nav_highlight_activated_timer", &ImGuiContext::NavHighlightActivatedTimer, "")
        .def_readwrite("nav_just_moved_to_id", &ImGuiContext::NavJustMovedToId, "Just navigated to this id (result of a successfully MoveRequest).")
        .def_readwrite("nav_just_moved_to_focus_scope_id", &ImGuiContext::NavJustMovedToFocusScopeId, "Just navigated to this focus scope id (result of a successfully MoveRequest).")
        .def_readwrite("nav_just_moved_to_key_mods", &ImGuiContext::NavJustMovedToKeyMods, "")
        .def_readwrite("nav_next_activate_id", &ImGuiContext::NavNextActivateId, "Set by ActivateItem(), queued until next frame.")
        .def_readwrite("nav_next_activate_flags", &ImGuiContext::NavNextActivateFlags, "")
        .def_readwrite("nav_input_source", &ImGuiContext::NavInputSource, "Keyboard or Gamepad mode? THIS CAN ONLY BE ImGuiInputSource_Keyboard or ImGuiInputSource_Mouse")
        .def_readwrite("nav_layer", &ImGuiContext::NavLayer, "Layer we are navigating on. For now the system is hard-coded for 0=main contents and 1=menu/title bar, may expose layers later.")
        .def_readwrite("nav_last_valid_selection_user_data", &ImGuiContext::NavLastValidSelectionUserData, "Last valid data passed to SetNextItemSelectionUser(), or -1. For current window. Not reset when focusing an item that doesn't have selection data.")
        .def_readwrite("nav_id_is_alive", &ImGuiContext::NavIdIsAlive, "Nav widget has been seen this frame ~~ NavRectRel is valid")
        .def_readwrite("nav_mouse_pos_dirty", &ImGuiContext::NavMousePosDirty, "When set we will update mouse position if (io.ConfigFlags & ImGuiConfigFlags_NavEnableSetMousePos) if set (NB: this not enabled by default)")
        .def_readwrite("nav_disable_highlight", &ImGuiContext::NavDisableHighlight, "When user starts using mouse, we hide gamepad/keyboard highlight (NB: but they are still available, which is why NavDisableHighlight isn't always != NavDisableMouseHover)")
        .def_readwrite("nav_disable_mouse_hover", &ImGuiContext::NavDisableMouseHover, "When user starts using gamepad/keyboard, we hide mouse hovering highlight until mouse is touched again.")
        .def_readwrite("nav_any_request", &ImGuiContext::NavAnyRequest, "~~ NavMoveRequest || NavInitRequest this is to perform early out in ItemAdd()")
        .def_readwrite("nav_init_request", &ImGuiContext::NavInitRequest, "Init request for appearing window to select first item")
        .def_readwrite("nav_init_request_from_move", &ImGuiContext::NavInitRequestFromMove, "")
        .def_readwrite("nav_init_result", &ImGuiContext::NavInitResult, "Init request result (first item of the window, or one for which SetItemDefaultFocus() was called)")
        .def_readwrite("nav_move_submitted", &ImGuiContext::NavMoveSubmitted, "Move request submitted, will process result on next NewFrame()")
        .def_readwrite("nav_move_scoring_items", &ImGuiContext::NavMoveScoringItems, "Move request submitted, still scoring incoming items")
        .def_readwrite("nav_move_forward_to_next_frame", &ImGuiContext::NavMoveForwardToNextFrame, "")
        .def_readwrite("nav_move_flags", &ImGuiContext::NavMoveFlags, "")
        .def_readwrite("nav_move_scroll_flags", &ImGuiContext::NavMoveScrollFlags, "")
        .def_readwrite("nav_move_key_mods", &ImGuiContext::NavMoveKeyMods, "")
        .def_readwrite("nav_move_dir", &ImGuiContext::NavMoveDir, "Direction of the move request (left/right/up/down)")
        .def_readwrite("nav_move_dir_for_debug", &ImGuiContext::NavMoveDirForDebug, "")
        .def_readwrite("nav_move_clip_dir", &ImGuiContext::NavMoveClipDir, "FIXME-NAV: Describe the purpose of this better. Might want to rename?")
        .def_readwrite("nav_scoring_rect", &ImGuiContext::NavScoringRect, "Rectangle used for scoring, in screen space. Based of window->NavRectRel[], modified for directional navigation scoring.")
        .def_readwrite("nav_scoring_no_clip_rect", &ImGuiContext::NavScoringNoClipRect, "Some nav operations (such as PageUp/PageDown) enforce a region which clipper will attempt to always keep submitted")
        .def_readwrite("nav_scoring_debug_count", &ImGuiContext::NavScoringDebugCount, "Metrics for debugging")
        .def_readwrite("nav_tabbing_dir", &ImGuiContext::NavTabbingDir, "Generally -1 or +1, 0 when tabbing without a nav id")
        .def_readwrite("nav_tabbing_counter", &ImGuiContext::NavTabbingCounter, ">0 when counting items for tabbing")
        .def_readwrite("nav_move_result_local", &ImGuiContext::NavMoveResultLocal, "Best move request candidate within NavWindow")
        .def_readwrite("nav_move_result_local_visible", &ImGuiContext::NavMoveResultLocalVisible, "Best move request candidate within NavWindow that are mostly visible (when using ImGuiNavMoveFlags_AlsoScoreVisibleSet flag)")
        .def_readwrite("nav_move_result_other", &ImGuiContext::NavMoveResultOther, "Best move request candidate within NavWindow's flattened hierarchy (when using ImGuiWindowFlags_NavFlattened flag)")
        .def_readwrite("nav_tabbing_result_first", &ImGuiContext::NavTabbingResultFirst, "First tabbing request candidate within NavWindow and flattened hierarchy")
        .def_readwrite("config_nav_windowing_key_next", &ImGuiContext::ConfigNavWindowingKeyNext, "= ImGuiMod_Ctrl | ImGuiKey_Tab, for reconfiguration (see #4828)")
        .def_readwrite("config_nav_windowing_key_prev", &ImGuiContext::ConfigNavWindowingKeyPrev, "= ImGuiMod_Ctrl | ImGuiMod_Shift | ImGuiKey_Tab")
        .def_readwrite("nav_windowing_target", &ImGuiContext::NavWindowingTarget, "Target window when doing CTRL+Tab (or Pad Menu + FocusPrev/Next), this window is temporarily displayed top-most!")
        .def_readwrite("nav_windowing_target_anim", &ImGuiContext::NavWindowingTargetAnim, "Record of last valid NavWindowingTarget until DimBgRatio and NavWindowingHighlightAlpha becomes 0.0, so the fade-out can stay on it.")
        .def_readwrite("nav_windowing_list_window", &ImGuiContext::NavWindowingListWindow, "Internal window actually listing the CTRL+Tab contents")
        .def_readwrite("nav_windowing_timer", &ImGuiContext::NavWindowingTimer, "")
        .def_readwrite("nav_windowing_highlight_alpha", &ImGuiContext::NavWindowingHighlightAlpha, "")
        .def_readwrite("nav_windowing_toggle_layer", &ImGuiContext::NavWindowingToggleLayer, "")
        .def_readwrite("nav_windowing_toggle_key", &ImGuiContext::NavWindowingToggleKey, "")
        .def_readwrite("nav_windowing_accum_delta_pos", &ImGuiContext::NavWindowingAccumDeltaPos, "")
        .def_readwrite("nav_windowing_accum_delta_size", &ImGuiContext::NavWindowingAccumDeltaSize, "")
        .def_readwrite("dim_bg_ratio", &ImGuiContext::DimBgRatio, "0.0..1.0 animation when fading in a dimming background (for modal window and CTRL+TAB list)")
        .def_readwrite("drag_drop_active", &ImGuiContext::DragDropActive, "")
        .def_readwrite("drag_drop_within_source", &ImGuiContext::DragDropWithinSource, "Set when within a BeginDragDropXXX/EndDragDropXXX block for a drag source.")
        .def_readwrite("drag_drop_within_target", &ImGuiContext::DragDropWithinTarget, "Set when within a BeginDragDropXXX/EndDragDropXXX block for a drag target.")
        .def_readwrite("drag_drop_source_flags", &ImGuiContext::DragDropSourceFlags, "")
        .def_readwrite("drag_drop_source_frame_count", &ImGuiContext::DragDropSourceFrameCount, "")
        .def_readwrite("drag_drop_mouse_button", &ImGuiContext::DragDropMouseButton, "")
        .def_readwrite("drag_drop_payload", &ImGuiContext::DragDropPayload, "")
        .def_readwrite("drag_drop_target_rect", &ImGuiContext::DragDropTargetRect, "Store rectangle of current target candidate (we favor small targets when overlapping)")
        .def_readwrite("drag_drop_target_clip_rect", &ImGuiContext::DragDropTargetClipRect, "Store ClipRect at the time of item's drawing")
        .def_readwrite("drag_drop_target_id", &ImGuiContext::DragDropTargetId, "")
        .def_readwrite("drag_drop_accept_flags", &ImGuiContext::DragDropAcceptFlags, "")
        .def_readwrite("drag_drop_accept_id_curr_rect_surface", &ImGuiContext::DragDropAcceptIdCurrRectSurface, "Target item surface (we resolve overlapping targets by prioritizing the smaller surface)")
        .def_readwrite("drag_drop_accept_id_curr", &ImGuiContext::DragDropAcceptIdCurr, "Target item id (set at the time of accepting the payload)")
        .def_readwrite("drag_drop_accept_id_prev", &ImGuiContext::DragDropAcceptIdPrev, "Target item id from previous frame (we need to store this to allow for overlapping drag and drop targets)")
        .def_readwrite("drag_drop_accept_frame_count", &ImGuiContext::DragDropAcceptFrameCount, "Last time a target expressed a desire to accept the source")
        .def_readwrite("drag_drop_hold_just_pressed_id", &ImGuiContext::DragDropHoldJustPressedId, "Set when holding a payload just made ButtonBehavior() return a press.")
        .def_readwrite("drag_drop_payload_buf_heap", &ImGuiContext::DragDropPayloadBufHeap, "We don't expose the ImVector<> directly, ImGuiPayload only holds pointer+size")
        .def_readwrite("clipper_temp_data_stacked", &ImGuiContext::ClipperTempDataStacked, "")
        .def_readwrite("clipper_temp_data", &ImGuiContext::ClipperTempData, "")
        .def_readwrite("current_table", &ImGuiContext::CurrentTable, "")
        .def_readwrite("debug_break_in_table", &ImGuiContext::DebugBreakInTable, "Set to break in BeginTable() call.")
        .def_readwrite("tables_temp_data_stacked", &ImGuiContext::TablesTempDataStacked, "Temporary table data size (because we leave previous instances undestructed, we generally don't use TablesTempData.Size)")
        .def_readwrite("tables_temp_data", &ImGuiContext::TablesTempData, "Temporary table data (buffers reused/shared across instances, support nesting)")
        .def_readwrite("tables_last_time_active", &ImGuiContext::TablesLastTimeActive, "Last used timestamp of each tables (SOA, for efficient GC)")
        .def_readwrite("draw_channels_temp_merge_buffer", &ImGuiContext::DrawChannelsTempMergeBuffer, "")
        .def_readwrite("current_tab_bar", &ImGuiContext::CurrentTabBar, "")
        .def_readwrite("current_tab_bar_stack", &ImGuiContext::CurrentTabBarStack, "")
        .def_readwrite("shrink_width_buffer", &ImGuiContext::ShrinkWidthBuffer, "")
        .def_readwrite("hover_item_delay_id", &ImGuiContext::HoverItemDelayId, "")
        .def_readwrite("hover_item_delay_id_previous_frame", &ImGuiContext::HoverItemDelayIdPreviousFrame, "")
        .def_readwrite("hover_item_delay_timer", &ImGuiContext::HoverItemDelayTimer, "Currently used by IsItemHovered()")
        .def_readwrite("hover_item_delay_clear_timer", &ImGuiContext::HoverItemDelayClearTimer, "Currently used by IsItemHovered(): grace time before g.TooltipHoverTimer gets cleared.")
        .def_readwrite("hover_item_unlocked_stationary_id", &ImGuiContext::HoverItemUnlockedStationaryId, "Mouse has once been stationary on this item. Only reset after departing the item.")
        .def_readwrite("hover_window_unlocked_stationary_id", &ImGuiContext::HoverWindowUnlockedStationaryId, "Mouse has once been stationary on this window. Only reset after departing the window.")
        .def_readwrite("mouse_cursor", &ImGuiContext::MouseCursor, "")
        .def_readwrite("mouse_stationary_timer", &ImGuiContext::MouseStationaryTimer, "Time the mouse has been stationary (with some loose heuristic)")
        .def_readwrite("mouse_last_valid_pos", &ImGuiContext::MouseLastValidPos, "")
        .def_readwrite("input_text_state", &ImGuiContext::InputTextState, "")
        .def_readwrite("input_text_deactivated_state", &ImGuiContext::InputTextDeactivatedState, "")
        .def_readwrite("input_text_password_font", &ImGuiContext::InputTextPasswordFont, "")
        .def_readwrite("temp_input_id", &ImGuiContext::TempInputId, "Temporary text input when CTRL+clicking on a slider, etc.")
        .def_readwrite("begin_menu_depth", &ImGuiContext::BeginMenuDepth, "")
        .def_readwrite("begin_combo_depth", &ImGuiContext::BeginComboDepth, "")
        .def_readwrite("color_edit_options", &ImGuiContext::ColorEditOptions, "Store user options for color edit widgets")
        .def_readwrite("color_edit_current_id", &ImGuiContext::ColorEditCurrentID, "Set temporarily while inside of the parent-most ColorEdit4/ColorPicker4 (because they call each others).")
        .def_readwrite("color_edit_saved_id", &ImGuiContext::ColorEditSavedID, "ID we are saving/restoring HS for")
        .def_readwrite("color_edit_saved_hue", &ImGuiContext::ColorEditSavedHue, "Backup of last Hue associated to LastColor, so we can restore Hue in lossy RGB<>HSV round trips")
        .def_readwrite("color_edit_saved_sat", &ImGuiContext::ColorEditSavedSat, "Backup of last Saturation associated to LastColor, so we can restore Saturation in lossy RGB<>HSV round trips")
        .def_readwrite("color_edit_saved_color", &ImGuiContext::ColorEditSavedColor, "RGB value with alpha set to 0.")
        .def_readwrite("color_picker_ref", &ImGuiContext::ColorPickerRef, "Initial/reference color at the time of opening the color picker.")
        .def_readwrite("combo_preview_data", &ImGuiContext::ComboPreviewData, "")
        .def_readwrite("window_resize_border_expected_rect", &ImGuiContext::WindowResizeBorderExpectedRect, "Expected border rect, switch to relative edit if moving")
        .def_readwrite("window_resize_relative_mode", &ImGuiContext::WindowResizeRelativeMode, "")
        .def_readwrite("slider_grab_click_offset", &ImGuiContext::SliderGrabClickOffset, "")
        .def_readwrite("slider_current_accum", &ImGuiContext::SliderCurrentAccum, "Accumulated slider delta when using navigation controls.")
        .def_readwrite("slider_current_accum_dirty", &ImGuiContext::SliderCurrentAccumDirty, "Has the accumulated slider delta changed since last time we tried to apply it?")
        .def_readwrite("drag_current_accum_dirty", &ImGuiContext::DragCurrentAccumDirty, "")
        .def_readwrite("drag_current_accum", &ImGuiContext::DragCurrentAccum, "Accumulator for dragging modification. Always high-precision, not rounded by end-user precision settings")
        .def_readwrite("drag_speed_default_ratio", &ImGuiContext::DragSpeedDefaultRatio, "If speed == 0.0, uses (max-min) * DragSpeedDefaultRatio")
        .def_readwrite("scrollbar_click_delta_to_grab_center", &ImGuiContext::ScrollbarClickDeltaToGrabCenter, "Distance between mouse and center of grab box, normalized in parent space. Use storage?")
        .def_readwrite("disabled_alpha_backup", &ImGuiContext::DisabledAlphaBackup, "Backup for style.Alpha for BeginDisabled()")
        .def_readwrite("disabled_stack_size", &ImGuiContext::DisabledStackSize, "")
        .def_readwrite("lock_mark_edited", &ImGuiContext::LockMarkEdited, "")
        .def_readwrite("tooltip_override_count", &ImGuiContext::TooltipOverrideCount, "")
        .def_readwrite("clipboard_handler_data", &ImGuiContext::ClipboardHandlerData, "If no custom clipboard handler is defined")
        .def_readwrite("menus_id_submitted_this_frame", &ImGuiContext::MenusIdSubmittedThisFrame, "A list of menu IDs that were rendered at least once")
        .def_readwrite("typing_select_state", &ImGuiContext::TypingSelectState, "State for GetTypingSelectRequest()")
        .def_readwrite("platform_ime_data", &ImGuiContext::PlatformImeData, "Data updated by current frame")
        .def_readwrite("platform_ime_data_prev", &ImGuiContext::PlatformImeDataPrev, "Previous frame data (when changing we will call io.SetPlatformImeDataFn")
        .def_readwrite("platform_ime_viewport", &ImGuiContext::PlatformImeViewport, "")
        .def_readwrite("dock_context", &ImGuiContext::DockContext, " Extensions\n FIXME: We could provide an API to register one slot in an array held in ImGuiContext?")
        .def_readwrite("settings_loaded", &ImGuiContext::SettingsLoaded, "")
        .def_readwrite("settings_dirty_timer", &ImGuiContext::SettingsDirtyTimer, "Save .ini Settings to memory when time reaches zero")
        .def_readwrite("settings_ini_data", &ImGuiContext::SettingsIniData, "In memory .ini settings")
        .def_readwrite("settings_handlers", &ImGuiContext::SettingsHandlers, "List of .ini settings handlers")
        .def_readwrite("hook_id_next", &ImGuiContext::HookIdNext, "Next available HookId")
        .def_readwrite("log_enabled", &ImGuiContext::LogEnabled, "Currently capturing")
        .def_readwrite("log_type", &ImGuiContext::LogType, "Capture target")
        .def_readwrite("log_file", &ImGuiContext::LogFile, "If != None log to stdout/ file")
        .def_readwrite("log_buffer", &ImGuiContext::LogBuffer, "Accumulation buffer when log to clipboard. This is pointer so our GImGui static constructor doesn't call heap allocators.")
        .def_readonly("log_next_prefix", &ImGuiContext::LogNextPrefix, "")
        .def_readonly("log_next_suffix", &ImGuiContext::LogNextSuffix, "")
        .def_readwrite("log_line_pos_y", &ImGuiContext::LogLinePosY, "")
        .def_readwrite("log_line_first_item", &ImGuiContext::LogLineFirstItem, "")
        .def_readwrite("log_depth_ref", &ImGuiContext::LogDepthRef, "")
        .def_readwrite("log_depth_to_expand", &ImGuiContext::LogDepthToExpand, "")
        .def_readwrite("log_depth_to_expand_default", &ImGuiContext::LogDepthToExpandDefault, "Default/stored value for LogDepthMaxExpand if not specified in the LogXXX function call.")
        .def_readwrite("debug_log_flags", &ImGuiContext::DebugLogFlags, "")
        .def_readwrite("debug_log_buf", &ImGuiContext::DebugLogBuf, "")
        .def_readwrite("debug_log_index", &ImGuiContext::DebugLogIndex, "")
        .def_readwrite("debug_log_auto_disable_flags", &ImGuiContext::DebugLogAutoDisableFlags, "")
        .def_readwrite("debug_log_auto_disable_frames", &ImGuiContext::DebugLogAutoDisableFrames, "")
        .def_readwrite("debug_locate_frames", &ImGuiContext::DebugLocateFrames, "For DebugLocateItemOnHover(). This is used together with DebugLocateId which is in a hot/cached spot above.")
        .def_readwrite("debug_break_in_locate_id", &ImGuiContext::DebugBreakInLocateId, "Debug break in ItemAdd() call for g.DebugLocateId.")
        .def_readwrite("debug_break_key_chord", &ImGuiContext::DebugBreakKeyChord, "= ImGuiKey_Pause")
        .def_readwrite("debug_begin_return_value_cull_depth", &ImGuiContext::DebugBeginReturnValueCullDepth, "Cycle between 0..9 then wrap around.")
        .def_readwrite("debug_item_picker_active", &ImGuiContext::DebugItemPickerActive, "Item picker is active (started with DebugStartItemPicker())")
        .def_readwrite("debug_item_picker_mouse_button", &ImGuiContext::DebugItemPickerMouseButton, "")
        .def_readwrite("debug_item_picker_break_id", &ImGuiContext::DebugItemPickerBreakId, "Will call IM_DEBUG_BREAK() when encountering this ID")
        .def_readwrite("debug_flash_style_color_time", &ImGuiContext::DebugFlashStyleColorTime, "")
        .def_readwrite("debug_flash_style_color_backup", &ImGuiContext::DebugFlashStyleColorBackup, "")
        .def_readwrite("debug_metrics_config", &ImGuiContext::DebugMetricsConfig, "")
        .def_readwrite("debug_id_stack_tool", &ImGuiContext::DebugIDStackTool, "")
        .def_readwrite("debug_alloc_info", &ImGuiContext::DebugAllocInfo, "")
        .def_readwrite("debug_hovered_dock_node", &ImGuiContext::DebugHoveredDockNode, "Hovered dock node.")
        .def_property("framerate_sec_per_frame",
            [](ImGuiContext &self) -> pybind11::array
            {
                auto dtype = pybind11::dtype(pybind11::format_descriptor<float>::format());
                auto base = pybind11::array(dtype, {60}, {sizeof(float)});
                return pybind11::array(dtype, {60}, {sizeof(float)}, self.FramerateSecPerFrame, base);
            }, [](ImGuiContext& self) {},
            "Calculate estimate of framerate for user over the last 60 frames..")
        .def_readwrite("framerate_sec_per_frame_idx", &ImGuiContext::FramerateSecPerFrameIdx, "")
        .def_readwrite("framerate_sec_per_frame_count", &ImGuiContext::FramerateSecPerFrameCount, "")
        .def_readwrite("framerate_sec_per_frame_accum", &ImGuiContext::FramerateSecPerFrameAccum, "")
        .def_readwrite("want_capture_mouse_next_frame", &ImGuiContext::WantCaptureMouseNextFrame, "Explicit capture override via SetNextFrameWantCaptureMouse()/SetNextFrameWantCaptureKeyboard(). Default to -1.")
        .def_readwrite("want_capture_keyboard_next_frame", &ImGuiContext::WantCaptureKeyboardNextFrame, "\"")
        .def_readwrite("want_text_input_next_frame", &ImGuiContext::WantTextInputNextFrame, "")
        .def_readwrite("temp_buffer", &ImGuiContext::TempBuffer, "Temporary text buffer")
        .def(py::init<ImFontAtlas *>(),
            py::arg("shared_font_atlas"))
        ;


    auto pyClassImGuiWindowTempData =
        py::class_<ImGuiWindowTempData>
            (m, "WindowTempData", " Transient per-window data, reset at the beginning of the frame. This used to be called ImGuiDrawContext, hence the DC variable name in ImGuiWindow.\n (That's theory, in practice the delimitation between ImGuiWindow and ImGuiWindowTempData is quite tenuous and could be reconsidered..)\n (This doesn't need a constructor because we zero-clear it as part of ImGuiWindow and all frame-temporary data are setup on Begin)")
        .def(py::init<>([](
        ImVec2 CursorPos = ImVec2(), ImVec2 CursorPosPrevLine = ImVec2(), ImVec2 CursorStartPos = ImVec2(), ImVec2 CursorMaxPos = ImVec2(), ImVec2 IdealMaxPos = ImVec2(), ImVec2 CurrLineSize = ImVec2(), ImVec2 PrevLineSize = ImVec2(), float CurrLineTextBaseOffset = float(), float PrevLineTextBaseOffset = float(), bool IsSameLine = bool(), bool IsSetPos = bool(), ImVec1 Indent = ImVec1(), ImVec1 ColumnsOffset = ImVec1(), ImVec1 GroupOffset = ImVec1(), ImVec2 CursorStartPosLossyness = ImVec2(), ImGuiNavLayer NavLayerCurrent = ImGuiNavLayer(), short NavLayersActiveMask = short(), short NavLayersActiveMaskNext = short(), bool NavIsScrollPushableX = bool(), bool NavHideHighlightOneFrame = bool(), bool NavWindowHasScrollY = bool(), bool MenuBarAppending = bool(), ImVec2 MenuBarOffset = ImVec2(), ImGuiMenuColumns MenuColumns = ImGuiMenuColumns(), int TreeDepth = int(), ImU32 TreeJumpToParentOnPopMask = ImU32(), ImVector<ImGuiWindow*> ChildWindows = ImVector<ImGuiWindow*>(), int CurrentTableIdx = int(), ImGuiLayoutType LayoutType = ImGuiLayoutType(), ImGuiLayoutType ParentLayoutType = ImGuiLayoutType(), ImU32 ModalDimBgColor = ImU32(), float ItemWidth = float(), float TextWrapPos = float(), ImVector<float> ItemWidthStack = ImVector<float>(), ImVector<float> TextWrapPosStack = ImVector<float>())
        {
            auto r = std::make_unique<ImGuiWindowTempData>();
            r->CursorPos = CursorPos;
            r->CursorPosPrevLine = CursorPosPrevLine;
            r->CursorStartPos = CursorStartPos;
            r->CursorMaxPos = CursorMaxPos;
            r->IdealMaxPos = IdealMaxPos;
            r->CurrLineSize = CurrLineSize;
            r->PrevLineSize = PrevLineSize;
            r->CurrLineTextBaseOffset = CurrLineTextBaseOffset;
            r->PrevLineTextBaseOffset = PrevLineTextBaseOffset;
            r->IsSameLine = IsSameLine;
            r->IsSetPos = IsSetPos;
            r->Indent = Indent;
            r->ColumnsOffset = ColumnsOffset;
            r->GroupOffset = GroupOffset;
            r->CursorStartPosLossyness = CursorStartPosLossyness;
            r->NavLayerCurrent = NavLayerCurrent;
            r->NavLayersActiveMask = NavLayersActiveMask;
            r->NavLayersActiveMaskNext = NavLayersActiveMaskNext;
            r->NavIsScrollPushableX = NavIsScrollPushableX;
            r->NavHideHighlightOneFrame = NavHideHighlightOneFrame;
            r->NavWindowHasScrollY = NavWindowHasScrollY;
            r->MenuBarAppending = MenuBarAppending;
            r->MenuBarOffset = MenuBarOffset;
            r->MenuColumns = MenuColumns;
            r->TreeDepth = TreeDepth;
            r->TreeJumpToParentOnPopMask = TreeJumpToParentOnPopMask;
            r->ChildWindows = ChildWindows;
            r->CurrentTableIdx = CurrentTableIdx;
            r->LayoutType = LayoutType;
            r->ParentLayoutType = ParentLayoutType;
            r->ModalDimBgColor = ModalDimBgColor;
            r->ItemWidth = ItemWidth;
            r->TextWrapPos = TextWrapPos;
            r->ItemWidthStack = ItemWidthStack;
            r->TextWrapPosStack = TextWrapPosStack;
            return r;
        })
        , py::arg("cursor_pos") = ImVec2(), py::arg("cursor_pos_prev_line") = ImVec2(), py::arg("cursor_start_pos") = ImVec2(), py::arg("cursor_max_pos") = ImVec2(), py::arg("ideal_max_pos") = ImVec2(), py::arg("curr_line_size") = ImVec2(), py::arg("prev_line_size") = ImVec2(), py::arg("curr_line_text_base_offset") = float(), py::arg("prev_line_text_base_offset") = float(), py::arg("is_same_line") = bool(), py::arg("is_set_pos") = bool(), py::arg("indent") = ImVec1(), py::arg("columns_offset") = ImVec1(), py::arg("group_offset") = ImVec1(), py::arg("cursor_start_pos_lossyness") = ImVec2(), py::arg("nav_layer_current") = ImGuiNavLayer(), py::arg("nav_layers_active_mask") = short(), py::arg("nav_layers_active_mask_next") = short(), py::arg("nav_is_scroll_pushable_x") = bool(), py::arg("nav_hide_highlight_one_frame") = bool(), py::arg("nav_window_has_scroll_y") = bool(), py::arg("menu_bar_appending") = bool(), py::arg("menu_bar_offset") = ImVec2(), py::arg("menu_columns") = ImGuiMenuColumns(), py::arg("tree_depth") = int(), py::arg("tree_jump_to_parent_on_pop_mask") = ImU32(), py::arg("child_windows") = ImVector<ImGuiWindow*>(), py::arg("current_table_idx") = int(), py::arg("layout_type") = ImGuiLayoutType(), py::arg("parent_layout_type") = ImGuiLayoutType(), py::arg("modal_dim_bg_color") = ImU32(), py::arg("item_width") = float(), py::arg("text_wrap_pos") = float(), py::arg("item_width_stack") = ImVector<float>(), py::arg("text_wrap_pos_stack") = ImVector<float>()
        )
        .def_readwrite("cursor_pos", &ImGuiWindowTempData::CursorPos, "Current emitting position, in absolute coordinates.")
        .def_readwrite("cursor_pos_prev_line", &ImGuiWindowTempData::CursorPosPrevLine, "")
        .def_readwrite("cursor_start_pos", &ImGuiWindowTempData::CursorStartPos, "Initial position after Begin(), generally ~ window position + WindowPadding.")
        .def_readwrite("cursor_max_pos", &ImGuiWindowTempData::CursorMaxPos, "Used to implicitly calculate ContentSize at the beginning of next frame, for scrolling range and auto-resize. Always growing during the frame.")
        .def_readwrite("ideal_max_pos", &ImGuiWindowTempData::IdealMaxPos, "Used to implicitly calculate ContentSizeIdeal at the beginning of next frame, for auto-resize only. Always growing during the frame.")
        .def_readwrite("curr_line_size", &ImGuiWindowTempData::CurrLineSize, "")
        .def_readwrite("prev_line_size", &ImGuiWindowTempData::PrevLineSize, "")
        .def_readwrite("curr_line_text_base_offset", &ImGuiWindowTempData::CurrLineTextBaseOffset, "Baseline offset (0.0 by default on a new line, generally == style.FramePadding.y when a framed item has been added).")
        .def_readwrite("prev_line_text_base_offset", &ImGuiWindowTempData::PrevLineTextBaseOffset, "")
        .def_readwrite("is_same_line", &ImGuiWindowTempData::IsSameLine, "")
        .def_readwrite("is_set_pos", &ImGuiWindowTempData::IsSetPos, "")
        .def_readwrite("indent", &ImGuiWindowTempData::Indent, "Indentation / start position from left of window (increased by TreePush/TreePop, etc.)")
        .def_readwrite("columns_offset", &ImGuiWindowTempData::ColumnsOffset, "Offset to the current column (if ColumnsCurrent > 0). FIXME: This and the above should be a stack to allow use cases like Tree->Column->Tree. Need revamp columns API.")
        .def_readwrite("group_offset", &ImGuiWindowTempData::GroupOffset, "")
        .def_readwrite("cursor_start_pos_lossyness", &ImGuiWindowTempData::CursorStartPosLossyness, "Record the loss of precision of CursorStartPos due to really large scrolling amount. This is used by clipper to compensate and fix the most common use case of large scroll area.")
        .def_readwrite("nav_layer_current", &ImGuiWindowTempData::NavLayerCurrent, "Current layer, 0..31 (we currently only use 0..1)")
        .def_readwrite("nav_layers_active_mask", &ImGuiWindowTempData::NavLayersActiveMask, "Which layers have been written to (result from previous frame)")
        .def_readwrite("nav_layers_active_mask_next", &ImGuiWindowTempData::NavLayersActiveMaskNext, "Which layers have been written to (accumulator for current frame)")
        .def_readwrite("nav_is_scroll_pushable_x", &ImGuiWindowTempData::NavIsScrollPushableX, "Set when current work location may be scrolled horizontally when moving left / right. This is generally always True UNLESS within a column.")
        .def_readwrite("nav_hide_highlight_one_frame", &ImGuiWindowTempData::NavHideHighlightOneFrame, "")
        .def_readwrite("nav_window_has_scroll_y", &ImGuiWindowTempData::NavWindowHasScrollY, "Set per window when scrolling can be used (== ScrollMax.y > 0.0)")
        .def_readwrite("menu_bar_appending", &ImGuiWindowTempData::MenuBarAppending, "FIXME: Remove this")
        .def_readwrite("menu_bar_offset", &ImGuiWindowTempData::MenuBarOffset, "MenuBarOffset.x is sort of equivalent of a per-layer CursorPos.x, saved/restored as we switch to the menu bar. The only situation when MenuBarOffset.y is > 0 if when (SafeAreaPadding.y > FramePadding.y), often used on TVs.")
        .def_readwrite("menu_columns", &ImGuiWindowTempData::MenuColumns, "Simplified columns storage for menu items measurement")
        .def_readwrite("tree_depth", &ImGuiWindowTempData::TreeDepth, "Current tree depth.")
        .def_readwrite("tree_jump_to_parent_on_pop_mask", &ImGuiWindowTempData::TreeJumpToParentOnPopMask, "Store a copy of !g.NavIdIsAlive for TreeDepth 0..31.. Could be turned into a ImU64 if necessary.")
        .def_readwrite("child_windows", &ImGuiWindowTempData::ChildWindows, "")
        .def_readwrite("state_storage", &ImGuiWindowTempData::StateStorage, "Current persistent per-window storage (store e.g. tree node open/close state)")
        .def_readwrite("current_columns", &ImGuiWindowTempData::CurrentColumns, "Current columns set")
        .def_readwrite("current_table_idx", &ImGuiWindowTempData::CurrentTableIdx, "Current table index (into g.Tables)")
        .def_readwrite("layout_type", &ImGuiWindowTempData::LayoutType, "")
        .def_readwrite("parent_layout_type", &ImGuiWindowTempData::ParentLayoutType, "Layout type of parent window at the time of Begin()")
        .def_readwrite("modal_dim_bg_color", &ImGuiWindowTempData::ModalDimBgColor, "")
        .def_readwrite("item_width", &ImGuiWindowTempData::ItemWidth, "Current item width (>0.0: width in pixels, <0.0: align xx pixels to the right of window).")
        .def_readwrite("text_wrap_pos", &ImGuiWindowTempData::TextWrapPos, "Current text wrap pos.")
        .def_readwrite("item_width_stack", &ImGuiWindowTempData::ItemWidthStack, "Store item widths to restore (attention: .back() is not == ItemWidth)")
        .def_readwrite("text_wrap_pos_stack", &ImGuiWindowTempData::TextWrapPosStack, "Store text wrap pos to restore (attention: .back() is not == TextWrapPos)")
        ;


    auto pyClassImGuiWindow =
        py::class_<ImGuiWindow>
            (m, "Window", "Storage for one window")
        .def_readwrite("ctx", &ImGuiWindow::Ctx, "Parent UI context (needs to be set explicitly by parent).")
        .def_readonly("name", &ImGuiWindow::Name, "Window name, owned by the window.")
        .def_readwrite("id_", &ImGuiWindow::ID, "== ImHashStr(Name)")
        .def_readwrite("flags", &ImGuiWindow::Flags, "See enum ImGuiWindowFlags_")
        .def_readwrite("flags_previous_frame", &ImGuiWindow::FlagsPreviousFrame, "See enum ImGuiWindowFlags_")
        .def_readwrite("child_flags", &ImGuiWindow::ChildFlags, "Set when window is a child window. See enum ImGuiChildFlags_")
        .def_readwrite("window_class", &ImGuiWindow::WindowClass, "Advanced users only. Set with SetNextWindowClass()")
        .def_readwrite("viewport", &ImGuiWindow::Viewport, "Always set in Begin(). Inactive windows may have a None value here if their viewport was discarded.")
        .def_readwrite("viewport_id", &ImGuiWindow::ViewportId, "We backup the viewport id (since the viewport may disappear or never be created if the window is inactive)")
        .def_readwrite("viewport_pos", &ImGuiWindow::ViewportPos, "We backup the viewport position (since the viewport may disappear or never be created if the window is inactive)")
        .def_readwrite("viewport_allow_platform_monitor_extend", &ImGuiWindow::ViewportAllowPlatformMonitorExtend, "Reset to -1 every frame (index is guaranteed to be valid between NewFrame..EndFrame), only used in the Appearing frame of a tooltip/popup to enforce clamping to a given monitor")
        .def_readwrite("pos", &ImGuiWindow::Pos, "Position (always rounded-up to nearest pixel)")
        .def_readwrite("size", &ImGuiWindow::Size, "Current size (==SizeFull or collapsed title bar size)")
        .def_readwrite("size_full", &ImGuiWindow::SizeFull, "Size when non collapsed")
        .def_readwrite("content_size", &ImGuiWindow::ContentSize, "Size of contents/scrollable client area (calculated from the extents reach of the cursor) from previous frame. Does not include window decoration or window padding.")
        .def_readwrite("content_size_ideal", &ImGuiWindow::ContentSizeIdeal, "")
        .def_readwrite("content_size_explicit", &ImGuiWindow::ContentSizeExplicit, "Size of contents/scrollable client area explicitly request by the user via SetNextWindowContentSize().")
        .def_readwrite("window_padding", &ImGuiWindow::WindowPadding, "Window padding at the time of Begin().")
        .def_readwrite("window_rounding", &ImGuiWindow::WindowRounding, "Window rounding at the time of Begin(). May be clamped lower to avoid rendering artifacts with title bar, menu bar etc.")
        .def_readwrite("window_border_size", &ImGuiWindow::WindowBorderSize, "Window border size at the time of Begin().")
        .def_readwrite("deco_outer_size_x1", &ImGuiWindow::DecoOuterSizeX1, "Left/Up offsets. Sum of non-scrolling outer decorations (X1 generally == 0.0. Y1 generally = TitleBarHeight + MenuBarHeight). Locked during Begin().")
        .def_readwrite("deco_outer_size_y1", &ImGuiWindow::DecoOuterSizeY1, "Left/Up offsets. Sum of non-scrolling outer decorations (X1 generally == 0.0. Y1 generally = TitleBarHeight + MenuBarHeight). Locked during Begin().")
        .def_readwrite("deco_outer_size_x2", &ImGuiWindow::DecoOuterSizeX2, "Right/Down offsets (X2 generally == ScrollbarSize.x, Y2 == ScrollbarSizes.y).")
        .def_readwrite("deco_outer_size_y2", &ImGuiWindow::DecoOuterSizeY2, "Right/Down offsets (X2 generally == ScrollbarSize.x, Y2 == ScrollbarSizes.y).")
        .def_readwrite("deco_inner_size_x1", &ImGuiWindow::DecoInnerSizeX1, "Applied AFTER/OVER InnerRect. Specialized for Tables as they use specialized form of clipping and frozen rows/columns are inside InnerRect (and not part of regular decoration sizes).")
        .def_readwrite("deco_inner_size_y1", &ImGuiWindow::DecoInnerSizeY1, "Applied AFTER/OVER InnerRect. Specialized for Tables as they use specialized form of clipping and frozen rows/columns are inside InnerRect (and not part of regular decoration sizes).")
        .def_readwrite("name_buf_len", &ImGuiWindow::NameBufLen, "Size of buffer storing Name. May be larger than strlen(Name)!")
        .def_readwrite("move_id", &ImGuiWindow::MoveId, "== window->GetID(\"#MOVE\")")
        .def_readwrite("tab_id", &ImGuiWindow::TabId, "== window->GetID(\"#TAB\")")
        .def_readwrite("child_id", &ImGuiWindow::ChildId, "ID of corresponding item in parent window (for navigation to return from child window to parent window)")
        .def_readwrite("scroll", &ImGuiWindow::Scroll, "")
        .def_readwrite("scroll_max", &ImGuiWindow::ScrollMax, "")
        .def_readwrite("scroll_target", &ImGuiWindow::ScrollTarget, "target scroll position. stored as cursor position with scrolling canceled out, so the highest point is always 0.0. (FLT_MAX for no change)")
        .def_readwrite("scroll_target_center_ratio", &ImGuiWindow::ScrollTargetCenterRatio, "0.0 = scroll so that target position is at top, 0.5 = scroll so that target position is centered")
        .def_readwrite("scroll_target_edge_snap_dist", &ImGuiWindow::ScrollTargetEdgeSnapDist, "0.0 = no snapping, >0.0 snapping threshold")
        .def_readwrite("scrollbar_sizes", &ImGuiWindow::ScrollbarSizes, "Size taken by each scrollbars on their smaller axis. Pay attention! ScrollbarSizes.x == width of the vertical scrollbar, ScrollbarSizes.y = height of the horizontal scrollbar.")
        .def_readwrite("scrollbar_x", &ImGuiWindow::ScrollbarX, "Are scrollbars visible?")
        .def_readwrite("scrollbar_y", &ImGuiWindow::ScrollbarY, "Are scrollbars visible?")
        .def_readwrite("viewport_owned", &ImGuiWindow::ViewportOwned, "")
        .def_readwrite("active", &ImGuiWindow::Active, "Set to True on Begin(), unless Collapsed")
        .def_readwrite("was_active", &ImGuiWindow::WasActive, "")
        .def_readwrite("write_accessed", &ImGuiWindow::WriteAccessed, "Set to True when any widget access the current window")
        .def_readwrite("collapsed", &ImGuiWindow::Collapsed, "Set when collapsing window to become only title-bar")
        .def_readwrite("want_collapse_toggle", &ImGuiWindow::WantCollapseToggle, "")
        .def_readwrite("skip_items", &ImGuiWindow::SkipItems, "Set when items can safely be all clipped (e.g. window not visible or collapsed)")
        .def_readwrite("appearing", &ImGuiWindow::Appearing, "Set during the frame where the window is appearing (or re-appearing)")
        .def_readwrite("hidden", &ImGuiWindow::Hidden, "Do not display (== HiddenFrames*** > 0)")
        .def_readwrite("is_fallback_window", &ImGuiWindow::IsFallbackWindow, "Set on the \"Debug##Default\" window.")
        .def_readwrite("is_explicit_child", &ImGuiWindow::IsExplicitChild, "Set when passed _ChildWindow, left to False by BeginDocked()")
        .def_readwrite("has_close_button", &ImGuiWindow::HasCloseButton, "Set when the window has a close button (p_open != None)")
        .def_readwrite("resize_border_hovered", &ImGuiWindow::ResizeBorderHovered, "Current border being hovered for resize (-1: none, otherwise 0-3)")
        .def_readwrite("resize_border_held", &ImGuiWindow::ResizeBorderHeld, "Current border being held for resize (-1: none, otherwise 0-3)")
        .def_readwrite("begin_count", &ImGuiWindow::BeginCount, "Number of Begin() during the current frame (generally 0 or 1, 1+ if appending via multiple Begin/End pairs)")
        .def_readwrite("begin_count_previous_frame", &ImGuiWindow::BeginCountPreviousFrame, "Number of Begin() during the previous frame")
        .def_readwrite("begin_order_within_parent", &ImGuiWindow::BeginOrderWithinParent, "Begin() order within immediate parent window, if we are a child window. Otherwise 0.")
        .def_readwrite("begin_order_within_context", &ImGuiWindow::BeginOrderWithinContext, "Begin() order within entire imgui context. This is mostly used for debugging submission order related issues.")
        .def_readwrite("focus_order", &ImGuiWindow::FocusOrder, "Order within WindowsFocusOrder[], altered when windows are focused.")
        .def_readwrite("popup_id", &ImGuiWindow::PopupId, "ID in the popup stack when this window is used as a popup/menu (because we use generic Name/ID for recycling)")
        .def_readwrite("auto_fit_frames_x", &ImGuiWindow::AutoFitFramesX, "")
        .def_readwrite("auto_fit_frames_y", &ImGuiWindow::AutoFitFramesY, "")
        .def_readwrite("auto_fit_only_grows", &ImGuiWindow::AutoFitOnlyGrows, "")
        .def_readwrite("auto_pos_last_direction", &ImGuiWindow::AutoPosLastDirection, "")
        .def_readwrite("hidden_frames_can_skip_items", &ImGuiWindow::HiddenFramesCanSkipItems, "Hide the window for N frames")
        .def_readwrite("hidden_frames_cannot_skip_items", &ImGuiWindow::HiddenFramesCannotSkipItems, "Hide the window for N frames while allowing items to be submitted so we can measure their size")
        .def_readwrite("hidden_frames_for_render_only", &ImGuiWindow::HiddenFramesForRenderOnly, "Hide the window until frame N at Render() time only")
        .def_readwrite("disable_inputs_frames", &ImGuiWindow::DisableInputsFrames, "Disable window interactions for N frames")
        .def_readwrite("set_window_pos_val", &ImGuiWindow::SetWindowPosVal, "store window position when using a non-zero Pivot (position set needs to be processed when we know the window size)")
        .def_readwrite("set_window_pos_pivot", &ImGuiWindow::SetWindowPosPivot, "store window pivot for positioning. ImVec2(0, 0) when positioning from top-left corner; ImVec2(0.5, 0.5) for centering; ImVec2(1, 1) for bottom right.")
        .def_readwrite("id_stack", &ImGuiWindow::IDStack, "ID stack. ID are hashes seeded with the value at the top of the stack. (In theory this should be in the TempData structure)")
        .def_readwrite("dc", &ImGuiWindow::DC, "Temporary per-window data, reset at the beginning of the frame. This used to be called ImGuiDrawContext, hence the \"DC\" variable name.")
        .def_readwrite("outer_rect_clipped", &ImGuiWindow::OuterRectClipped, "== Window->Rect() just after setup in Begin(). == window->Rect() for root window.")
        .def_readwrite("inner_rect", &ImGuiWindow::InnerRect, "Inner rectangle (omit title bar, menu bar, scroll bar)")
        .def_readwrite("inner_clip_rect", &ImGuiWindow::InnerClipRect, "== InnerRect shrunk by WindowPadding*0.5 on each side, clipped within viewport or parent clip rect.")
        .def_readwrite("work_rect", &ImGuiWindow::WorkRect, "Initially covers the whole scrolling region. Reduced by containers e.g columns/tables when active. Shrunk by WindowPadding*1.0 on each side. This is meant to replace ContentRegionRect over time (from 1.71+ onward).")
        .def_readwrite("parent_work_rect", &ImGuiWindow::ParentWorkRect, "Backup of WorkRect before entering a container such as columns/tables. Used by e.g. SpanAllColumns functions to easily access. Stacked containers are responsible for maintaining this. // FIXME-WORKRECT: Could be a stack?")
        .def_readwrite("clip_rect", &ImGuiWindow::ClipRect, "Current clipping/scissoring rectangle, evolve as we are using PushClipRect(), etc. == DrawList->clip_rect_stack.back().")
        .def_readwrite("content_region_rect", &ImGuiWindow::ContentRegionRect, "FIXME: This is currently confusing/misleading. It is essentially WorkRect but not handling of scrolling. We currently rely on it as right/bottom aligned sizing operation need some size to rely on.")
        .def_readwrite("hit_test_hole_size", &ImGuiWindow::HitTestHoleSize, "Define an optional rectangular hole where mouse will pass-through the window.")
        .def_readwrite("hit_test_hole_offset", &ImGuiWindow::HitTestHoleOffset, "")
        .def_readwrite("last_frame_active", &ImGuiWindow::LastFrameActive, "Last frame number the window was Active.")
        .def_readwrite("last_frame_just_focused", &ImGuiWindow::LastFrameJustFocused, "Last frame number the window was made Focused.")
        .def_readwrite("last_time_active", &ImGuiWindow::LastTimeActive, "Last timestamp the window was Active (using float as we don't need high precision there)")
        .def_readwrite("item_width_default", &ImGuiWindow::ItemWidthDefault, "")
        .def_readwrite("state_storage", &ImGuiWindow::StateStorage, "")
        .def_readwrite("columns_storage", &ImGuiWindow::ColumnsStorage, "")
        .def_readwrite("font_window_scale", &ImGuiWindow::FontWindowScale, "User scale multiplier per-window, via SetWindowFontScale()")
        .def_readwrite("font_dpi_scale", &ImGuiWindow::FontDpiScale, "")
        .def_readwrite("settings_offset", &ImGuiWindow::SettingsOffset, "Offset into SettingsWindows[] (offsets are always valid as we only grow the array from the back)")
        .def_readwrite("draw_list", &ImGuiWindow::DrawList, "== &DrawListInst (for backward compatibility reason with code using imgui_internal.h we keep this a pointer)")
        .def_readwrite("draw_list_inst", &ImGuiWindow::DrawListInst, "")
        .def_readwrite("parent_window", &ImGuiWindow::ParentWindow, "If we are a child _or_ popup _or_ docked window, this is pointing to our parent. Otherwise None.")
        .def_readwrite("parent_window_in_begin_stack", &ImGuiWindow::ParentWindowInBeginStack, "")
        .def_readwrite("root_window", &ImGuiWindow::RootWindow, "Point to ourself or first ancestor that is not a child window. Doesn't cross through popups/dock nodes.")
        .def_readwrite("root_window_popup_tree", &ImGuiWindow::RootWindowPopupTree, "Point to ourself or first ancestor that is not a child window. Cross through popups parent<>child.")
        .def_readwrite("root_window_dock_tree", &ImGuiWindow::RootWindowDockTree, "Point to ourself or first ancestor that is not a child window. Cross through dock nodes.")
        .def_readwrite("root_window_for_title_bar_highlight", &ImGuiWindow::RootWindowForTitleBarHighlight, "Point to ourself or first ancestor which will display TitleBgActive color when this window is active.")
        .def_readwrite("root_window_for_nav", &ImGuiWindow::RootWindowForNav, "Point to ourself or first ancestor which doesn't have the NavFlattened flag.")
        .def_readwrite("parent_window_for_focus_route", &ImGuiWindow::ParentWindowForFocusRoute, "Set to manual link a window to its logical parent so that Shortcut() chain are honoerd (e.g. Tool linked to Document)")
        .def_readwrite("nav_last_child_nav_window", &ImGuiWindow::NavLastChildNavWindow, "When going to the menu bar, we remember the child window we came from. (This could probably be made implicit if we kept g.Windows sorted by last focused including child window.)")
        .def_readwrite("nav_root_focus_scope_id", &ImGuiWindow::NavRootFocusScopeId, "Focus Scope ID at the time of Begin()")
        .def_readwrite("memory_draw_list_idx_capacity", &ImGuiWindow::MemoryDrawListIdxCapacity, "Backup of last idx/vtx count, so when waking up the window we can preallocate and avoid iterative alloc/copy")
        .def_readwrite("memory_draw_list_vtx_capacity", &ImGuiWindow::MemoryDrawListVtxCapacity, "")
        .def_readwrite("memory_compacted", &ImGuiWindow::MemoryCompacted, "Set when window extraneous data have been garbage collected")
        .def_readwrite("dock_order", &ImGuiWindow::DockOrder, "Order of the last time the window was visible within its DockNode. This is used to reorder windows that are reappearing on the same frame. Same value between windows that were active and windows that were none are possible.")
        .def_readwrite("dock_style", &ImGuiWindow::DockStyle, "")
        .def_readwrite("dock_node", &ImGuiWindow::DockNode, "Which node are we docked into. Important: Prefer testing DockIsActive in many cases as this will still be set when the dock node is hidden.")
        .def_readwrite("dock_node_as_host", &ImGuiWindow::DockNodeAsHost, "Which node are we owning (for parent windows)")
        .def_readwrite("dock_id", &ImGuiWindow::DockId, "Backup of last valid DockNode->ID, so single window remember their dock node id even when they are not bound any more")
        .def_readwrite("dock_tab_item_status_flags", &ImGuiWindow::DockTabItemStatusFlags, "")
        .def_readwrite("dock_tab_item_rect", &ImGuiWindow::DockTabItemRect, "")
        .def(py::init<ImGuiContext *, const char *>(),
            py::arg("context"), py::arg("name"))
        .def("get_id",
            py::overload_cast<const char *, const char *>(&ImGuiWindow::GetID),
            py::arg("str"), py::arg("str_end") = py::none(),
            "(private API)")
        .def("get_id",
            py::overload_cast<const void *>(&ImGuiWindow::GetID),
            py::arg("ptr"),
            "(private API)")
        .def("get_id",
            py::overload_cast<int>(&ImGuiWindow::GetID),
            py::arg("n"),
            "(private API)")
        .def("get_id_from_rectangle",
            &ImGuiWindow::GetIDFromRectangle,
            py::arg("r_abs"),
            "(private API)")
        .def("rect",
            &ImGuiWindow::Rect, "(private API)")
        .def("calc_font_size",
            &ImGuiWindow::CalcFontSize, "(private API)")
        .def("title_bar_height",
            &ImGuiWindow::TitleBarHeight, "(private API)")
        .def("title_bar_rect",
            &ImGuiWindow::TitleBarRect, "(private API)")
        .def("menu_bar_height",
            &ImGuiWindow::MenuBarHeight, "(private API)")
        .def("menu_bar_rect",
            &ImGuiWindow::MenuBarRect, "(private API)")
        ;


    py::enum_<ImGuiTabBarFlagsPrivate_>(m, "TabBarFlagsPrivate_", py::arithmetic(), "Extend ImGuiTabBarFlags_")
        .value("im_gui_tab_bar_flags_dock_node", ImGuiTabBarFlags_DockNode, "Part of a dock node [we don't use this in the master branch but it facilitate branch syncing to keep this around]")
        .value("im_gui_tab_bar_flags_is_focused", ImGuiTabBarFlags_IsFocused, "")
        .value("im_gui_tab_bar_flags_save_settings", ImGuiTabBarFlags_SaveSettings, "FIXME: Settings are handled by the docking system, this only request the tab bar to mark settings dirty when reordering tabs");


    py::enum_<ImGuiTabItemFlagsPrivate_>(m, "TabItemFlagsPrivate_", py::arithmetic(), "Extend ImGuiTabItemFlags_")
        .value("im_gui_tab_item_flags_section_mask_", ImGuiTabItemFlags_SectionMask_, "")
        .value("im_gui_tab_item_flags_no_close_button", ImGuiTabItemFlags_NoCloseButton, "Track whether p_open was set or not (we'll need this info on the next frame to recompute ContentWidth during layout)")
        .value("im_gui_tab_item_flags_button", ImGuiTabItemFlags_Button, "Used by TabItemButton, change the tab item behavior to mimic a button")
        .value("im_gui_tab_item_flags_unsorted", ImGuiTabItemFlags_Unsorted, "[Docking] Trailing tabs with the _Unsorted flag will be sorted based on the DockOrder of their Window.");


    auto pyClassImGuiTabItem =
        py::class_<ImGuiTabItem>
            (m, "TabItem", "Storage for one active tab item (sizeof() 48 bytes)")
        .def_readwrite("id_", &ImGuiTabItem::ID, "")
        .def_readwrite("flags", &ImGuiTabItem::Flags, "")
        .def_readwrite("window", &ImGuiTabItem::Window, "When TabItem is part of a DockNode's TabBar, we hold on to a window.")
        .def_readwrite("last_frame_visible", &ImGuiTabItem::LastFrameVisible, "")
        .def_readwrite("last_frame_selected", &ImGuiTabItem::LastFrameSelected, "This allows us to infer an ordered list of the last activated tabs with little maintenance")
        .def_readwrite("offset", &ImGuiTabItem::Offset, "Position relative to beginning of tab")
        .def_readwrite("width", &ImGuiTabItem::Width, "Width currently displayed")
        .def_readwrite("content_width", &ImGuiTabItem::ContentWidth, "Width of label, stored during BeginTabItem() call")
        .def_readwrite("requested_width", &ImGuiTabItem::RequestedWidth, "Width optionally requested by caller, -1.0 is unused")
        .def_readwrite("name_offset", &ImGuiTabItem::NameOffset, "When Window==None, offset to name within parent ImGuiTabBar::TabsNames")
        .def_readwrite("begin_order", &ImGuiTabItem::BeginOrder, "BeginTabItem() order, used to re-order tabs after toggling ImGuiTabBarFlags_Reorderable")
        .def_readwrite("index_during_layout", &ImGuiTabItem::IndexDuringLayout, "Index only used during TabBarLayout(). Tabs gets reordered so 'Tabs[n].IndexDuringLayout == n' but may mismatch during additions.")
        .def_readwrite("want_close", &ImGuiTabItem::WantClose, "Marked as closed by SetTabItemClosed()")
        .def(py::init<>())
        ;


    auto pyClassImGuiTabBar =
        py::class_<ImGuiTabBar>
            (m, "TabBar", "Storage for a tab bar (sizeof() 152 bytes)")
        .def_readwrite("tabs", &ImGuiTabBar::Tabs, "")
        .def_readwrite("flags", &ImGuiTabBar::Flags, "")
        .def_readwrite("id_", &ImGuiTabBar::ID, "Zero for tab-bars used by docking")
        .def_readwrite("selected_tab_id", &ImGuiTabBar::SelectedTabId, "Selected tab/window")
        .def_readwrite("next_selected_tab_id", &ImGuiTabBar::NextSelectedTabId, "Next selected tab/window. Will also trigger a scrolling animation")
        .def_readwrite("visible_tab_id", &ImGuiTabBar::VisibleTabId, "Can occasionally be != SelectedTabId (e.g. when previewing contents for CTRL+TAB preview)")
        .def_readwrite("curr_frame_visible", &ImGuiTabBar::CurrFrameVisible, "")
        .def_readwrite("prev_frame_visible", &ImGuiTabBar::PrevFrameVisible, "")
        .def_readwrite("bar_rect", &ImGuiTabBar::BarRect, "")
        .def_readwrite("curr_tabs_contents_height", &ImGuiTabBar::CurrTabsContentsHeight, "")
        .def_readwrite("prev_tabs_contents_height", &ImGuiTabBar::PrevTabsContentsHeight, "Record the height of contents submitted below the tab bar")
        .def_readwrite("width_all_tabs", &ImGuiTabBar::WidthAllTabs, "Actual width of all tabs (locked during layout)")
        .def_readwrite("width_all_tabs_ideal", &ImGuiTabBar::WidthAllTabsIdeal, "Ideal width if all tabs were visible and not clipped")
        .def_readwrite("scrolling_anim", &ImGuiTabBar::ScrollingAnim, "")
        .def_readwrite("scrolling_target", &ImGuiTabBar::ScrollingTarget, "")
        .def_readwrite("scrolling_target_dist_to_visibility", &ImGuiTabBar::ScrollingTargetDistToVisibility, "")
        .def_readwrite("scrolling_speed", &ImGuiTabBar::ScrollingSpeed, "")
        .def_readwrite("scrolling_rect_min_x", &ImGuiTabBar::ScrollingRectMinX, "")
        .def_readwrite("scrolling_rect_max_x", &ImGuiTabBar::ScrollingRectMaxX, "")
        .def_readwrite("separator_min_x", &ImGuiTabBar::SeparatorMinX, "")
        .def_readwrite("separator_max_x", &ImGuiTabBar::SeparatorMaxX, "")
        .def_readwrite("reorder_request_tab_id", &ImGuiTabBar::ReorderRequestTabId, "")
        .def_readwrite("reorder_request_offset", &ImGuiTabBar::ReorderRequestOffset, "")
        .def_readwrite("begin_count", &ImGuiTabBar::BeginCount, "")
        .def_readwrite("want_layout", &ImGuiTabBar::WantLayout, "")
        .def_readwrite("visible_tab_was_submitted", &ImGuiTabBar::VisibleTabWasSubmitted, "")
        .def_readwrite("tabs_added_new", &ImGuiTabBar::TabsAddedNew, "Set to True when a new tab item or button has been added to the tab bar during last frame")
        .def_readwrite("tabs_active_count", &ImGuiTabBar::TabsActiveCount, "Number of tabs submitted this frame.")
        .def_readwrite("last_tab_item_idx", &ImGuiTabBar::LastTabItemIdx, "Index of last BeginTabItem() tab for use by EndTabItem()")
        .def_readwrite("item_spacing_y", &ImGuiTabBar::ItemSpacingY, "")
        .def_readwrite("frame_padding", &ImGuiTabBar::FramePadding, "style.FramePadding locked at the time of BeginTabBar()")
        .def_readwrite("backup_cursor_pos", &ImGuiTabBar::BackupCursorPos, "")
        .def_readwrite("tabs_names", &ImGuiTabBar::TabsNames, "For non-docking tab bar we re-append names in a contiguous buffer.")
        .def(py::init<>())
        ;


    auto pyClassImGuiTableColumn =
        py::class_<ImGuiTableColumn>
            (m, "TableColumn", " [Internal] sizeof() ~ 112\n We use the terminology \"Enabled\" to refer to a column that is not Hidden by user/api.\n We use the terminology \"Clipped\" to refer to a column that is out of sight because of scrolling/clipping.\n This is in contrast with some user-facing api such as IsItemVisible() / IsRectVisible() which use \"Visible\" to mean \"not clipped\".")
        .def_readwrite("flags", &ImGuiTableColumn::Flags, "Flags after some patching (not directly same as provided by user). See ImGuiTableColumnFlags_")
        .def_readwrite("width_given", &ImGuiTableColumn::WidthGiven, "Final/actual width visible == (MaxX - MinX), locked in TableUpdateLayout(). May be > WidthRequest to honor minimum width, may be < WidthRequest to honor shrinking columns down in tight space.")
        .def_readwrite("min_x", &ImGuiTableColumn::MinX, "Absolute positions")
        .def_readwrite("max_x", &ImGuiTableColumn::MaxX, "")
        .def_readwrite("width_request", &ImGuiTableColumn::WidthRequest, "Master width absolute value when !(Flags & _WidthStretch). When Stretch this is derived every frame from StretchWeight in TableUpdateLayout()")
        .def_readwrite("width_auto", &ImGuiTableColumn::WidthAuto, "Automatic width")
        .def_readwrite("stretch_weight", &ImGuiTableColumn::StretchWeight, "Master width weight when (Flags & _WidthStretch). Often around ~1.0 initially.")
        .def_readwrite("init_stretch_weight_or_width", &ImGuiTableColumn::InitStretchWeightOrWidth, "Value passed to TableSetupColumn(). For Width it is a content width (_without padding_).")
        .def_readwrite("clip_rect", &ImGuiTableColumn::ClipRect, "Clipping rectangle for the column")
        .def_readwrite("user_id", &ImGuiTableColumn::UserID, "Optional, value passed to TableSetupColumn()")
        .def_readwrite("work_min_x", &ImGuiTableColumn::WorkMinX, "Contents region min ~(MinX + CellPaddingX + CellSpacingX1) == cursor start position when entering column")
        .def_readwrite("work_max_x", &ImGuiTableColumn::WorkMaxX, "Contents region max ~(MaxX - CellPaddingX - CellSpacingX2)")
        .def_readwrite("item_width", &ImGuiTableColumn::ItemWidth, "Current item width for the column, preserved across rows")
        .def_readwrite("content_max_x_frozen", &ImGuiTableColumn::ContentMaxXFrozen, "Contents maximum position for frozen rows (apart from headers), from which we can infer content width.")
        .def_readwrite("content_max_x_unfrozen", &ImGuiTableColumn::ContentMaxXUnfrozen, "")
        .def_readwrite("content_max_x_headers_used", &ImGuiTableColumn::ContentMaxXHeadersUsed, "Contents maximum position for headers rows (regardless of freezing). TableHeader() automatically softclip itself + report ideal desired size, to avoid creating extraneous draw calls")
        .def_readwrite("content_max_x_headers_ideal", &ImGuiTableColumn::ContentMaxXHeadersIdeal, "")
        .def_readwrite("name_offset", &ImGuiTableColumn::NameOffset, "Offset into parent ColumnsNames[]")
        .def_readwrite("display_order", &ImGuiTableColumn::DisplayOrder, "Index within Table's IndexToDisplayOrder[] (column may be reordered by users)")
        .def_readwrite("index_within_enabled_set", &ImGuiTableColumn::IndexWithinEnabledSet, "Index within enabled/visible set (<= IndexToDisplayOrder)")
        .def_readwrite("prev_enabled_column", &ImGuiTableColumn::PrevEnabledColumn, "Index of prev enabled/visible column within Columns[], -1 if first enabled/visible column")
        .def_readwrite("next_enabled_column", &ImGuiTableColumn::NextEnabledColumn, "Index of next enabled/visible column within Columns[], -1 if last enabled/visible column")
        .def_readwrite("sort_order", &ImGuiTableColumn::SortOrder, "Index of this column within sort specs, -1 if not sorting on this column, 0 for single-sort, may be >0 on multi-sort")
        .def_readwrite("draw_channel_current", &ImGuiTableColumn::DrawChannelCurrent, "Index within DrawSplitter.Channels[]")
        .def_readwrite("draw_channel_frozen", &ImGuiTableColumn::DrawChannelFrozen, "Draw channels for frozen rows (often headers)")
        .def_readwrite("draw_channel_unfrozen", &ImGuiTableColumn::DrawChannelUnfrozen, "Draw channels for unfrozen rows")
        .def_readwrite("is_enabled", &ImGuiTableColumn::IsEnabled, "IsUserEnabled && (Flags & ImGuiTableColumnFlags_Disabled) == 0")
        .def_readwrite("is_user_enabled", &ImGuiTableColumn::IsUserEnabled, "Is the column not marked Hidden by the user? (unrelated to being off view, e.g. clipped by scrolling).")
        .def_readwrite("is_user_enabled_next_frame", &ImGuiTableColumn::IsUserEnabledNextFrame, "")
        .def_readwrite("is_visible_x", &ImGuiTableColumn::IsVisibleX, "Is actually in view (e.g. overlapping the host window clipping rectangle, not scrolled).")
        .def_readwrite("is_visible_y", &ImGuiTableColumn::IsVisibleY, "")
        .def_readwrite("is_request_output", &ImGuiTableColumn::IsRequestOutput, "Return value for TableSetColumnIndex() / TableNextColumn(): whether we request user to output contents or not.")
        .def_readwrite("is_skip_items", &ImGuiTableColumn::IsSkipItems, "Do we want item submissions to this column to be completely ignored (no layout will happen).")
        .def_readwrite("is_preserve_width_auto", &ImGuiTableColumn::IsPreserveWidthAuto, "")
        .def_readwrite("nav_layer_current", &ImGuiTableColumn::NavLayerCurrent, "ImGuiNavLayer in 1 byte")
        .def_readwrite("auto_fit_queue", &ImGuiTableColumn::AutoFitQueue, "Queue of 8 values for the next 8 frames to request auto-fit")
        .def_readwrite("cannot_skip_items_queue", &ImGuiTableColumn::CannotSkipItemsQueue, "Queue of 8 values for the next 8 frames to disable Clipped/SkipItem")
        .def_readwrite("sort_directions_avail_list", &ImGuiTableColumn::SortDirectionsAvailList, "Ordered list of available sort directions (2-bits each, total 8-bits)")
        .def(py::init<>())
        ;


    auto pyClassImGuiTableCellData =
        py::class_<ImGuiTableCellData>
            (m, "TableCellData", " Transient cell data stored per row.\n sizeof() ~ 6")
        .def(py::init<>([](
        ImU32 BgColor = ImU32(), ImGuiTableColumnIdx Column = ImGuiTableColumnIdx())
        {
            auto r = std::make_unique<ImGuiTableCellData>();
            r->BgColor = BgColor;
            r->Column = Column;
            return r;
        })
        , py::arg("bg_color") = ImU32(), py::arg("column") = ImGuiTableColumnIdx()
        )
        .def_readwrite("bg_color", &ImGuiTableCellData::BgColor, "Actual color")
        .def_readwrite("column", &ImGuiTableCellData::Column, "Column number")
        ;


    auto pyClassImGuiTableInstanceData =
        py::class_<ImGuiTableInstanceData>
            (m, "TableInstanceData", " Per-instance data that needs preserving across frames (seemingly most others do not need to be preserved aside from debug needs. Does that means they could be moved to ImGuiTableTempData?)\n sizeof() ~ 24 bytes")
        .def_readwrite("table_instance_id", &ImGuiTableInstanceData::TableInstanceID, "")
        .def_readwrite("last_outer_height", &ImGuiTableInstanceData::LastOuterHeight, "Outer height from last frame")
        .def_readwrite("last_top_headers_row_height", &ImGuiTableInstanceData::LastTopHeadersRowHeight, "Height of first consecutive header rows from last frame (FIXME: this is used assuming consecutive headers are in same frozen set)")
        .def_readwrite("last_frozen_height", &ImGuiTableInstanceData::LastFrozenHeight, "Height of frozen section from last frame")
        .def_readwrite("hovered_row_last", &ImGuiTableInstanceData::HoveredRowLast, "Index of row which was hovered last frame.")
        .def_readwrite("hovered_row_next", &ImGuiTableInstanceData::HoveredRowNext, "Index of row hovered this frame, set after encountering it.")
        .def(py::init<>())
        ;


    auto pyClassImGuiTable =
        py::class_<ImGuiTable>
            (m, "Table", " FIXME-TABLE: more transient data could be stored in a stacked ImGuiTableTempData: e.g. SortSpecs, incoming RowData\n sizeof() ~ 580 bytes + heap allocs described in TableBeginInitMemory()")
        .def_readwrite("id_", &ImGuiTable::ID, "")
        .def_readwrite("flags", &ImGuiTable::Flags, "")
        .def_readwrite("raw_data", &ImGuiTable::RawData, "Single allocation to hold Columns[], DisplayOrderToIndex[] and RowCellData[]")
        .def_readwrite("temp_data", &ImGuiTable::TempData, "Transient data while table is active. Point within g.CurrentTableStack[]")
        .def_readwrite("settings_loaded_flags", &ImGuiTable::SettingsLoadedFlags, "Which data were loaded from the .ini file (e.g. when order is not altered we won't save order)")
        .def_readwrite("settings_offset", &ImGuiTable::SettingsOffset, "Offset in g.SettingsTables")
        .def_readwrite("last_frame_active", &ImGuiTable::LastFrameActive, "")
        .def_readwrite("columns_count", &ImGuiTable::ColumnsCount, "Number of columns declared in BeginTable()")
        .def_readwrite("current_row", &ImGuiTable::CurrentRow, "")
        .def_readwrite("current_column", &ImGuiTable::CurrentColumn, "")
        .def_readwrite("instance_current", &ImGuiTable::InstanceCurrent, "Count of BeginTable() calls with same ID in the same frame (generally 0). This is a little bit similar to BeginCount for a window, but multiple table with same ID look are multiple tables, they are just synched.")
        .def_readwrite("instance_interacted", &ImGuiTable::InstanceInteracted, "Mark which instance (generally 0) of the same ID is being interacted with")
        .def_readwrite("row_pos_y1", &ImGuiTable::RowPosY1, "")
        .def_readwrite("row_pos_y2", &ImGuiTable::RowPosY2, "")
        .def_readwrite("row_min_height", &ImGuiTable::RowMinHeight, "Height submitted to TableNextRow()")
        .def_readwrite("row_cell_padding_y", &ImGuiTable::RowCellPaddingY, "Top and bottom padding. Reloaded during row change.")
        .def_readwrite("row_text_baseline", &ImGuiTable::RowTextBaseline, "")
        .def_readwrite("row_indent_offset_x", &ImGuiTable::RowIndentOffsetX, "")
        .def_readwrite("row_bg_color_counter", &ImGuiTable::RowBgColorCounter, "Counter for alternating background colors (can be fast-forwarded by e.g clipper), not same as CurrentRow because header rows typically don't increase this.")
        .def_property("row_bg_color",
            [](ImGuiTable &self) -> pybind11::array
            {
                auto dtype = pybind11::dtype(pybind11::format_descriptor<ImU32>::format());
                auto base = pybind11::array(dtype, {2}, {sizeof(ImU32)});
                return pybind11::array(dtype, {2}, {sizeof(ImU32)}, self.RowBgColor, base);
            }, [](ImGuiTable& self) {},
            "Background color override for current row.")
        .def_readwrite("border_color_strong", &ImGuiTable::BorderColorStrong, "")
        .def_readwrite("border_color_light", &ImGuiTable::BorderColorLight, "")
        .def_readwrite("border_x1", &ImGuiTable::BorderX1, "")
        .def_readwrite("border_x2", &ImGuiTable::BorderX2, "")
        .def_readwrite("host_indent_x", &ImGuiTable::HostIndentX, "")
        .def_readwrite("min_column_width", &ImGuiTable::MinColumnWidth, "")
        .def_readwrite("outer_padding_x", &ImGuiTable::OuterPaddingX, "")
        .def_readwrite("cell_padding_x", &ImGuiTable::CellPaddingX, "Padding from each borders. Locked in BeginTable()/Layout.")
        .def_readwrite("cell_spacing_x1", &ImGuiTable::CellSpacingX1, "Spacing between non-bordered cells. Locked in BeginTable()/Layout.")
        .def_readwrite("cell_spacing_x2", &ImGuiTable::CellSpacingX2, "")
        .def_readwrite("inner_width", &ImGuiTable::InnerWidth, "User value passed to BeginTable(), see comments at the top of BeginTable() for details.")
        .def_readwrite("columns_given_width", &ImGuiTable::ColumnsGivenWidth, "Sum of current column width")
        .def_readwrite("columns_auto_fit_width", &ImGuiTable::ColumnsAutoFitWidth, "Sum of ideal column width in order nothing to be clipped, used for auto-fitting and content width submission in outer window")
        .def_readwrite("columns_stretch_sum_weights", &ImGuiTable::ColumnsStretchSumWeights, "Sum of weight of all enabled stretching columns")
        .def_readwrite("resized_column_next_width", &ImGuiTable::ResizedColumnNextWidth, "")
        .def_readwrite("resize_lock_min_contents_x2", &ImGuiTable::ResizeLockMinContentsX2, "Lock minimum contents width while resizing down in order to not create feedback loops. But we allow growing the table.")
        .def_readwrite("ref_scale", &ImGuiTable::RefScale, "Reference scale to be able to rescale columns on font/dpi changes.")
        .def_readwrite("angled_headers_height", &ImGuiTable::AngledHeadersHeight, "Set by TableAngledHeadersRow(), used in TableUpdateLayout()")
        .def_readwrite("angled_headers_slope", &ImGuiTable::AngledHeadersSlope, "Set by TableAngledHeadersRow(), used in TableUpdateLayout()")
        .def_readwrite("outer_rect", &ImGuiTable::OuterRect, "Note: for non-scrolling table, OuterRect.Max.y is often FLT_MAX until EndTable(), unless a height has been specified in BeginTable().")
        .def_readwrite("inner_rect", &ImGuiTable::InnerRect, "InnerRect but without decoration. As with OuterRect, for non-scrolling tables, InnerRect.Max.y is")
        .def_readwrite("work_rect", &ImGuiTable::WorkRect, "")
        .def_readwrite("inner_clip_rect", &ImGuiTable::InnerClipRect, "")
        .def_readwrite("bg_clip_rect", &ImGuiTable::BgClipRect, "We use this to cpu-clip cell background color fill, evolve during the frame as we cross frozen rows boundaries")
        .def_readwrite("bg0_clip_rect_for_draw_cmd", &ImGuiTable::Bg0ClipRectForDrawCmd, "Actual ImDrawCmd clip rect for BG0/1 channel. This tends to be == OuterWindow->ClipRect at BeginTable() because output in BG0/BG1 is cpu-clipped")
        .def_readwrite("bg2_clip_rect_for_draw_cmd", &ImGuiTable::Bg2ClipRectForDrawCmd, "Actual ImDrawCmd clip rect for BG2 channel. This tends to be a correct, tight-fit, because output to BG2 are done by widgets relying on regular ClipRect.")
        .def_readwrite("host_clip_rect", &ImGuiTable::HostClipRect, "This is used to check if we can eventually merge our columns draw calls into the current draw call of the current window.")
        .def_readwrite("host_backup_inner_clip_rect", &ImGuiTable::HostBackupInnerClipRect, "Backup of InnerWindow->ClipRect during PushTableBackground()/PopTableBackground()")
        .def_readwrite("outer_window", &ImGuiTable::OuterWindow, "Parent window for the table")
        .def_readwrite("inner_window", &ImGuiTable::InnerWindow, "Window holding the table data (== OuterWindow or a child window)")
        .def_readwrite("columns_names", &ImGuiTable::ColumnsNames, "Contiguous buffer holding columns names")
        .def_readwrite("draw_splitter", &ImGuiTable::DrawSplitter, "Shortcut to TempData->DrawSplitter while in table. Isolate draw commands per columns to avoid switching clip rect constantly")
        .def_readwrite("instance_data_first", &ImGuiTable::InstanceDataFirst, "")
        .def_readwrite("instance_data_extra", &ImGuiTable::InstanceDataExtra, "FIXME-OPT: Using a small-vector pattern would be good.")
        .def_readwrite("sort_specs_single", &ImGuiTable::SortSpecsSingle, "")
        .def_readwrite("sort_specs_multi", &ImGuiTable::SortSpecsMulti, "FIXME-OPT: Using a small-vector pattern would be good.")
        .def_readwrite("sort_specs", &ImGuiTable::SortSpecs, "Public facing sorts specs, this is what we return in TableGetSortSpecs()")
        .def_readwrite("sort_specs_count", &ImGuiTable::SortSpecsCount, "")
        .def_readwrite("columns_enabled_count", &ImGuiTable::ColumnsEnabledCount, "Number of enabled columns (<= ColumnsCount)")
        .def_readwrite("columns_enabled_fixed_count", &ImGuiTable::ColumnsEnabledFixedCount, "Number of enabled columns (<= ColumnsCount)")
        .def_readwrite("decl_columns_count", &ImGuiTable::DeclColumnsCount, "Count calls to TableSetupColumn()")
        .def_readwrite("angled_headers_count", &ImGuiTable::AngledHeadersCount, "Count columns with angled headers")
        .def_readwrite("hovered_column_body", &ImGuiTable::HoveredColumnBody, "Index of column whose visible region is being hovered. Important: == ColumnsCount when hovering empty region after the right-most column!")
        .def_readwrite("hovered_column_border", &ImGuiTable::HoveredColumnBorder, "Index of column whose right-border is being hovered (for resizing).")
        .def_readwrite("highlight_column_header", &ImGuiTable::HighlightColumnHeader, "Index of column which should be highlighted.")
        .def_readwrite("auto_fit_single_column", &ImGuiTable::AutoFitSingleColumn, "Index of single column requesting auto-fit.")
        .def_readwrite("resized_column", &ImGuiTable::ResizedColumn, "Index of column being resized. Reset when InstanceCurrent==0.")
        .def_readwrite("last_resized_column", &ImGuiTable::LastResizedColumn, "Index of column being resized from previous frame.")
        .def_readwrite("held_header_column", &ImGuiTable::HeldHeaderColumn, "Index of column header being held.")
        .def_readwrite("reorder_column", &ImGuiTable::ReorderColumn, "Index of column being reordered. (not cleared)")
        .def_readwrite("reorder_column_dir", &ImGuiTable::ReorderColumnDir, "-1 or +1")
        .def_readwrite("left_most_enabled_column", &ImGuiTable::LeftMostEnabledColumn, "Index of left-most non-hidden column.")
        .def_readwrite("right_most_enabled_column", &ImGuiTable::RightMostEnabledColumn, "Index of right-most non-hidden column.")
        .def_readwrite("left_most_stretched_column", &ImGuiTable::LeftMostStretchedColumn, "Index of left-most stretched column.")
        .def_readwrite("right_most_stretched_column", &ImGuiTable::RightMostStretchedColumn, "Index of right-most stretched column.")
        .def_readwrite("context_popup_column", &ImGuiTable::ContextPopupColumn, "Column right-clicked on, of -1 if opening context menu from a neutral/empty spot")
        .def_readwrite("freeze_rows_request", &ImGuiTable::FreezeRowsRequest, "Requested frozen rows count")
        .def_readwrite("freeze_rows_count", &ImGuiTable::FreezeRowsCount, "Actual frozen row count (== FreezeRowsRequest, or == 0 when no scrolling offset)")
        .def_readwrite("freeze_columns_request", &ImGuiTable::FreezeColumnsRequest, "Requested frozen columns count")
        .def_readwrite("freeze_columns_count", &ImGuiTable::FreezeColumnsCount, "Actual frozen columns count (== FreezeColumnsRequest, or == 0 when no scrolling offset)")
        .def_readwrite("row_cell_data_current", &ImGuiTable::RowCellDataCurrent, "Index of current RowCellData[] entry in current row")
        .def_readwrite("dummy_draw_channel", &ImGuiTable::DummyDrawChannel, "Redirect non-visible columns here.")
        .def_readwrite("bg2_draw_channel_current", &ImGuiTable::Bg2DrawChannelCurrent, "For Selectable() and other widgets drawing across columns after the freezing line. Index within DrawSplitter.Channels[]")
        .def_readwrite("bg2_draw_channel_unfrozen", &ImGuiTable::Bg2DrawChannelUnfrozen, "")
        .def_readwrite("is_layout_locked", &ImGuiTable::IsLayoutLocked, "Set by TableUpdateLayout() which is called when beginning the first row.")
        .def_readwrite("is_inside_row", &ImGuiTable::IsInsideRow, "Set when inside TableBeginRow()/TableEndRow().")
        .def_readwrite("is_initializing", &ImGuiTable::IsInitializing, "")
        .def_readwrite("is_sort_specs_dirty", &ImGuiTable::IsSortSpecsDirty, "")
        .def_readwrite("is_using_headers", &ImGuiTable::IsUsingHeaders, "Set when the first row had the ImGuiTableRowFlags_Headers flag.")
        .def_readwrite("is_context_popup_open", &ImGuiTable::IsContextPopupOpen, "Set when default context menu is open (also see: ContextPopupColumn, InstanceInteracted).")
        .def_readwrite("disable_default_context_menu", &ImGuiTable::DisableDefaultContextMenu, "Disable default context menu contents. You may submit your own using TableBeginContextMenuPopup()/EndPopup()")
        .def_readwrite("is_settings_request_load", &ImGuiTable::IsSettingsRequestLoad, "")
        .def_readwrite("is_settings_dirty", &ImGuiTable::IsSettingsDirty, "Set when table settings have changed and needs to be reported into ImGuiTableSetttings data.")
        .def_readwrite("is_default_display_order", &ImGuiTable::IsDefaultDisplayOrder, "Set when display order is unchanged from default (DisplayOrder contains 0...Count-1)")
        .def_readwrite("is_reset_all_request", &ImGuiTable::IsResetAllRequest, "")
        .def_readwrite("is_reset_display_order_request", &ImGuiTable::IsResetDisplayOrderRequest, "")
        .def_readwrite("is_unfrozen_rows", &ImGuiTable::IsUnfrozenRows, "Set when we got past the frozen row.")
        .def_readwrite("is_default_sizing_policy", &ImGuiTable::IsDefaultSizingPolicy, "Set if user didn't explicitly set a sizing policy in BeginTable()")
        .def_readwrite("is_active_id_alive_before_table", &ImGuiTable::IsActiveIdAliveBeforeTable, "")
        .def_readwrite("is_active_id_in_table", &ImGuiTable::IsActiveIdInTable, "")
        .def_readwrite("has_scrollbar_y_curr", &ImGuiTable::HasScrollbarYCurr, "Whether ANY instance of this table had a vertical scrollbar during the current frame.")
        .def_readwrite("has_scrollbar_y_prev", &ImGuiTable::HasScrollbarYPrev, "Whether ANY instance of this table had a vertical scrollbar during the previous.")
        .def_readwrite("memory_compacted", &ImGuiTable::MemoryCompacted, "")
        .def_readwrite("host_skip_items", &ImGuiTable::HostSkipItems, "Backup of InnerWindow->SkipItem at the end of BeginTable(), because we will overwrite InnerWindow->SkipItem on a per-column basis")
        .def(py::init<>())
        ;


    auto pyClassImGuiTableTempData =
        py::class_<ImGuiTableTempData>
            (m, "TableTempData", " Transient data that are only needed between BeginTable() and EndTable(), those buffers are shared (1 per level of stacked table).\n - Accessing those requires chasing an extra pointer so for very frequently used data we leave them in the main table structure.\n - We also leave out of this structure data that tend to be particularly useful for debugging/metrics.\n sizeof() ~ 120 bytes.")
        .def_readwrite("table_index", &ImGuiTableTempData::TableIndex, "Index in g.Tables.Buf[] pool")
        .def_readwrite("last_time_active", &ImGuiTableTempData::LastTimeActive, "Last timestamp this structure was used")
        .def_readwrite("angled_headers_extra_width", &ImGuiTableTempData::AngledHeadersExtraWidth, "Used in EndTable()")
        .def_readwrite("user_outer_size", &ImGuiTableTempData::UserOuterSize, "outer_size.x passed to BeginTable()")
        .def_readwrite("draw_splitter", &ImGuiTableTempData::DrawSplitter, "")
        .def_readwrite("host_backup_work_rect", &ImGuiTableTempData::HostBackupWorkRect, "Backup of InnerWindow->WorkRect at the end of BeginTable()")
        .def_readwrite("host_backup_parent_work_rect", &ImGuiTableTempData::HostBackupParentWorkRect, "Backup of InnerWindow->ParentWorkRect at the end of BeginTable()")
        .def_readwrite("host_backup_prev_line_size", &ImGuiTableTempData::HostBackupPrevLineSize, "Backup of InnerWindow->DC.PrevLineSize at the end of BeginTable()")
        .def_readwrite("host_backup_curr_line_size", &ImGuiTableTempData::HostBackupCurrLineSize, "Backup of InnerWindow->DC.CurrLineSize at the end of BeginTable()")
        .def_readwrite("host_backup_cursor_max_pos", &ImGuiTableTempData::HostBackupCursorMaxPos, "Backup of InnerWindow->DC.CursorMaxPos at the end of BeginTable()")
        .def_readwrite("host_backup_columns_offset", &ImGuiTableTempData::HostBackupColumnsOffset, "Backup of OuterWindow->DC.ColumnsOffset at the end of BeginTable()")
        .def_readwrite("host_backup_item_width", &ImGuiTableTempData::HostBackupItemWidth, "Backup of OuterWindow->DC.ItemWidth at the end of BeginTable()")
        .def_readwrite("host_backup_item_width_stack_size", &ImGuiTableTempData::HostBackupItemWidthStackSize, "Backup of OuterWindow->DC.ItemWidthStack.Size at the end of BeginTable()")
        .def(py::init<>())
        ;


    auto pyClassImGuiTableColumnSettings =
        py::class_<ImGuiTableColumnSettings>
            (m, "TableColumnSettings", "sizeof() ~ 12")
        .def_readwrite("width_or_weight", &ImGuiTableColumnSettings::WidthOrWeight, "")
        .def_readwrite("user_id", &ImGuiTableColumnSettings::UserID, "")
        .def_readwrite("index", &ImGuiTableColumnSettings::Index, "")
        .def_readwrite("display_order", &ImGuiTableColumnSettings::DisplayOrder, "")
        .def_readwrite("sort_order", &ImGuiTableColumnSettings::SortOrder, "")
        .def(py::init<>())
        ;


    auto pyClassImGuiTableSettings =
        py::class_<ImGuiTableSettings>
            (m, "TableSettings", "This is designed to be stored in a single ImChunkStream (1 header followed by N ImGuiTableColumnSettings, etc.)")
        .def_readwrite("id_", &ImGuiTableSettings::ID, "Set to 0 to invalidate/delete the setting")
        .def_readwrite("save_flags", &ImGuiTableSettings::SaveFlags, "Indicate data we want to save using the Resizable/Reorderable/Sortable/Hideable flags (could be using its own flags..)")
        .def_readwrite("ref_scale", &ImGuiTableSettings::RefScale, "Reference scale to be able to rescale columns on font/dpi changes.")
        .def_readwrite("columns_count", &ImGuiTableSettings::ColumnsCount, "")
        .def_readwrite("columns_count_max", &ImGuiTableSettings::ColumnsCountMax, "Maximum number of columns this settings instance can store, we can recycle a settings instance with lower number of columns but not higher")
        .def_readwrite("want_apply", &ImGuiTableSettings::WantApply, "Set when loaded from .ini data (to enable merging/loading .ini data into an already running context)")
        .def(py::init<>())
        .def("get_column_settings",
            &ImGuiTableSettings::GetColumnSettings,
            "(private API)",
            pybind11::return_value_policy::reference)
        ;


    m.def("get_current_window_read",
        ImGui::GetCurrentWindowRead,
        "(private API)",
        pybind11::return_value_policy::reference);

    m.def("get_current_window",
        ImGui::GetCurrentWindow, pybind11::return_value_policy::reference);

    m.def("find_window_by_id",
        ImGui::FindWindowByID,
        py::arg("id_"),
        pybind11::return_value_policy::reference);

    m.def("find_window_by_name",
        ImGui::FindWindowByName,
        py::arg("name"),
        pybind11::return_value_policy::reference);

    m.def("update_window_parent_and_root_links",
        ImGui::UpdateWindowParentAndRootLinks, py::arg("window"), py::arg("flags"), py::arg("parent_window"));

    m.def("calc_window_next_auto_fit_size",
        ImGui::CalcWindowNextAutoFitSize, py::arg("window"));

    m.def("is_window_child_of",
        ImGui::IsWindowChildOf, py::arg("window"), py::arg("potential_parent"), py::arg("popup_hierarchy"), py::arg("dock_hierarchy"));

    m.def("is_window_within_begin_stack_of",
        ImGui::IsWindowWithinBeginStackOf, py::arg("window"), py::arg("potential_parent"));

    m.def("is_window_above",
        ImGui::IsWindowAbove, py::arg("potential_above"), py::arg("potential_below"));

    m.def("is_window_nav_focusable",
        ImGui::IsWindowNavFocusable, py::arg("window"));

    m.def("set_window_pos",
        py::overload_cast<ImGuiWindow *, const ImVec2 &, ImGuiCond>(ImGui::SetWindowPos), py::arg("window"), py::arg("pos"), py::arg("cond") = 0);

    m.def("set_window_size",
        py::overload_cast<ImGuiWindow *, const ImVec2 &, ImGuiCond>(ImGui::SetWindowSize), py::arg("window"), py::arg("size"), py::arg("cond") = 0);

    m.def("set_window_collapsed",
        py::overload_cast<ImGuiWindow *, bool, ImGuiCond>(ImGui::SetWindowCollapsed), py::arg("window"), py::arg("collapsed"), py::arg("cond") = 0);

    m.def("set_window_hit_test_hole",
        ImGui::SetWindowHitTestHole, py::arg("window"), py::arg("pos"), py::arg("size"));

    m.def("set_window_hidden_and_skip_items_for_current_frame",
        ImGui::SetWindowHiddenAndSkipItemsForCurrentFrame, py::arg("window"));

    m.def("set_window_parent_window_for_focus_route",
        ImGui::SetWindowParentWindowForFocusRoute,
        py::arg("window"), py::arg("parent_window"),
        "(private API)\n\n You may also use SetNextWindowClass()'s FocusRouteParentWindowId field.");

    m.def("window_rect_abs_to_rel",
        ImGui::WindowRectAbsToRel,
        py::arg("window"), py::arg("r"),
        "(private API)");

    m.def("window_rect_rel_to_abs",
        ImGui::WindowRectRelToAbs,
        py::arg("window"), py::arg("r"),
        "(private API)");

    m.def("window_pos_rel_to_abs",
        ImGui::WindowPosRelToAbs,
        py::arg("window"), py::arg("p"),
        "(private API)");

    m.def("focus_window",
        ImGui::FocusWindow, py::arg("window"), py::arg("flags") = 0);

    m.def("focus_top_most_window_under_one",
        ImGui::FocusTopMostWindowUnderOne, py::arg("under_this_window"), py::arg("ignore_window"), py::arg("filter_viewport"), py::arg("flags"));

    m.def("bring_window_to_focus_front",
        ImGui::BringWindowToFocusFront, py::arg("window"));

    m.def("bring_window_to_display_front",
        ImGui::BringWindowToDisplayFront, py::arg("window"));

    m.def("bring_window_to_display_back",
        ImGui::BringWindowToDisplayBack, py::arg("window"));

    m.def("bring_window_to_display_behind",
        ImGui::BringWindowToDisplayBehind, py::arg("window"), py::arg("above_window"));

    m.def("find_window_display_index",
        ImGui::FindWindowDisplayIndex, py::arg("window"));

    m.def("find_bottom_most_visible_window_within_begin_stack",
        ImGui::FindBottomMostVisibleWindowWithinBeginStack,
        py::arg("window"),
        pybind11::return_value_policy::reference);

    m.def("set_current_font",
        ImGui::SetCurrentFont,
        py::arg("font"),
        "Fonts, drawing");

    m.def("get_default_font",
        ImGui::GetDefaultFont,
        "(private API)",
        pybind11::return_value_policy::reference);

    m.def("get_foreground_draw_list",
        py::overload_cast<ImGuiWindow *>(ImGui::GetForegroundDrawList),
        py::arg("window"),
        "(private API)",
        pybind11::return_value_policy::reference);

    m.def("add_draw_list_to_draw_data_ex",
        py::overload_cast<ImDrawData *, ImVector<ImDrawList*> *, ImDrawList *>(ImGui::AddDrawListToDrawDataEx), py::arg("draw_data"), py::arg("out_list"), py::arg("draw_list"));

    m.def("initialize",
        ImGui::Initialize);

    m.def("shutdown",
        ImGui::Shutdown, "Since 1.60 this is a _private_ function. You can call DestroyContext() to destroy the context created by CreateContext().");

    m.def("update_input_events",
        ImGui::UpdateInputEvents, py::arg("trickle_fast_inputs"));

    m.def("update_hovered_window_and_capture_flags",
        ImGui::UpdateHoveredWindowAndCaptureFlags);

    m.def("start_mouse_moving_window",
        ImGui::StartMouseMovingWindow, py::arg("window"));

    m.def("start_mouse_moving_window_or_node",
        ImGui::StartMouseMovingWindowOrNode, py::arg("window"), py::arg("node"), py::arg("undock"));

    m.def("update_mouse_moving_window_new_frame",
        ImGui::UpdateMouseMovingWindowNewFrame);

    m.def("update_mouse_moving_window_end_frame",
        ImGui::UpdateMouseMovingWindowEndFrame);

    m.def("add_context_hook",
        ImGui::AddContextHook, py::arg("context"), py::arg("hook"));

    m.def("remove_context_hook",
        ImGui::RemoveContextHook, py::arg("context"), py::arg("hook_to_remove"));

    m.def("call_context_hooks",
        ImGui::CallContextHooks, py::arg("context"), py::arg("type"));

    m.def("translate_windows_in_viewport",
        ImGui::TranslateWindowsInViewport, py::arg("viewport"), py::arg("old_pos"), py::arg("new_pos"));

    m.def("scale_windows_in_viewport",
        ImGui::ScaleWindowsInViewport, py::arg("viewport"), py::arg("scale"));

    m.def("destroy_platform_window",
        ImGui::DestroyPlatformWindow, py::arg("viewport"));

    m.def("set_window_viewport",
        ImGui::SetWindowViewport, py::arg("window"), py::arg("viewport"));

    m.def("set_current_viewport",
        ImGui::SetCurrentViewport, py::arg("window"), py::arg("viewport"));

    m.def("get_viewport_platform_monitor",
        ImGui::GetViewportPlatformMonitor,
        py::arg("viewport"),
        pybind11::return_value_policy::reference);

    m.def("find_hovered_viewport_from_platform_window_stack",
        ImGui::FindHoveredViewportFromPlatformWindowStack,
        py::arg("mouse_platform_pos"),
        pybind11::return_value_policy::reference);

    m.def("mark_ini_settings_dirty",
        py::overload_cast<>(ImGui::MarkIniSettingsDirty));

    m.def("mark_ini_settings_dirty",
        py::overload_cast<ImGuiWindow *>(ImGui::MarkIniSettingsDirty), py::arg("window"));

    m.def("clear_ini_settings",
        ImGui::ClearIniSettings);

    m.def("add_settings_handler",
        ImGui::AddSettingsHandler, py::arg("handler"));

    m.def("remove_settings_handler",
        ImGui::RemoveSettingsHandler, py::arg("type_name"));

    m.def("find_settings_handler",
        ImGui::FindSettingsHandler,
        py::arg("type_name"),
        pybind11::return_value_policy::reference);

    m.def("create_new_window_settings",
        ImGui::CreateNewWindowSettings,
        py::arg("name"),
        pybind11::return_value_policy::reference);

    m.def("find_window_settings_by_id",
        ImGui::FindWindowSettingsByID,
        py::arg("id_"),
        pybind11::return_value_policy::reference);

    m.def("find_window_settings_by_window",
        ImGui::FindWindowSettingsByWindow,
        py::arg("window"),
        pybind11::return_value_policy::reference);

    m.def("clear_window_settings",
        ImGui::ClearWindowSettings, py::arg("name"));

    m.def("localize_register_entries",
        ImGui::LocalizeRegisterEntries,
        py::arg("entries"), py::arg("count"),
        "Localization");

    m.def("localize_get_msg",
        ImGui::LocalizeGetMsg,
        py::arg("key"),
        "(private API)",
        pybind11::return_value_policy::reference);

    m.def("set_scroll_x",
        py::overload_cast<ImGuiWindow *, float>(ImGui::SetScrollX), py::arg("window"), py::arg("scroll_x"));

    m.def("set_scroll_y",
        py::overload_cast<ImGuiWindow *, float>(ImGui::SetScrollY), py::arg("window"), py::arg("scroll_y"));

    m.def("set_scroll_from_pos_x",
        py::overload_cast<ImGuiWindow *, float, float>(ImGui::SetScrollFromPosX), py::arg("window"), py::arg("local_x"), py::arg("center_x_ratio"));

    m.def("set_scroll_from_pos_y",
        py::overload_cast<ImGuiWindow *, float, float>(ImGui::SetScrollFromPosY), py::arg("window"), py::arg("local_y"), py::arg("center_y_ratio"));

    m.def("scroll_to_item",
        ImGui::ScrollToItem, py::arg("flags") = 0);

    m.def("scroll_to_rect",
        ImGui::ScrollToRect, py::arg("window"), py::arg("rect"), py::arg("flags") = 0);

    m.def("scroll_to_rect_ex",
        ImGui::ScrollToRectEx, py::arg("window"), py::arg("rect"), py::arg("flags") = 0);

    m.def("scroll_to_bring_rect_into_view",
        ImGui::ScrollToBringRectIntoView,
        py::arg("window"), py::arg("rect"),
        "#ifndef IMGUI_DISABLE_OBSOLETE_FUNCTIONS\n(private API)");

    m.def("get_item_status_flags",
        ImGui::GetItemStatusFlags, "(private API)");

    m.def("get_item_flags",
        ImGui::GetItemFlags, "(private API)");

    m.def("get_active_id",
        ImGui::GetActiveID, "(private API)");

    m.def("get_focus_id",
        ImGui::GetFocusID, "(private API)");

    m.def("set_active_id",
        ImGui::SetActiveID, py::arg("id_"), py::arg("window"));

    m.def("set_focus_id",
        ImGui::SetFocusID, py::arg("id_"), py::arg("window"));

    m.def("clear_active_id",
        ImGui::ClearActiveID);

    m.def("get_hovered_id",
        ImGui::GetHoveredID);

    m.def("set_hovered_id",
        ImGui::SetHoveredID, py::arg("id_"));

    m.def("keep_alive_id",
        ImGui::KeepAliveID, py::arg("id_"));

    m.def("mark_item_edited",
        ImGui::MarkItemEdited,
        py::arg("id_"),
        "Mark data associated to given item as \"edited\", used by IsItemDeactivatedAfterEdit() function.");

    m.def("push_override_id",
        ImGui::PushOverrideID,
        py::arg("id_"),
        "Push given value as-is at the top of the ID stack (whereas PushID combines old and new hashes)");

    m.def("get_id_with_seed",
        py::overload_cast<const char *, const char *, ImGuiID>(ImGui::GetIDWithSeed), py::arg("str_id_begin"), py::arg("str_id_end"), py::arg("seed"));

    m.def("get_id_with_seed",
        py::overload_cast<int, ImGuiID>(ImGui::GetIDWithSeed), py::arg("n"), py::arg("seed"));

    m.def("item_size",
        py::overload_cast<const ImVec2 &, float>(ImGui::ItemSize),
        py::arg("size"), py::arg("text_baseline_y") = -1.0f,
        "Basic Helpers for widget code");

    m.def("item_size",
        py::overload_cast<const ImRect &, float>(ImGui::ItemSize),
        py::arg("bb"), py::arg("text_baseline_y") = -1.0f,
        "(private API)\n\n FIXME: This is a misleading API since we expect CursorPos to be bb.Min.");

    m.def("item_add",
        ImGui::ItemAdd, py::arg("bb"), py::arg("id_"), py::arg("nav_bb") = py::none(), py::arg("extra_flags") = 0);

    m.def("item_hoverable",
        ImGui::ItemHoverable, py::arg("bb"), py::arg("id_"), py::arg("item_flags"));

    m.def("is_window_content_hoverable",
        ImGui::IsWindowContentHoverable, py::arg("window"), py::arg("flags") = 0);

    m.def("is_clipped_ex",
        ImGui::IsClippedEx, py::arg("bb"), py::arg("id_"));

    m.def("set_last_item_data",
        ImGui::SetLastItemData, py::arg("item_id"), py::arg("in_flags"), py::arg("status_flags"), py::arg("item_rect"));

    m.def("calc_item_size",
        ImGui::CalcItemSize, py::arg("size"), py::arg("default_w"), py::arg("default_h"));

    m.def("calc_wrap_width_for_pos",
        ImGui::CalcWrapWidthForPos, py::arg("pos"), py::arg("wrap_pos_x"));

    m.def("push_multi_items_widths",
        ImGui::PushMultiItemsWidths, py::arg("components"), py::arg("width_full"));

    m.def("is_item_toggled_selection",
        ImGui::IsItemToggledSelection, "Was the last item selection toggled? (after Selectable(), TreeNode() etc. We only returns toggle _event_ in order to handle clipping correctly)");

    m.def("get_content_region_max_abs",
        ImGui::GetContentRegionMaxAbs);

    m.def("shrink_widths",
        ImGui::ShrinkWidths, py::arg("items"), py::arg("count"), py::arg("width_excess"));

    m.def("push_item_flag",
        ImGui::PushItemFlag, py::arg("option"), py::arg("enabled"));

    m.def("pop_item_flag",
        ImGui::PopItemFlag);

    m.def("get_style_var_info",
        ImGui::GetStyleVarInfo,
        py::arg("idx"),
        pybind11::return_value_policy::reference);

    m.def("log_begin",
        ImGui::LogBegin,
        py::arg("type"), py::arg("auto_open_depth"),
        "-> BeginCapture() when we design v2 api, for now stay under the radar by using the old name.");

    m.def("log_to_buffer",
        ImGui::LogToBuffer,
        py::arg("auto_open_depth") = -1,
        "Start logging/capturing to internal buffer");

    m.def("log_rendered_text",
        ImGui::LogRenderedText, py::arg("ref_pos"), py::arg("text"), py::arg("text_end") = py::none());

    m.def("log_set_next_text_decoration",
        ImGui::LogSetNextTextDecoration, py::arg("prefix"), py::arg("suffix"));

    m.def("begin_child_ex",
        ImGui::BeginChildEx, py::arg("name"), py::arg("id_"), py::arg("size_arg"), py::arg("child_flags"), py::arg("window_flags"));

    m.def("open_popup_ex",
        py::overload_cast<ImGuiID, ImGuiPopupFlags>(ImGui::OpenPopupEx), py::arg("id_"), py::arg("popup_flags") = ImGuiPopupFlags_None);

    m.def("close_popup_to_level",
        py::overload_cast<int, bool>(ImGui::ClosePopupToLevel), py::arg("remaining"), py::arg("restore_focus_to_window_under_popup"));

    m.def("close_popups_over_window",
        py::overload_cast<ImGuiWindow *, bool>(ImGui::ClosePopupsOverWindow), py::arg("ref_window"), py::arg("restore_focus_to_window_under_popup"));

    m.def("close_popups_except_modals",
        py::overload_cast<>(ImGui::ClosePopupsExceptModals));

    m.def("is_popup_open",
        py::overload_cast<ImGuiID, ImGuiPopupFlags>(ImGui::IsPopupOpen), py::arg("id_"), py::arg("popup_flags"));

    m.def("begin_popup_ex",
        py::overload_cast<ImGuiID, ImGuiWindowFlags>(ImGui::BeginPopupEx), py::arg("id_"), py::arg("extra_flags"));

    m.def("begin_tooltip_ex",
        ImGui::BeginTooltipEx, py::arg("tooltip_flags"), py::arg("extra_window_flags"));

    m.def("begin_tooltip_hidden",
        ImGui::BeginTooltipHidden);

    m.def("get_popup_allowed_extent_rect",
        py::overload_cast<ImGuiWindow *>(ImGui::GetPopupAllowedExtentRect), py::arg("window"));

    m.def("get_top_most_popup_modal",
        py::overload_cast<>(ImGui::GetTopMostPopupModal), pybind11::return_value_policy::reference);

    m.def("get_top_most_and_visible_popup_modal",
        py::overload_cast<>(ImGui::GetTopMostAndVisiblePopupModal), pybind11::return_value_policy::reference);

    m.def("find_blocking_modal",
        ImGui::FindBlockingModal,
        py::arg("window"),
        pybind11::return_value_policy::reference);

    m.def("find_best_window_pos_for_popup",
        py::overload_cast<ImGuiWindow *>(ImGui::FindBestWindowPosForPopup), py::arg("window"));

    m.def("find_best_window_pos_for_popup_ex",
        py::overload_cast<const ImVec2 &, const ImVec2 &, ImGuiDir *, const ImRect &, const ImRect &, ImGuiPopupPositionPolicy>(ImGui::FindBestWindowPosForPopupEx), py::arg("ref_pos"), py::arg("size"), py::arg("last_dir"), py::arg("r_outer"), py::arg("r_avoid"), py::arg("policy"));

    m.def("begin_viewport_side_bar",
        ImGui::BeginViewportSideBar, py::arg("name"), py::arg("viewport"), py::arg("dir"), py::arg("size"), py::arg("window_flags"));

    m.def("begin_menu_ex",
        ImGui::BeginMenuEx, py::arg("label"), py::arg("icon"), py::arg("enabled") = true);

    m.def("menu_item_ex",
        ImGui::MenuItemEx, py::arg("label"), py::arg("icon"), py::arg("shortcut") = py::none(), py::arg("selected") = false, py::arg("enabled") = true);

    m.def("begin_combo_popup",
        py::overload_cast<ImGuiID, const ImRect &, ImGuiComboFlags>(ImGui::BeginComboPopup), py::arg("popup_id"), py::arg("bb"), py::arg("flags"));

    m.def("begin_combo_preview",
        ImGui::BeginComboPreview);

    m.def("end_combo_preview",
        ImGui::EndComboPreview);

    m.def("nav_init_window",
        ImGui::NavInitWindow, py::arg("window"), py::arg("force_reinit"));

    m.def("nav_init_request_apply_result",
        ImGui::NavInitRequestApplyResult);

    m.def("nav_move_request_but_no_result_yet",
        ImGui::NavMoveRequestButNoResultYet);

    m.def("nav_move_request_submit",
        ImGui::NavMoveRequestSubmit, py::arg("move_dir"), py::arg("clip_dir"), py::arg("move_flags"), py::arg("scroll_flags"));

    m.def("nav_move_request_forward",
        ImGui::NavMoveRequestForward, py::arg("move_dir"), py::arg("clip_dir"), py::arg("move_flags"), py::arg("scroll_flags"));

    m.def("nav_move_request_resolve_with_last_item",
        ImGui::NavMoveRequestResolveWithLastItem, py::arg("result"));

    m.def("nav_move_request_resolve_with_past_tree_node",
        ImGui::NavMoveRequestResolveWithPastTreeNode, py::arg("result"), py::arg("tree_node_data"));

    m.def("nav_move_request_cancel",
        ImGui::NavMoveRequestCancel);

    m.def("nav_move_request_apply_result",
        ImGui::NavMoveRequestApplyResult);

    m.def("nav_move_request_try_wrapping",
        ImGui::NavMoveRequestTryWrapping, py::arg("window"), py::arg("move_flags"));

    m.def("nav_highlight_activated",
        ImGui::NavHighlightActivated, py::arg("id_"));

    m.def("nav_clear_preferred_pos_for_axis",
        ImGui::NavClearPreferredPosForAxis, py::arg("axis"));

    m.def("nav_restore_highlight_after_move",
        ImGui::NavRestoreHighlightAfterMove);

    m.def("nav_update_current_window_is_scroll_pushable_x",
        ImGui::NavUpdateCurrentWindowIsScrollPushableX);

    m.def("set_nav_window",
        ImGui::SetNavWindow, py::arg("window"));

    m.def("set_nav_id",
        ImGui::SetNavID, py::arg("id_"), py::arg("nav_layer"), py::arg("focus_scope_id"), py::arg("rect_rel"));

    m.def("set_nav_focus_scope",
        ImGui::SetNavFocusScope, py::arg("focus_scope_id"));

    m.def("focus_item",
        ImGui::FocusItem, "Focus last item (no selection/activation).");

    m.def("activate_item_by_id",
        ImGui::ActivateItemByID,
        py::arg("id_"),
        "Activate an item by ID (button, checkbox, tree node etc.). Activation is queued and processed on the next frame when the item is encountered again.");

    m.def("is_named_key",
        ImGui::IsNamedKey,
        py::arg("key"),
        "(private API)");

    m.def("is_named_key_or_mod_key",
        ImGui::IsNamedKeyOrModKey,
        py::arg("key"),
        "(private API)");

    m.def("is_legacy_key",
        ImGui::IsLegacyKey,
        py::arg("key"),
        "(private API)");

    m.def("is_keyboard_key",
        py::overload_cast<ImGuiKey>(ImGui::IsKeyboardKey),
        py::arg("key"),
        "(private API)");

    m.def("is_gamepad_key",
        ImGui::IsGamepadKey,
        py::arg("key"),
        "(private API)");

    m.def("is_mouse_key",
        py::overload_cast<ImGuiKey>(ImGui::IsMouseKey),
        py::arg("key"),
        "(private API)");

    m.def("is_alias_key",
        ImGui::IsAliasKey,
        py::arg("key"),
        "(private API)");

    m.def("is_mod_key",
        ImGui::IsModKey,
        py::arg("key"),
        "(private API)");

    m.def("fixup_key_chord",
        ImGui::FixupKeyChord,
        py::arg("ctx"), py::arg("key_chord"),
        "(private API)");

    m.def("convert_single_mod_flag_to_key",
        ImGui::ConvertSingleModFlagToKey,
        py::arg("ctx"), py::arg("key"),
        "(private API)");

    m.def("get_key_data",
        py::overload_cast<ImGuiContext *, ImGuiKey>(ImGui::GetKeyData),
        py::arg("ctx"), py::arg("key"),
        pybind11::return_value_policy::reference);

    m.def("get_key_data",
        py::overload_cast<ImGuiKey>(ImGui::GetKeyData),
        py::arg("key"),
        "(private API)",
        pybind11::return_value_policy::reference);

    m.def("mouse_button_to_key",
        ImGui::MouseButtonToKey,
        py::arg("button"),
        "(private API)");

    m.def("is_mouse_drag_past_threshold",
        py::overload_cast<ImGuiMouseButton, float>(ImGui::IsMouseDragPastThreshold), py::arg("button"), py::arg("lock_threshold") = -1.0f);

    m.def("get_key_magnitude2d",
        ImGui::GetKeyMagnitude2d, py::arg("key_left"), py::arg("key_right"), py::arg("key_up"), py::arg("key_down"));

    m.def("get_nav_tweak_pressed_amount",
        ImGui::GetNavTweakPressedAmount, py::arg("axis"));

    m.def("calc_typematic_repeat_amount",
        ImGui::CalcTypematicRepeatAmount, py::arg("t0"), py::arg("t1"), py::arg("repeat_delay"), py::arg("repeat_rate"));

    m.def("get_typematic_repeat_rate",
        [](ImGuiInputFlags flags, float repeat_delay, float repeat_rate) -> std::tuple<float, float>
        {
            auto GetTypematicRepeatRate_adapt_modifiable_immutable_to_return = [](ImGuiInputFlags flags, float repeat_delay, float repeat_rate) -> std::tuple<float, float>
            {
                float * repeat_delay_adapt_modifiable = & repeat_delay;
                float * repeat_rate_adapt_modifiable = & repeat_rate;

                ImGui::GetTypematicRepeatRate(flags, repeat_delay_adapt_modifiable, repeat_rate_adapt_modifiable);
                return std::make_tuple(repeat_delay, repeat_rate);
            };

            return GetTypematicRepeatRate_adapt_modifiable_immutable_to_return(flags, repeat_delay, repeat_rate);
        },     py::arg("flags"), py::arg("repeat_delay"), py::arg("repeat_rate"));

    m.def("teleport_mouse_pos",
        ImGui::TeleportMousePos, py::arg("pos"));

    m.def("set_active_id_using_all_keyboard_keys",
        ImGui::SetActiveIdUsingAllKeyboardKeys);

    m.def("is_active_id_using_nav_dir",
        ImGui::IsActiveIdUsingNavDir,
        py::arg("dir"),
        "(private API)");

    m.def("get_key_owner",
        ImGui::GetKeyOwner, py::arg("key"));

    m.def("set_key_owner",
        ImGui::SetKeyOwner, py::arg("key"), py::arg("owner_id"), py::arg("flags") = 0);

    m.def("set_key_owners_for_key_chord",
        ImGui::SetKeyOwnersForKeyChord, py::arg("key"), py::arg("owner_id"), py::arg("flags") = 0);

    m.def("set_item_key_owner",
        ImGui::SetItemKeyOwner,
        py::arg("key"), py::arg("flags") = 0,
        "Set key owner to last item if it is hovered or active. Equivalent to 'if (IsItemHovered() || IsItemActive()) { SetKeyOwner(key, GetItemID());'.");

    m.def("test_key_owner",
        ImGui::TestKeyOwner,
        py::arg("key"), py::arg("owner_id"),
        "Test that key is either not owned, either owned by 'owner_id'");

    m.def("get_key_owner_data",
        ImGui::GetKeyOwnerData,
        py::arg("ctx"), py::arg("key"),
        "(private API)",
        pybind11::return_value_policy::reference);

    m.def("is_key_down",
        py::overload_cast<ImGuiKey, ImGuiID>(ImGui::IsKeyDown), py::arg("key"), py::arg("owner_id"));

    m.def("is_key_pressed",
        py::overload_cast<ImGuiKey, ImGuiID, ImGuiInputFlags>(ImGui::IsKeyPressed),
        py::arg("key"), py::arg("owner_id"), py::arg("flags") = 0,
        "Important: when transitioning from old to new IsKeyPressed(): old API has \"bool repeat = True\", so would default to repeat. New API requiress explicit ImGuiInputFlags_Repeat.");

    m.def("is_key_released",
        py::overload_cast<ImGuiKey, ImGuiID>(ImGui::IsKeyReleased), py::arg("key"), py::arg("owner_id"));

    m.def("is_mouse_down",
        py::overload_cast<ImGuiMouseButton, ImGuiID>(ImGui::IsMouseDown), py::arg("button"), py::arg("owner_id"));

    m.def("is_mouse_clicked",
        py::overload_cast<ImGuiMouseButton, ImGuiID, ImGuiInputFlags>(ImGui::IsMouseClicked), py::arg("button"), py::arg("owner_id"), py::arg("flags") = 0);

    m.def("is_mouse_released",
        py::overload_cast<ImGuiMouseButton, ImGuiID>(ImGui::IsMouseReleased), py::arg("button"), py::arg("owner_id"));

    m.def("is_mouse_double_clicked",
        py::overload_cast<ImGuiMouseButton, ImGuiID>(ImGui::IsMouseDoubleClicked), py::arg("button"), py::arg("owner_id"));

    m.def("is_key_chord_pressed",
        py::overload_cast<ImGuiKeyChord, ImGuiID, ImGuiInputFlags>(ImGui::IsKeyChordPressed), py::arg("key_chord"), py::arg("owner_id"), py::arg("flags") = 0);

    m.def("set_next_item_shortcut",
        ImGui::SetNextItemShortcut, py::arg("key_chord"));

    m.def("shortcut",
        ImGui::Shortcut, py::arg("key_chord"), py::arg("owner_id") = 0, py::arg("flags") = 0);

    m.def("set_shortcut_routing",
        ImGui::SetShortcutRouting,
        py::arg("key_chord"), py::arg("owner_id"), py::arg("flags") = 0,
        "owner_id needs to be explicit and cannot be 0");

    m.def("test_shortcut_routing",
        ImGui::TestShortcutRouting, py::arg("key_chord"), py::arg("owner_id"));

    m.def("get_shortcut_routing_data",
        ImGui::GetShortcutRoutingData,
        py::arg("key_chord"),
        pybind11::return_value_policy::reference);

    m.def("dock_context_initialize",
        ImGui::DockContextInitialize, py::arg("ctx"));

    m.def("dock_context_shutdown",
        ImGui::DockContextShutdown, py::arg("ctx"));

    m.def("dock_context_clear_nodes",
        ImGui::DockContextClearNodes,
        py::arg("ctx"), py::arg("root_id"), py::arg("clear_settings_refs"),
        "Use root_id==0 to clear all");

    m.def("dock_context_rebuild_nodes",
        ImGui::DockContextRebuildNodes, py::arg("ctx"));

    m.def("dock_context_new_frame_update_undocking",
        ImGui::DockContextNewFrameUpdateUndocking, py::arg("ctx"));

    m.def("dock_context_new_frame_update_docking",
        ImGui::DockContextNewFrameUpdateDocking, py::arg("ctx"));

    m.def("dock_context_end_frame",
        ImGui::DockContextEndFrame, py::arg("ctx"));

    m.def("dock_context_gen_node_id",
        ImGui::DockContextGenNodeID, py::arg("ctx"));

    m.def("dock_context_queue_dock",
        ImGui::DockContextQueueDock, py::arg("ctx"), py::arg("target"), py::arg("target_node"), py::arg("payload"), py::arg("split_dir"), py::arg("split_ratio"), py::arg("split_outer"));

    m.def("dock_context_queue_undock_window",
        ImGui::DockContextQueueUndockWindow, py::arg("ctx"), py::arg("window"));

    m.def("dock_context_queue_undock_node",
        ImGui::DockContextQueueUndockNode, py::arg("ctx"), py::arg("node"));

    m.def("dock_context_process_undock_window",
        ImGui::DockContextProcessUndockWindow, py::arg("ctx"), py::arg("window"), py::arg("clear_persistent_docking_ref") = true);

    m.def("dock_context_process_undock_node",
        ImGui::DockContextProcessUndockNode, py::arg("ctx"), py::arg("node"));

    m.def("dock_context_calc_drop_pos_for_docking",
        ImGui::DockContextCalcDropPosForDocking, py::arg("target"), py::arg("target_node"), py::arg("payload_window"), py::arg("payload_node"), py::arg("split_dir"), py::arg("split_outer"), py::arg("out_pos"));

    m.def("dock_context_find_node_by_id",
        ImGui::DockContextFindNodeByID,
        py::arg("ctx"), py::arg("id_"),
        pybind11::return_value_policy::reference);

    m.def("dock_node_window_menu_handler_default",
        ImGui::DockNodeWindowMenuHandler_Default, py::arg("ctx"), py::arg("node"), py::arg("tab_bar"));

    m.def("dock_node_begin_amend_tab_bar",
        ImGui::DockNodeBeginAmendTabBar, py::arg("node"));

    m.def("dock_node_end_amend_tab_bar",
        ImGui::DockNodeEndAmendTabBar);

    m.def("dock_node_get_root_node",
        ImGui::DockNodeGetRootNode,
        py::arg("node"),
        "(private API)",
        pybind11::return_value_policy::reference);

    m.def("dock_node_is_in_hierarchy_of",
        ImGui::DockNodeIsInHierarchyOf,
        py::arg("node"), py::arg("parent"),
        "(private API)");

    m.def("dock_node_get_depth",
        ImGui::DockNodeGetDepth,
        py::arg("node"),
        "(private API)");

    m.def("dock_node_get_window_menu_button_id",
        ImGui::DockNodeGetWindowMenuButtonId,
        py::arg("node"),
        "(private API)");

    m.def("get_window_dock_node",
        ImGui::GetWindowDockNode,
        "(private API)",
        pybind11::return_value_policy::reference);

    m.def("get_window_always_want_own_tab_bar",
        ImGui::GetWindowAlwaysWantOwnTabBar, py::arg("window"));

    m.def("begin_docked",
        [](ImGuiWindow * window, bool p_open) -> bool
        {
            auto BeginDocked_adapt_modifiable_immutable_to_return = [](ImGuiWindow * window, bool p_open) -> bool
            {
                bool * p_open_adapt_modifiable = & p_open;

                ImGui::BeginDocked(window, p_open_adapt_modifiable);
                return p_open;
            };

            return BeginDocked_adapt_modifiable_immutable_to_return(window, p_open);
        },     py::arg("window"), py::arg("p_open"));

    m.def("begin_dockable_drag_drop_source",
        ImGui::BeginDockableDragDropSource, py::arg("window"));

    m.def("begin_dockable_drag_drop_target",
        ImGui::BeginDockableDragDropTarget, py::arg("window"));

    m.def("set_window_dock",
        ImGui::SetWindowDock, py::arg("window"), py::arg("dock_id"), py::arg("cond"));

    m.def("dock_builder_dock_window",
        ImGui::DockBuilderDockWindow, py::arg("window_name"), py::arg("node_id"));

    m.def("dock_builder_get_node",
        ImGui::DockBuilderGetNode,
        py::arg("node_id"),
        pybind11::return_value_policy::reference);

    m.def("dock_builder_get_central_node",
        ImGui::DockBuilderGetCentralNode,
        py::arg("node_id"),
        "(private API)",
        pybind11::return_value_policy::reference);

    m.def("dock_builder_add_node",
        ImGui::DockBuilderAddNode, py::arg("node_id") = 0, py::arg("flags") = 0);

    m.def("dock_builder_remove_node",
        ImGui::DockBuilderRemoveNode,
        py::arg("node_id"),
        "Remove node and all its child, undock all windows");

    m.def("dock_builder_remove_node_docked_windows",
        ImGui::DockBuilderRemoveNodeDockedWindows, py::arg("node_id"), py::arg("clear_settings_refs") = true);

    m.def("dock_builder_remove_node_child_nodes",
        ImGui::DockBuilderRemoveNodeChildNodes,
        py::arg("node_id"),
        "Remove all split/hierarchy. All remaining docked windows will be re-docked to the remaining root node (node_id).");

    m.def("dock_builder_set_node_pos",
        ImGui::DockBuilderSetNodePos, py::arg("node_id"), py::arg("pos"));

    m.def("dock_builder_set_node_size",
        ImGui::DockBuilderSetNodeSize, py::arg("node_id"), py::arg("size"));
    // #ifdef IMGUI_BUNDLE_PYTHON_API
    //


    auto pyClassDockBuilderSplitNodeResult =
        py::class_<ImGui::DockBuilderSplitNodeResult>
            (m, "DockBuilderSplitNodeResult", "")
        .def(py::init<>([](
        ImGuiID id_at_dir = ImGuiID(), ImGuiID id_at_opposite_dir = ImGuiID())
        {
            auto r = std::make_unique<ImGui::DockBuilderSplitNodeResult>();
            r->id_at_dir = id_at_dir;
            r->id_at_opposite_dir = id_at_opposite_dir;
            return r;
        })
        , py::arg("id_at_dir") = ImGuiID(), py::arg("id_at_opposite_dir") = ImGuiID()
        )
        .def_readwrite("id_at_dir", &ImGui::DockBuilderSplitNodeResult::id_at_dir, "")
        .def_readwrite("id_at_opposite_dir", &ImGui::DockBuilderSplitNodeResult::id_at_opposite_dir, "")
        ;


    m.def("dock_builder_split_node",
        [](ImGuiID node_id, ImGuiDir split_dir, float size_ratio_for_node_at_dir) -> ImGui::DockBuilderSplitNodeResult
        {
            auto DockBuilderSplitNode_adapt_force_lambda = [](ImGuiID node_id, ImGuiDir split_dir, float size_ratio_for_node_at_dir) -> ImGui::DockBuilderSplitNodeResult
            {
                auto lambda_result = ImGui::DockBuilderSplitNode(node_id, split_dir, size_ratio_for_node_at_dir);
                return lambda_result;
            };

            return DockBuilderSplitNode_adapt_force_lambda(node_id, split_dir, size_ratio_for_node_at_dir);
        },     py::arg("node_id"), py::arg("split_dir"), py::arg("size_ratio_for_node_at_dir"));
    // #endif
    //

    m.def("dock_builder_copy_node",
        ImGui::DockBuilderCopyNode, py::arg("src_node_id"), py::arg("dst_node_id"), py::arg("out_node_remap_pairs"));

    m.def("dock_builder_copy_window_settings",
        ImGui::DockBuilderCopyWindowSettings, py::arg("src_name"), py::arg("dst_name"));

    m.def("dock_builder_finish",
        ImGui::DockBuilderFinish, py::arg("node_id"));

    m.def("push_focus_scope",
        ImGui::PushFocusScope, py::arg("id_"));

    m.def("pop_focus_scope",
        ImGui::PopFocusScope);

    m.def("get_current_focus_scope",
        ImGui::GetCurrentFocusScope, "(private API)\n\n Focus scope we are outputting into, set by PushFocusScope()");

    m.def("is_drag_drop_active",
        ImGui::IsDragDropActive);

    m.def("begin_drag_drop_target_custom",
        ImGui::BeginDragDropTargetCustom, py::arg("bb"), py::arg("id_"));

    m.def("clear_drag_drop",
        ImGui::ClearDragDrop);

    m.def("is_drag_drop_payload_being_accepted",
        ImGui::IsDragDropPayloadBeingAccepted);

    m.def("render_drag_drop_target_rect",
        ImGui::RenderDragDropTargetRect, py::arg("bb"), py::arg("item_clip_rect"));

    m.def("get_typing_select_request",
        ImGui::GetTypingSelectRequest,
        py::arg("flags") = ImGuiTypingSelectFlags_None,
        pybind11::return_value_policy::reference);

    m.def("set_window_clip_rect_before_set_channel",
        ImGui::SetWindowClipRectBeforeSetChannel, py::arg("window"), py::arg("clip_rect"));

    m.def("begin_columns",
        ImGui::BeginColumns,
        py::arg("str_id"), py::arg("count"), py::arg("flags") = 0,
        "setup number of columns. use an identifier to distinguish multiple column sets. close with EndColumns().");

    m.def("end_columns",
        ImGui::EndColumns, "close columns");

    m.def("push_column_clip_rect",
        ImGui::PushColumnClipRect, py::arg("column_index"));

    m.def("push_columns_background",
        ImGui::PushColumnsBackground);

    m.def("pop_columns_background",
        ImGui::PopColumnsBackground);

    m.def("get_columns_id",
        ImGui::GetColumnsID, py::arg("str_id"), py::arg("count"));

    m.def("find_or_create_columns",
        ImGui::FindOrCreateColumns,
        py::arg("window"), py::arg("id_"),
        pybind11::return_value_policy::reference);

    m.def("get_column_offset_from_norm",
        ImGui::GetColumnOffsetFromNorm, py::arg("columns"), py::arg("offset_norm"));

    m.def("get_column_norm_from_offset",
        ImGui::GetColumnNormFromOffset, py::arg("columns"), py::arg("offset"));

    m.def("table_open_context_menu",
        py::overload_cast<int>(ImGui::TableOpenContextMenu), py::arg("column_n") = -1);

    m.def("table_set_column_width",
        py::overload_cast<int, float>(ImGui::TableSetColumnWidth), py::arg("column_n"), py::arg("width"));

    m.def("table_set_column_sort_direction",
        py::overload_cast<int, ImGuiSortDirection, bool>(ImGui::TableSetColumnSortDirection), py::arg("column_n"), py::arg("sort_direction"), py::arg("append_to_sort_specs"));

    m.def("table_get_hovered_column",
        py::overload_cast<>(ImGui::TableGetHoveredColumn), "May use (TableGetColumnFlags() & ImGuiTableColumnFlags_IsHovered) instead. Return hovered column. return -1 when table is not hovered. return columns_count if the unused space at the right of visible columns is hovered.");

    m.def("table_get_hovered_row",
        py::overload_cast<>(ImGui::TableGetHoveredRow), "Retrieve *PREVIOUS FRAME* hovered row. This difference with TableGetHoveredColumn() is the reason why this is not public yet.");

    m.def("table_get_header_row_height",
        py::overload_cast<>(ImGui::TableGetHeaderRowHeight));

    m.def("table_get_header_angled_max_label_width",
        py::overload_cast<>(ImGui::TableGetHeaderAngledMaxLabelWidth));

    m.def("table_push_background_channel",
        py::overload_cast<>(ImGui::TablePushBackgroundChannel));

    m.def("table_pop_background_channel",
        py::overload_cast<>(ImGui::TablePopBackgroundChannel));

    m.def("table_angled_headers_row_ex",
        py::overload_cast<float, float>(ImGui::TableAngledHeadersRowEx), py::arg("angle"), py::arg("max_label_width") = 0.0f);

    m.def("get_current_table",
        ImGui::GetCurrentTable,
        " Tables: Internals\n(private API)",
        pybind11::return_value_policy::reference);

    m.def("table_find_by_id",
        py::overload_cast<ImGuiID>(ImGui::TableFindByID),
        py::arg("id_"),
        pybind11::return_value_policy::reference);

    m.def("begin_table_ex",
        ImGui::BeginTableEx, py::arg("name"), py::arg("id_"), py::arg("columns_count"), py::arg("flags") = 0, py::arg("outer_size") = ImVec2(0, 0), py::arg("inner_width") = 0.0f);

    m.def("table_begin_init_memory",
        py::overload_cast<ImGuiTable *, int>(ImGui::TableBeginInitMemory), py::arg("table"), py::arg("columns_count"));

    m.def("table_begin_apply_requests",
        py::overload_cast<ImGuiTable *>(ImGui::TableBeginApplyRequests), py::arg("table"));

    m.def("table_setup_draw_channels",
        py::overload_cast<ImGuiTable *>(ImGui::TableSetupDrawChannels), py::arg("table"));

    m.def("table_update_layout",
        py::overload_cast<ImGuiTable *>(ImGui::TableUpdateLayout), py::arg("table"));

    m.def("table_update_borders",
        py::overload_cast<ImGuiTable *>(ImGui::TableUpdateBorders), py::arg("table"));

    m.def("table_update_columns_weight_from_width",
        py::overload_cast<ImGuiTable *>(ImGui::TableUpdateColumnsWeightFromWidth), py::arg("table"));

    m.def("table_draw_borders",
        py::overload_cast<ImGuiTable *>(ImGui::TableDrawBorders), py::arg("table"));

    m.def("table_draw_default_context_menu",
        py::overload_cast<ImGuiTable *, ImGuiTableFlags>(ImGui::TableDrawDefaultContextMenu), py::arg("table"), py::arg("flags_for_section_to_display"));

    m.def("table_begin_context_menu_popup",
        py::overload_cast<ImGuiTable *>(ImGui::TableBeginContextMenuPopup), py::arg("table"));

    m.def("table_merge_draw_channels",
        py::overload_cast<ImGuiTable *>(ImGui::TableMergeDrawChannels), py::arg("table"));

    m.def("table_get_instance_data",
        py::overload_cast<ImGuiTable *, int>(ImGui::TableGetInstanceData),
        py::arg("table"), py::arg("instance_no"),
        "(private API)",
        pybind11::return_value_policy::reference);

    m.def("table_get_instance_id",
        py::overload_cast<ImGuiTable *, int>(ImGui::TableGetInstanceID),
        py::arg("table"), py::arg("instance_no"),
        "(private API)");

    m.def("table_sort_specs_sanitize",
        py::overload_cast<ImGuiTable *>(ImGui::TableSortSpecsSanitize), py::arg("table"));

    m.def("table_sort_specs_build",
        py::overload_cast<ImGuiTable *>(ImGui::TableSortSpecsBuild), py::arg("table"));

    m.def("table_get_column_next_sort_direction",
        py::overload_cast<ImGuiTableColumn *>(ImGui::TableGetColumnNextSortDirection), py::arg("column"));

    m.def("table_fix_column_sort_direction",
        py::overload_cast<ImGuiTable *, ImGuiTableColumn *>(ImGui::TableFixColumnSortDirection), py::arg("table"), py::arg("column"));

    m.def("table_get_column_width_auto",
        py::overload_cast<ImGuiTable *, ImGuiTableColumn *>(ImGui::TableGetColumnWidthAuto), py::arg("table"), py::arg("column"));

    m.def("table_begin_row",
        py::overload_cast<ImGuiTable *>(ImGui::TableBeginRow), py::arg("table"));

    m.def("table_end_row",
        py::overload_cast<ImGuiTable *>(ImGui::TableEndRow), py::arg("table"));

    m.def("table_begin_cell",
        py::overload_cast<ImGuiTable *, int>(ImGui::TableBeginCell), py::arg("table"), py::arg("column_n"));

    m.def("table_end_cell",
        py::overload_cast<ImGuiTable *>(ImGui::TableEndCell), py::arg("table"));

    m.def("table_get_cell_bg_rect",
        py::overload_cast<const ImGuiTable *, int>(ImGui::TableGetCellBgRect), py::arg("table"), py::arg("column_n"));

    m.def("table_get_column_name",
        py::overload_cast<const ImGuiTable *, int>(ImGui::TableGetColumnName),
        py::arg("table"), py::arg("column_n"),
        pybind11::return_value_policy::reference);

    m.def("table_get_column_resize_id",
        py::overload_cast<ImGuiTable *, int, int>(ImGui::TableGetColumnResizeID), py::arg("table"), py::arg("column_n"), py::arg("instance_no") = 0);

    m.def("table_get_max_column_width",
        py::overload_cast<const ImGuiTable *, int>(ImGui::TableGetMaxColumnWidth), py::arg("table"), py::arg("column_n"));

    m.def("table_set_column_width_auto_single",
        py::overload_cast<ImGuiTable *, int>(ImGui::TableSetColumnWidthAutoSingle), py::arg("table"), py::arg("column_n"));

    m.def("table_set_column_width_auto_all",
        py::overload_cast<ImGuiTable *>(ImGui::TableSetColumnWidthAutoAll), py::arg("table"));

    m.def("table_remove",
        py::overload_cast<ImGuiTable *>(ImGui::TableRemove), py::arg("table"));

    m.def("table_gc_compact_transient_buffers",
        py::overload_cast<ImGuiTable *>(ImGui::TableGcCompactTransientBuffers), py::arg("table"));

    m.def("table_gc_compact_transient_buffers",
        py::overload_cast<ImGuiTableTempData *>(ImGui::TableGcCompactTransientBuffers), py::arg("table"));

    m.def("table_gc_compact_settings",
        py::overload_cast<>(ImGui::TableGcCompactSettings));

    m.def("table_load_settings",
        py::overload_cast<ImGuiTable *>(ImGui::TableLoadSettings), py::arg("table"));

    m.def("table_save_settings",
        py::overload_cast<ImGuiTable *>(ImGui::TableSaveSettings), py::arg("table"));

    m.def("table_reset_settings",
        py::overload_cast<ImGuiTable *>(ImGui::TableResetSettings), py::arg("table"));

    m.def("table_get_bound_settings",
        py::overload_cast<ImGuiTable *>(ImGui::TableGetBoundSettings),
        py::arg("table"),
        pybind11::return_value_policy::reference);

    m.def("table_settings_add_settings_handler",
        py::overload_cast<>(ImGui::TableSettingsAddSettingsHandler));

    m.def("table_settings_create",
        py::overload_cast<ImGuiID, int>(ImGui::TableSettingsCreate),
        py::arg("id_"), py::arg("columns_count"),
        pybind11::return_value_policy::reference);

    m.def("table_settings_find_by_id",
        py::overload_cast<ImGuiID>(ImGui::TableSettingsFindByID),
        py::arg("id_"),
        pybind11::return_value_policy::reference);

    m.def("get_current_tab_bar",
        ImGui::GetCurrentTabBar,
        " Tab Bars\n(private API)",
        pybind11::return_value_policy::reference);

    m.def("begin_tab_bar_ex",
        ImGui::BeginTabBarEx, py::arg("tab_bar"), py::arg("bb"), py::arg("flags"));

    m.def("tab_bar_find_tab_by_id",
        ImGui::TabBarFindTabByID,
        py::arg("tab_bar"), py::arg("tab_id"),
        pybind11::return_value_policy::reference);

    m.def("tab_bar_find_tab_by_order",
        ImGui::TabBarFindTabByOrder,
        py::arg("tab_bar"), py::arg("order"),
        pybind11::return_value_policy::reference);

    m.def("tab_bar_find_most_recently_selected_tab_for_active_window",
        ImGui::TabBarFindMostRecentlySelectedTabForActiveWindow,
        py::arg("tab_bar"),
        pybind11::return_value_policy::reference);

    m.def("tab_bar_get_current_tab",
        ImGui::TabBarGetCurrentTab,
        py::arg("tab_bar"),
        pybind11::return_value_policy::reference);

    m.def("tab_bar_get_tab_order",
        ImGui::TabBarGetTabOrder,
        py::arg("tab_bar"), py::arg("tab"),
        "(private API)");

    m.def("tab_bar_get_tab_name",
        ImGui::TabBarGetTabName,
        py::arg("tab_bar"), py::arg("tab"),
        pybind11::return_value_policy::reference);

    m.def("tab_bar_add_tab",
        ImGui::TabBarAddTab, py::arg("tab_bar"), py::arg("tab_flags"), py::arg("window"));

    m.def("tab_bar_remove_tab",
        ImGui::TabBarRemoveTab, py::arg("tab_bar"), py::arg("tab_id"));

    m.def("tab_bar_close_tab",
        ImGui::TabBarCloseTab, py::arg("tab_bar"), py::arg("tab"));

    m.def("tab_bar_queue_focus",
        ImGui::TabBarQueueFocus, py::arg("tab_bar"), py::arg("tab"));

    m.def("tab_bar_queue_reorder",
        ImGui::TabBarQueueReorder, py::arg("tab_bar"), py::arg("tab"), py::arg("offset"));

    m.def("tab_bar_queue_reorder_from_mouse_pos",
        ImGui::TabBarQueueReorderFromMousePos, py::arg("tab_bar"), py::arg("tab"), py::arg("mouse_pos"));

    m.def("tab_bar_process_reorder",
        ImGui::TabBarProcessReorder, py::arg("tab_bar"));

    m.def("tab_item_ex",
        [](ImGuiTabBar * tab_bar, const char * label, bool p_open, ImGuiTabItemFlags flags, ImGuiWindow * docked_window) -> std::tuple<bool, bool>
        {
            auto TabItemEx_adapt_modifiable_immutable_to_return = [](ImGuiTabBar * tab_bar, const char * label, bool p_open, ImGuiTabItemFlags flags, ImGuiWindow * docked_window) -> std::tuple<bool, bool>
            {
                bool * p_open_adapt_modifiable = & p_open;

                bool r = ImGui::TabItemEx(tab_bar, label, p_open_adapt_modifiable, flags, docked_window);
                return std::make_tuple(r, p_open);
            };

            return TabItemEx_adapt_modifiable_immutable_to_return(tab_bar, label, p_open, flags, docked_window);
        },     py::arg("tab_bar"), py::arg("label"), py::arg("p_open"), py::arg("flags"), py::arg("docked_window"));

    m.def("tab_item_calc_size",
        py::overload_cast<const char *, bool>(ImGui::TabItemCalcSize), py::arg("label"), py::arg("has_close_button_or_unsaved_marker"));

    m.def("tab_item_calc_size",
        py::overload_cast<ImGuiWindow *>(ImGui::TabItemCalcSize), py::arg("window"));

    m.def("tab_item_background",
        ImGui::TabItemBackground, py::arg("draw_list"), py::arg("bb"), py::arg("flags"), py::arg("col"));

    m.def("tab_item_label_and_close_button",
        [](ImDrawList * draw_list, const ImRect & bb, ImGuiTabItemFlags flags, ImVec2 frame_padding, const char * label, ImGuiID tab_id, ImGuiID close_button_id, bool is_contents_visible, bool out_just_closed, bool out_text_clipped) -> std::tuple<bool, bool>
        {
            auto TabItemLabelAndCloseButton_adapt_modifiable_immutable_to_return = [](ImDrawList * draw_list, const ImRect & bb, ImGuiTabItemFlags flags, ImVec2 frame_padding, const char * label, ImGuiID tab_id, ImGuiID close_button_id, bool is_contents_visible, bool out_just_closed, bool out_text_clipped) -> std::tuple<bool, bool>
            {
                bool * out_just_closed_adapt_modifiable = & out_just_closed;
                bool * out_text_clipped_adapt_modifiable = & out_text_clipped;

                ImGui::TabItemLabelAndCloseButton(draw_list, bb, flags, frame_padding, label, tab_id, close_button_id, is_contents_visible, out_just_closed_adapt_modifiable, out_text_clipped_adapt_modifiable);
                return std::make_tuple(out_just_closed, out_text_clipped);
            };

            return TabItemLabelAndCloseButton_adapt_modifiable_immutable_to_return(draw_list, bb, flags, frame_padding, label, tab_id, close_button_id, is_contents_visible, out_just_closed, out_text_clipped);
        },     py::arg("draw_list"), py::arg("bb"), py::arg("flags"), py::arg("frame_padding"), py::arg("label"), py::arg("tab_id"), py::arg("close_button_id"), py::arg("is_contents_visible"), py::arg("out_just_closed"), py::arg("out_text_clipped"));

    m.def("render_text",
        ImGui::RenderText, py::arg("pos"), py::arg("text"), py::arg("text_end") = py::none(), py::arg("hide_text_after_hash") = true);

    m.def("render_text_wrapped",
        ImGui::RenderTextWrapped, py::arg("pos"), py::arg("text"), py::arg("text_end"), py::arg("wrap_width"));

    m.def("render_text_clipped",
        ImGui::RenderTextClipped, py::arg("pos_min"), py::arg("pos_max"), py::arg("text"), py::arg("text_end"), py::arg("text_size_if_known"), py::arg("align") = ImVec2(0, 0), py::arg("clip_rect") = py::none());

    m.def("render_text_clipped_ex",
        ImGui::RenderTextClippedEx, py::arg("draw_list"), py::arg("pos_min"), py::arg("pos_max"), py::arg("text"), py::arg("text_end"), py::arg("text_size_if_known"), py::arg("align") = ImVec2(0, 0), py::arg("clip_rect") = py::none());

    m.def("render_text_ellipsis",
        ImGui::RenderTextEllipsis, py::arg("draw_list"), py::arg("pos_min"), py::arg("pos_max"), py::arg("clip_max_x"), py::arg("ellipsis_max_x"), py::arg("text"), py::arg("text_end"), py::arg("text_size_if_known"));

    m.def("render_frame",
        ImGui::RenderFrame, py::arg("p_min"), py::arg("p_max"), py::arg("fill_col"), py::arg("border") = true, py::arg("rounding") = 0.0f);

    m.def("render_frame_border",
        ImGui::RenderFrameBorder, py::arg("p_min"), py::arg("p_max"), py::arg("rounding") = 0.0f);

    m.def("render_color_rect_with_alpha_checkerboard",
        ImGui::RenderColorRectWithAlphaCheckerboard, py::arg("draw_list"), py::arg("p_min"), py::arg("p_max"), py::arg("fill_col"), py::arg("grid_step"), py::arg("grid_off"), py::arg("rounding") = 0.0f, py::arg("flags") = 0);

    m.def("render_nav_highlight",
        ImGui::RenderNavHighlight,
        py::arg("bb"), py::arg("id_"), py::arg("flags") = ImGuiNavHighlightFlags_None,
        "Navigation highlight");

    m.def("find_rendered_text_end",
        ImGui::FindRenderedTextEnd,
        py::arg("text"), py::arg("text_end") = py::none(),
        "Find the optional ## from which we stop displaying text.",
        pybind11::return_value_policy::reference);

    m.def("render_mouse_cursor",
        ImGui::RenderMouseCursor, py::arg("pos"), py::arg("scale"), py::arg("mouse_cursor"), py::arg("col_fill"), py::arg("col_border"), py::arg("col_shadow"));

    m.def("render_arrow",
        ImGui::RenderArrow, py::arg("draw_list"), py::arg("pos"), py::arg("col"), py::arg("dir"), py::arg("scale") = 1.0f);

    m.def("render_bullet",
        ImGui::RenderBullet, py::arg("draw_list"), py::arg("pos"), py::arg("col"));

    m.def("render_check_mark",
        ImGui::RenderCheckMark, py::arg("draw_list"), py::arg("pos"), py::arg("col"), py::arg("sz"));

    m.def("render_arrow_pointing_at",
        ImGui::RenderArrowPointingAt, py::arg("draw_list"), py::arg("pos"), py::arg("half_sz"), py::arg("direction"), py::arg("col"));

    m.def("render_arrow_dock_menu",
        ImGui::RenderArrowDockMenu, py::arg("draw_list"), py::arg("p_min"), py::arg("sz"), py::arg("col"));

    m.def("render_rect_filled_range_h",
        ImGui::RenderRectFilledRangeH, py::arg("draw_list"), py::arg("rect"), py::arg("col"), py::arg("x_start_norm"), py::arg("x_end_norm"), py::arg("rounding"));

    m.def("render_rect_filled_with_hole",
        ImGui::RenderRectFilledWithHole, py::arg("draw_list"), py::arg("outer"), py::arg("inner"), py::arg("col"), py::arg("rounding"));

    m.def("calc_rounding_flags_for_rect_in_rect",
        ImGui::CalcRoundingFlagsForRectInRect, py::arg("r_in"), py::arg("r_outer"), py::arg("threshold"));

    m.def("text_ex",
        ImGui::TextEx, py::arg("text"), py::arg("text_end") = py::none(), py::arg("flags") = 0);

    m.def("button_ex",
        ImGui::ButtonEx, py::arg("label"), py::arg("size_arg") = ImVec2(0, 0), py::arg("flags") = 0);

    m.def("arrow_button_ex",
        ImGui::ArrowButtonEx, py::arg("str_id"), py::arg("dir"), py::arg("size_arg"), py::arg("flags") = 0);

    m.def("image_button_ex",
        ImGui::ImageButtonEx, py::arg("id_"), py::arg("texture_id"), py::arg("image_size"), py::arg("uv0"), py::arg("uv1"), py::arg("bg_col"), py::arg("tint_col"), py::arg("flags") = 0);

    m.def("separator_ex",
        ImGui::SeparatorEx, py::arg("flags"), py::arg("thickness") = 1.0f);

    m.def("separator_text_ex",
        ImGui::SeparatorTextEx, py::arg("id_"), py::arg("label"), py::arg("label_end"), py::arg("extra_width"));

    m.def("checkbox_flags",
        py::overload_cast<const char *, ImS64 *, ImS64>(ImGui::CheckboxFlags), py::arg("label"), py::arg("flags"), py::arg("flags_value"));

    m.def("checkbox_flags",
        py::overload_cast<const char *, ImU64 *, ImU64>(ImGui::CheckboxFlags), py::arg("label"), py::arg("flags"), py::arg("flags_value"));

    m.def("close_button",
        ImGui::CloseButton, py::arg("id_"), py::arg("pos"));

    m.def("collapse_button",
        ImGui::CollapseButton, py::arg("id_"), py::arg("pos"), py::arg("dock_node"));

    m.def("scrollbar",
        ImGui::Scrollbar, py::arg("axis"));

    m.def("scrollbar_ex",
        ImGui::ScrollbarEx, py::arg("bb"), py::arg("id_"), py::arg("axis"), py::arg("p_scroll_v"), py::arg("avail_v"), py::arg("contents_v"), py::arg("flags"));

    m.def("get_window_scrollbar_rect",
        ImGui::GetWindowScrollbarRect, py::arg("window"), py::arg("axis"));

    m.def("get_window_scrollbar_id",
        ImGui::GetWindowScrollbarID, py::arg("window"), py::arg("axis"));

    m.def("get_window_resize_corner_id",
        ImGui::GetWindowResizeCornerID,
        py::arg("window"), py::arg("n"),
        "0..3: corners");

    m.def("get_window_resize_border_id",
        ImGui::GetWindowResizeBorderID, py::arg("window"), py::arg("dir"));

    m.def("button_behavior",
        [](const ImRect & bb, ImGuiID id, bool out_hovered, bool out_held, ImGuiButtonFlags flags = 0) -> std::tuple<bool, bool, bool>
        {
            auto ButtonBehavior_adapt_modifiable_immutable_to_return = [](const ImRect & bb, ImGuiID id, bool out_hovered, bool out_held, ImGuiButtonFlags flags = 0) -> std::tuple<bool, bool, bool>
            {
                bool * out_hovered_adapt_modifiable = & out_hovered;
                bool * out_held_adapt_modifiable = & out_held;

                bool r = ImGui::ButtonBehavior(bb, id, out_hovered_adapt_modifiable, out_held_adapt_modifiable, flags);
                return std::make_tuple(r, out_hovered, out_held);
            };

            return ButtonBehavior_adapt_modifiable_immutable_to_return(bb, id, out_hovered, out_held, flags);
        },     py::arg("bb"), py::arg("id_"), py::arg("out_hovered"), py::arg("out_held"), py::arg("flags") = 0);

    m.def("drag_behavior",
        py::overload_cast<ImGuiID, ImGuiDataType, void *, float, const void *, const void *, const char *, ImGuiSliderFlags>(ImGui::DragBehavior), py::arg("id_"), py::arg("data_type"), py::arg("p_v"), py::arg("v_speed"), py::arg("p_min"), py::arg("p_max"), py::arg("format"), py::arg("flags"));

    m.def("slider_behavior",
        py::overload_cast<const ImRect &, ImGuiID, ImGuiDataType, void *, const void *, const void *, const char *, ImGuiSliderFlags, ImRect *>(ImGui::SliderBehavior), py::arg("bb"), py::arg("id_"), py::arg("data_type"), py::arg("p_v"), py::arg("p_min"), py::arg("p_max"), py::arg("format"), py::arg("flags"), py::arg("out_grab_bb"));

    m.def("splitter_behavior",
        [](const ImRect & bb, ImGuiID id, ImGuiAxis axis, float size1, float size2, float min_size1, float min_size2, float hover_extend = 0.0f, float hover_visibility_delay = 0.0f, ImU32 bg_col = 0) -> std::tuple<bool, float, float>
        {
            auto SplitterBehavior_adapt_modifiable_immutable_to_return = [](const ImRect & bb, ImGuiID id, ImGuiAxis axis, float size1, float size2, float min_size1, float min_size2, float hover_extend = 0.0f, float hover_visibility_delay = 0.0f, ImU32 bg_col = 0) -> std::tuple<bool, float, float>
            {
                float * size1_adapt_modifiable = & size1;
                float * size2_adapt_modifiable = & size2;

                bool r = ImGui::SplitterBehavior(bb, id, axis, size1_adapt_modifiable, size2_adapt_modifiable, min_size1, min_size2, hover_extend, hover_visibility_delay, bg_col);
                return std::make_tuple(r, size1, size2);
            };

            return SplitterBehavior_adapt_modifiable_immutable_to_return(bb, id, axis, size1, size2, min_size1, min_size2, hover_extend, hover_visibility_delay, bg_col);
        },     py::arg("bb"), py::arg("id_"), py::arg("axis"), py::arg("size1"), py::arg("size2"), py::arg("min_size1"), py::arg("min_size2"), py::arg("hover_extend") = 0.0f, py::arg("hover_visibility_delay") = 0.0f, py::arg("bg_col") = 0);

    m.def("tree_node_behavior",
        ImGui::TreeNodeBehavior, py::arg("id_"), py::arg("flags"), py::arg("label"), py::arg("label_end") = py::none());

    m.def("tree_push_override_id",
        ImGui::TreePushOverrideID, py::arg("id_"));

    m.def("tree_node_set_open",
        ImGui::TreeNodeSetOpen, py::arg("id_"), py::arg("open"));

    m.def("tree_node_update_next_open",
        ImGui::TreeNodeUpdateNextOpen,
        py::arg("id_"), py::arg("flags"),
        "Return open state. Consume previous SetNextItemOpen() data, if any. May return True when logging.");

    m.def("set_next_item_selection_user_data",
        ImGui::SetNextItemSelectionUserData, py::arg("selection_user_data"));
    // #ifdef IMGUI_BUNDLE_PYTHON_API
    //

    m.def("input_text_ex",
        [](const char * label, const char * hint, std::string s, const ImVec2 & size_arg, ImGuiInputTextFlags flags, ImGuiInputTextCallback callback = NULL) -> std::tuple<bool, std::string>
        {
            auto InputTextEx_adapt_modifiable_immutable_to_return = [](const char * label, const char * hint, std::string s, const ImVec2 & size_arg, ImGuiInputTextFlags flags, ImGuiInputTextCallback callback = NULL) -> std::tuple<bool, std::string>
            {
                std::string * s_adapt_modifiable = & s;

                bool r = ImGui::InputTextEx(label, hint, s_adapt_modifiable, size_arg, flags, callback);
                return std::make_tuple(r, s);
            };

            return InputTextEx_adapt_modifiable_immutable_to_return(label, hint, s, size_arg, flags, callback);
        },     py::arg("label"), py::arg("hint"), py::arg("s"), py::arg("size_arg"), py::arg("flags"), py::arg("callback") = py::none());

    m.def("temp_input_text",
        [](const ImRect & bb, ImGuiID id, const char * label, std::string s, ImGuiInputTextFlags flags) -> std::tuple<bool, std::string>
        {
            auto TempInputText_adapt_modifiable_immutable_to_return = [](const ImRect & bb, ImGuiID id, const char * label, std::string s, ImGuiInputTextFlags flags) -> std::tuple<bool, std::string>
            {
                std::string * s_adapt_modifiable = & s;

                bool r = ImGui::TempInputText(bb, id, label, s_adapt_modifiable, flags);
                return std::make_tuple(r, s);
            };

            return TempInputText_adapt_modifiable_immutable_to_return(bb, id, label, s, flags);
        },     py::arg("bb"), py::arg("id_"), py::arg("label"), py::arg("s"), py::arg("flags"));
    // #endif
    //

    m.def("input_text_deactivate_hook",
        py::overload_cast<ImGuiID>(ImGui::InputTextDeactivateHook), py::arg("id_"));

    m.def("temp_input_is_active",
        ImGui::TempInputIsActive,
        py::arg("id_"),
        "(private API)");

    m.def("get_input_text_state",
        ImGui::GetInputTextState,
        py::arg("id_"),
        "(private API)\n\n Get input text state if active",
        pybind11::return_value_policy::reference);

    m.def("color_tooltip",
        ImGui::ColorTooltip, py::arg("text"), py::arg("col"), py::arg("flags"));

    m.def("color_edit_options_popup",
        py::overload_cast<const float *, ImGuiColorEditFlags>(ImGui::ColorEditOptionsPopup), py::arg("col"), py::arg("flags"));

    m.def("color_picker_options_popup",
        py::overload_cast<const float *, ImGuiColorEditFlags>(ImGui::ColorPickerOptionsPopup), py::arg("ref_col"), py::arg("flags"));

    m.def("shade_verts_linear_color_gradient_keep_alpha",
        ImGui::ShadeVertsLinearColorGradientKeepAlpha, py::arg("draw_list"), py::arg("vert_start_idx"), py::arg("vert_end_idx"), py::arg("gradient_p0"), py::arg("gradient_p1"), py::arg("col0"), py::arg("col1"));

    m.def("shade_verts_linear_uv",
        ImGui::ShadeVertsLinearUV, py::arg("draw_list"), py::arg("vert_start_idx"), py::arg("vert_end_idx"), py::arg("a"), py::arg("b"), py::arg("uv_a"), py::arg("uv_b"), py::arg("clamp"));

    m.def("shade_verts_transform_pos",
        ImGui::ShadeVertsTransformPos, py::arg("draw_list"), py::arg("vert_start_idx"), py::arg("vert_end_idx"), py::arg("pivot_in"), py::arg("cos_a"), py::arg("sin_a"), py::arg("pivot_out"));

    m.def("gc_compact_transient_misc_buffers",
        ImGui::GcCompactTransientMiscBuffers);

    m.def("gc_compact_transient_window_buffers",
        ImGui::GcCompactTransientWindowBuffers, py::arg("window"));

    m.def("gc_awake_transient_window_buffers",
        ImGui::GcAwakeTransientWindowBuffers, py::arg("window"));

    m.def("debug_log",
        [](const char * fmt)
        {
            auto DebugLog_adapt_variadic_format = [](const char * fmt)
            {
                ImGui::DebugLog("%s", fmt);
            };

            DebugLog_adapt_variadic_format(fmt);
        },     py::arg("fmt"));

    m.def("debug_alloc_hook",
        ImGui::DebugAllocHook,
        py::arg("info"), py::arg("frame_count"), py::arg("ptr"), py::arg("size"),
        "size >= 0 : alloc, size = -1 : free");

    m.def("error_check_using_set_cursor_pos_to_extend_parent_boundaries",
        ImGui::ErrorCheckUsingSetCursorPosToExtendParentBoundaries);
    // #ifdef IMGUI_BUNDLE_PYTHON_API
    //

    m.def("error_check_end_frame_recover",
        py::overload_cast<ImGuiErrorStringCallback>(ImGui::ErrorCheckEndFrameRecover), py::arg("callback"));

    m.def("error_check_end_window_recover",
        py::overload_cast<ImGuiErrorStringCallback>(ImGui::ErrorCheckEndWindowRecover), py::arg("callback"));
    // #endif
    //

    m.def("debug_draw_cursor_pos",
        ImGui::DebugDrawCursorPos, py::arg("col") = IM_COL32(255, 0, 0, 255));

    m.def("debug_draw_line_extents",
        ImGui::DebugDrawLineExtents, py::arg("col") = IM_COL32(255, 0, 0, 255));

    m.def("debug_draw_item_rect",
        ImGui::DebugDrawItemRect, py::arg("col") = IM_COL32(255, 0, 0, 255));

    m.def("debug_locate_item",
        ImGui::DebugLocateItem,
        py::arg("target_id"),
        "Call sparingly: only 1 at the same time!");

    m.def("debug_locate_item_on_hover",
        ImGui::DebugLocateItemOnHover,
        py::arg("target_id"),
        "Only call on reaction to a mouse Hover: because only 1 at the same time!");

    m.def("debug_locate_item_resolve_with_last_item",
        ImGui::DebugLocateItemResolveWithLastItem);

    m.def("debug_break_clear_data",
        ImGui::DebugBreakClearData);

    m.def("debug_break_button",
        ImGui::DebugBreakButton, py::arg("label"), py::arg("description_of_location"));

    m.def("debug_break_button_tooltip",
        ImGui::DebugBreakButtonTooltip, py::arg("keyboard_only"), py::arg("description_of_location"));

    m.def("show_font_atlas",
        ImGui::ShowFontAtlas, py::arg("atlas"));

    m.def("debug_hook_id_info",
        ImGui::DebugHookIdInfo, py::arg("id_"), py::arg("data_type"), py::arg("data_id"), py::arg("data_id_end"));

    m.def("debug_node_columns",
        ImGui::DebugNodeColumns, py::arg("columns"));

    m.def("debug_node_dock_node",
        ImGui::DebugNodeDockNode, py::arg("node"), py::arg("label"));

    m.def("debug_node_draw_list",
        py::overload_cast<ImGuiWindow *, ImGuiViewportP *, const ImDrawList *, const char *>(ImGui::DebugNodeDrawList), py::arg("window"), py::arg("viewport"), py::arg("draw_list"), py::arg("label"));

    m.def("debug_node_draw_cmd_show_mesh_and_bounding_box",
        ImGui::DebugNodeDrawCmdShowMeshAndBoundingBox, py::arg("out_draw_list"), py::arg("draw_list"), py::arg("draw_cmd"), py::arg("show_mesh"), py::arg("show_aabb"));

    m.def("debug_node_font",
        ImGui::DebugNodeFont, py::arg("font"));

    m.def("debug_node_font_glyph",
        ImGui::DebugNodeFontGlyph, py::arg("font"), py::arg("glyph"));

    m.def("debug_node_storage",
        ImGui::DebugNodeStorage, py::arg("storage"), py::arg("label"));

    m.def("debug_node_tab_bar",
        ImGui::DebugNodeTabBar, py::arg("tab_bar"), py::arg("label"));

    m.def("debug_node_table",
        ImGui::DebugNodeTable, py::arg("table"));

    m.def("debug_node_table_settings",
        ImGui::DebugNodeTableSettings, py::arg("settings"));

    m.def("debug_node_input_text_state",
        ImGui::DebugNodeInputTextState, py::arg("state"));

    m.def("debug_node_typing_select_state",
        ImGui::DebugNodeTypingSelectState, py::arg("state"));

    m.def("debug_node_window",
        ImGui::DebugNodeWindow, py::arg("window"), py::arg("label"));

    m.def("debug_node_window_settings",
        ImGui::DebugNodeWindowSettings, py::arg("settings"));

    m.def("debug_node_windows_list",
        ImGui::DebugNodeWindowsList, py::arg("windows"), py::arg("label"));

    m.def("debug_node_viewport",
        ImGui::DebugNodeViewport, py::arg("viewport"));

    m.def("debug_render_keyboard_preview",
        ImGui::DebugRenderKeyboardPreview, py::arg("draw_list"));

    m.def("debug_render_viewport_thumbnail",
        ImGui::DebugRenderViewportThumbnail, py::arg("draw_list"), py::arg("viewport"), py::arg("bb"));


    auto pyClassImFontBuilderIO =
        py::class_<ImFontBuilderIO>
            (m, "ImFontBuilderIO", "This structure is likely to evolve as we add support for incremental atlas updates")
        .def(py::init<>()) // implicit default constructor
        ;


    m.def("im_font_atlas_update_config_data_pointers",
        ImFontAtlasUpdateConfigDataPointers, py::arg("atlas"));


    py::enum_<ImGuiDockRequestType>(m, "DockRequestType", py::arithmetic(), "")
        .value("none", ImGuiDockRequestType_None, "")
        .value("dock", ImGuiDockRequestType_Dock, "")
        .value("undock", ImGuiDockRequestType_Undock, "")
        .value("split", ImGuiDockRequestType_Split, "Split is the same as Dock but without a DockPayload");


    auto pyClassImGuiDockRequest =
        py::class_<ImGuiDockRequest>
            (m, "DockRequest", "")
        .def_readwrite("type", &ImGuiDockRequest::Type, "")
        .def_readwrite("dock_target_window", &ImGuiDockRequest::DockTargetWindow, "Destination/Target Window to dock into (may be a loose window or a DockNode, might be None in which case DockTargetNode cannot be None)")
        .def_readwrite("dock_target_node", &ImGuiDockRequest::DockTargetNode, "Destination/Target Node to dock into")
        .def_readwrite("dock_payload", &ImGuiDockRequest::DockPayload, "Source/Payload window to dock (may be a loose window or a DockNode), [Optional]")
        .def_readwrite("dock_split_dir", &ImGuiDockRequest::DockSplitDir, "")
        .def_readwrite("dock_split_ratio", &ImGuiDockRequest::DockSplitRatio, "")
        .def_readwrite("dock_split_outer", &ImGuiDockRequest::DockSplitOuter, "")
        .def_readwrite("undock_target_window", &ImGuiDockRequest::UndockTargetWindow, "")
        .def_readwrite("undock_target_node", &ImGuiDockRequest::UndockTargetNode, "")
        .def(py::init<>())
        ;


    auto pyClassImGuiDockPreviewData =
        py::class_<ImGuiDockPreviewData>
            (m, "DockPreviewData", "")
        .def_readwrite("future_node", &ImGuiDockPreviewData::FutureNode, "")
        .def_readwrite("is_drop_allowed", &ImGuiDockPreviewData::IsDropAllowed, "")
        .def_readwrite("is_center_available", &ImGuiDockPreviewData::IsCenterAvailable, "")
        .def_readwrite("is_sides_available", &ImGuiDockPreviewData::IsSidesAvailable, "Hold your breath, grammar freaks..")
        .def_readwrite("is_split_dir_explicit", &ImGuiDockPreviewData::IsSplitDirExplicit, "Set when hovered the drop rect (vs. implicit SplitDir==None when hovered the window)")
        .def_readwrite("split_node", &ImGuiDockPreviewData::SplitNode, "")
        .def_readwrite("split_dir", &ImGuiDockPreviewData::SplitDir, "")
        .def_readwrite("split_ratio", &ImGuiDockPreviewData::SplitRatio, "")
        .def(py::init<>())
        ;


    auto pyClassImGuiDockNodeSettings =
        py::class_<ImGuiDockNodeSettings>
            (m, "DockNodeSettings", "Persistent Settings data, stored contiguously in SettingsNodes (sizeof() ~32 bytes)")
        .def_readwrite("id_", &ImGuiDockNodeSettings::ID, "")
        .def_readwrite("parent_node_id", &ImGuiDockNodeSettings::ParentNodeId, "")
        .def_readwrite("parent_window_id", &ImGuiDockNodeSettings::ParentWindowId, "")
        .def_readwrite("selected_tab_id", &ImGuiDockNodeSettings::SelectedTabId, "")
        .def_readwrite("split_axis", &ImGuiDockNodeSettings::SplitAxis, "")
        .def_readwrite("depth", &ImGuiDockNodeSettings::Depth, "")
        .def_readwrite("flags", &ImGuiDockNodeSettings::Flags, "NB: We save individual flags one by one in ascii format (ImGuiDockNodeFlags_SavedFlagsMask_)")
        .def_readwrite("pos", &ImGuiDockNodeSettings::Pos, "")
        .def_readwrite("size", &ImGuiDockNodeSettings::Size, "")
        .def_readwrite("size_ref", &ImGuiDockNodeSettings::SizeRef, "")
        .def(py::init<>())
        ;
    // #endif

    { // <namespace ImStb>
        py::module_ pyNsImStb = m.def_submodule("im_stb", "");
    } // </namespace ImStb>
    ////////////////////    </generated_from:imgui_internal.h>    ////////////////////


    ////////////////////    <generated_from:imgui_internal_pywrappers.h>    ////////////////////
    m.def("dock_builder_split_node_py",
        [](ImGuiID node_id, ImGuiDir split_dir, float size_ratio_for_node_at_dir) -> std::tuple<ImGuiID, ImGuiID, ImGuiID>
        {
            auto DockBuilderSplitNode_Py_adapt_force_lambda = [](ImGuiID node_id, ImGuiDir split_dir, float size_ratio_for_node_at_dir) -> std::tuple<ImGuiID, ImGuiID, ImGuiID>
            {
                auto lambda_result = ImGui::DockBuilderSplitNode_Py(node_id, split_dir, size_ratio_for_node_at_dir);
                return lambda_result;
            };

            return DockBuilderSplitNode_Py_adapt_force_lambda(node_id, split_dir, size_ratio_for_node_at_dir);
        },
        py::arg("node_id"), py::arg("split_dir"), py::arg("size_ratio_for_node_at_dir"),
        " DockBuilderSplitNode_Py() create 2 child nodes within 1 node. The initial node becomes a parent node.\n This version is an adaptation for the python bindings (the C++ version uses two output parameters for the ID of the child nodes, this version returns a tuple)");
    ////////////////////    </generated_from:imgui_internal_pywrappers.h>    ////////////////////

    // </litgen_pydef> // Autogenerated code end
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  AUTOGENERATED CODE END !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
}
